/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.asynctasks;

import android.os.AsyncTask;


import com.behance.sdk.asynctask.listeners.IBehanceSDKGetCuratedGalleryProjectsAsyncTaskListener;
import com.behance.sdk.asynctask.params.BehanceSDKGetCuratedGalleryProjectsAsyncTaskParams;
import com.behance.sdk.asynctasks.result.BehanceSDKAsyncTaskResultWrapper;
import com.behance.sdk.dto.parser.BehanceSDKProjectDTOParser;
import com.behance.sdk.dto.project.BehanceSDKProjectDTO;
import com.behance.sdk.enums.BehanceSDKCuratedGalleryType;
import com.behance.sdk.exception.BehanceSDKException;
import com.behance.sdk.exception.BehanceSDKHTTPStatusCodeNotOKException;
import com.behance.sdk.exception.BehanceSDKProjectParseException;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.network.BehanceConnectionResponse;
import com.behance.sdk.network.BehanceHttpsConnection;
import com.behance.sdk.network.IBehanceHttpConnection;
import com.behance.sdk.util.BehanceSDKConstants;
import com.behance.sdk.util.BehanceSDKUrlUtil;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class BehanceSDKGetCuratedGalleryProjectsAsyncTask extends
		AsyncTask<BehanceSDKGetCuratedGalleryProjectsAsyncTaskParams, Void, BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKProjectDTO>>> {

	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKGetCuratedGalleryProjectsAsyncTask.class);

	private IBehanceSDKGetCuratedGalleryProjectsAsyncTaskListener taskHandler;

	private BehanceSDKGetCuratedGalleryProjectsAsyncTaskParams taskParams;

	public BehanceSDKGetCuratedGalleryProjectsAsyncTask(IBehanceSDKGetCuratedGalleryProjectsAsyncTaskListener taskHandler) {
		this.taskHandler = taskHandler;
	}

	@Override
	protected BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKProjectDTO>> doInBackground(BehanceSDKGetCuratedGalleryProjectsAsyncTaskParams... params) {
		BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKProjectDTO>> result = new BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKProjectDTO>>();

		taskParams = params[0];
		String collectionId = String.valueOf(taskParams.getCuratedGalleryId());
		BehanceSDKCuratedGalleryType galleryType = taskParams.getGalleryType();

		try {
			List<BehanceSDKProjectDTO> projectsFromServer = getCuratedGalleryProjects(collectionId, taskParams.getPageNumber(), taskParams.getPageSize(),
					galleryType, result);
			result.setResult(projectsFromServer);

		} catch (Exception e) {
			logger.error(e, "Unknown problem fetching Projects of Curated Gallery [Gallery ID - %s]", collectionId);
			result.setExceptionOccurred(true);
			result.setException(e);
		} catch (Throwable t) {
			logger.error(t, "Problem fetching Projects of Curated Gallery");
			BehanceSDKException ex = new BehanceSDKException(t);
			result.setException(ex);
			result.setExceptionOccurred(true);
		}
		return result;
	}

	private List<BehanceSDKProjectDTO> getCuratedGalleryProjects(String curatedGalleryId, int pageNumber, int pageSize,
																 BehanceSDKCuratedGalleryType galleryType,BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKProjectDTO>> result) throws IOException,
			JSONException, BehanceSDKProjectParseException, BehanceSDKHTTPStatusCodeNotOKException {

		List<BehanceSDKProjectDTO> response = new ArrayList<BehanceSDKProjectDTO>();

		try {
			Map<String, String> urlParams = new HashMap<String, String>();
			urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, BehanceSDKConstants.BEHANCE_API_CLIENT_ID);
			urlParams.put(BehanceSDKUrlUtil.KEY_SITE_ID, curatedGalleryId);
			String url = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.GET_CURATED_GALLERIES_PROJECTS_API_URL, urlParams);

			if (galleryType == BehanceSDKCuratedGalleryType.ORGANIZATION) {
				url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_NETWORK_FIELD, curatedGalleryId);
			} else if (galleryType == BehanceSDKCuratedGalleryType.SERVED_SITE) {
				url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_SERVED_FIELD, curatedGalleryId);
			}

			url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_PAGE_NUMBER, pageNumber);
			url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_PAGE_SIZE, pageSize);


			String userAccessToken = taskParams.getUserAccessToken();
			if (userAccessToken != null) {
				url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_ACCESS_TOKEN, userAccessToken);
			}

			logger.debug("Get Projects of Curated Gallery url - %s", url);


			BehanceConnectionResponse<String> connectionResponse = BehanceHttpsConnection.getInstance().invokeHttpGetRequest(url);
			String jsonResponseString = connectionResponse.getResponseObject();

			logger.debug("Get Projects of Curated Gallery response: %s", jsonResponseString);

			JSONObject jsonObject = new JSONObject(jsonResponseString);
			int httpResponseCode = jsonObject.optInt("http_code");

			if (httpResponseCode == IBehanceHttpConnection.SC_OK) {
				BehanceSDKProjectDTOParser parser = new BehanceSDKProjectDTOParser();
				List<BehanceSDKProjectDTO> projectsList = parser.parseJustProjects(jsonResponseString);
				if (projectsList != null) {
					response.addAll(projectsList);
				}
			} else if (httpResponseCode == IBehanceHttpConnection.SC_NOT_FOUND) {
				logger.error("HTTP Response code 404 when trying to  fetch Projects of Curated Gallery. [Gallery ID - %s]", curatedGalleryId);
				result.setException(new BehanceSDKException("Curated Gallery not found"));
				result.setExceptionOccurred(true);
			} else {
				logger.error("Unexpected HTTP Response code when trying to fetch Projects of Curated Gallery. [Gallery ID - %s] [Response code - %d]",
						curatedGalleryId, httpResponseCode);
				result.setException(new BehanceSDKException("Invalid server response code "
						+ httpResponseCode));
				result.setExceptionOccurred(true);
			}
		}catch (Exception e){
			logger.error("Unexpected HTTP Response code when trying to fetch Projects of Curated Gallery");
			result.setException(new BehanceSDKException(e.getMessage()));
			result.setExceptionOccurred(true);
		}
		return response;
	}

	@Override
	protected void onPostExecute(BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKProjectDTO>> resultWrapper) {
		if (resultWrapper.isExceptionOccurred()) {
			taskHandler.onGetCuratedGalleryProjectsFailure(resultWrapper.getException(), taskParams);
		} else {
			taskHandler.onGetCuratedGalleryProjectsSuccess(resultWrapper.getResult(), taskParams);
		}
	}
}