/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.asynctasks;

import android.os.AsyncTask;


import com.behance.sdk.asynctask.listeners.IBehanceSDKGetProjectCommentsAsyncTaskListener;
import com.behance.sdk.asynctask.params.BehanceSDKGetProjectCommentsAsyncTaskParams;
import com.behance.sdk.asynctasks.result.BehanceSDKAsyncTaskResultWrapper;
import com.behance.sdk.dto.project.BehanceSDKProjectCommentDTO;
import com.behance.sdk.dto.search.BehanceSDKUserDTO;
import com.behance.sdk.exception.BehanceSDKException;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.network.BehanceConnectionResponse;
import com.behance.sdk.network.BehanceHttpsConnection;
import com.behance.sdk.util.BehanceSDKConstants;
import com.behance.sdk.util.BehanceSDKUrlUtil;

import org.json.JSONArray;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class BehanceSDKGetProjectCommentsAsyncTask extends AsyncTask<BehanceSDKGetProjectCommentsAsyncTaskParams, Void, BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKProjectCommentDTO>>> {

	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKGetProjectCommentsAsyncTask.class);

	private IBehanceSDKGetProjectCommentsAsyncTaskListener taskHandler;

	private BehanceSDKGetProjectCommentsAsyncTaskParams taskParams;

	public BehanceSDKGetProjectCommentsAsyncTask(IBehanceSDKGetProjectCommentsAsyncTaskListener taskHandler) {
		this.taskHandler = taskHandler;
	}

	@Override
	protected BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKProjectCommentDTO>> doInBackground(BehanceSDKGetProjectCommentsAsyncTaskParams... params) {

		BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKProjectCommentDTO>> result = new BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKProjectCommentDTO>>();

		if (params.length != 1) {
			result.setException(new BehanceSDKException("Project Id is required"));
			result.setExceptionOccurred(true);
			return result;
		}

		try {
			taskParams = params[0];
			String projectId = taskParams.getProjectId();

			Map<String, String> urlParams = new HashMap<String, String>();
			urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, BehanceSDKConstants.BEHANCE_API_CLIENT_ID);
			urlParams.put(BehanceSDKUrlUtil.KEY_PROJECT_ID, projectId);
			String url = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.GET_PROJECT_COMMENTS_API_URL, urlParams);

			String userAccessToken = taskParams.getUserAccessToken();
			if (userAccessToken != null) {
				url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_ACCESS_TOKEN, userAccessToken);
			}

			if (taskParams.getPageNumber() > 0) {
				url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_PAGE_NUMBER, taskParams.getPageNumber());
				url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_PAGE_SIZE, taskParams.getPageSize());
			}

			logger.debug("Get project comments url - %s", url);
			BehanceConnectionResponse<String> response = BehanceHttpsConnection.getInstance().invokeHttpGetRequest(url);
			String jsonResponseString = response.getResponseObject();
			logger.debug("Get project comments response: %s", jsonResponseString);
			JSONObject jsonObject = new JSONObject(jsonResponseString);
			JSONArray commentsArray = jsonObject.getJSONArray("comments");
			List<BehanceSDKProjectCommentDTO> commentsList = new ArrayList<BehanceSDKProjectCommentDTO>();
			if (commentsArray != null && commentsArray.length() > 0) {
				for (int i = 0; i < commentsArray.length(); i++) {
					JSONObject comment = commentsArray.getJSONObject(i);

					BehanceSDKProjectCommentDTO commentDTO = new BehanceSDKProjectCommentDTO();

					commentDTO.setId(comment.getString("id"));
					commentDTO.setContent(comment.getString("comment"));
					commentDTO.setCreatedDate(comment.getLong("created_on"));

					// commenter details
					JSONObject commenter = comment.getJSONObject("user");
					BehanceSDKUserDTO userDTO = new BehanceSDKUserDTO();
					userDTO.setId(commenter.getInt("id"));
					userDTO.setFirstName(commenter.getString("first_name"));
					userDTO.setLastName(commenter.getString("last_name"));

					// get user profile images
					JSONObject imagesJSONObject = commenter.optJSONObject("images");
					if (null != imagesJSONObject) {
						userDTO.addImage(BehanceSDKUserDTO.PROFILE_IMAGE_SIZE_50, imagesJSONObject.optString("50"));
						userDTO.addImage(BehanceSDKUserDTO.PROFILE_IMAGE_SIZE_115, imagesJSONObject.optString("115"));
						userDTO.addImage(BehanceSDKUserDTO.PROFILE_IMAGE_SIZE_138, imagesJSONObject.optString("138"));
					}

					commentDTO.setUser(userDTO);

					commentsList.add(commentDTO);
				}
			}
			result.setResult(commentsList);
		} catch (Exception e) {
			logger.error(e, "Problem getting Project comments from server");
			result.setExceptionOccurred(true);
			result.setException(new BehanceSDKException(e));
		} catch (Throwable t) {
			logger.error(t, "Problem getting Project comments from server");
			BehanceSDKException ex = new BehanceSDKException(t);
			result.setException(ex);
			result.setExceptionOccurred(true);
		}
		return result;
	}

	@Override
	protected void onPostExecute(BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKProjectCommentDTO>> resultWrapper) {
		if (resultWrapper.isExceptionOccurred()) {
			taskHandler.onGetProjectCommentsFailure(resultWrapper.getException(), taskParams);
		} else {
			taskHandler.onGetProjectCommentsSuccess(resultWrapper.getResult(), taskParams);
		}
	}
}
