/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.asynctasks;

import android.os.AsyncTask;
import android.os.Build;


import com.behance.sdk.asynctask.listeners.IBehanceSDKGetProjectDetailsAsyncTaskListener;
import com.behance.sdk.asynctask.params.BehanceSDKGetProjectDetailsAsyncTaskParams;
import com.behance.sdk.asynctasks.result.BehanceSDKAsyncTaskResultWrapper;
import com.behance.sdk.dto.parser.BehanceSDKProjectDTOParser;
import com.behance.sdk.dto.project.BehanceSDKAbstractProjectModuleDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectModuleAudioDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectModuleEmbedDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectModuleImageDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectModuleVideoDTO;
import com.behance.sdk.enums.BehanceSDKProjectModuleType;
import com.behance.sdk.exception.BehanceSDKException;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.network.BehanceConnectionResponse;
import com.behance.sdk.network.BehanceHttpsConnection;
import com.behance.sdk.network.IBehanceHttpConnection;
import com.behance.sdk.util.BehanceSDKConstants;
import com.behance.sdk.util.BehanceSDKFileUtils;
import com.behance.sdk.util.BehanceSDKUrlUtil;

import org.json.JSONObject;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

/**
 * @author Karthik Ramesh
 * 
 */

public class BehanceSDKGetProjectDetailsAsyncTask extends AsyncTask<BehanceSDKGetProjectDetailsAsyncTaskParams, Void, BehanceSDKAsyncTaskResultWrapper<BehanceSDKProjectDTO>> {

	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKGetProjectDetailsAsyncTask.class);

	private IBehanceSDKGetProjectDetailsAsyncTaskListener taskHandler;
	private BehanceSDKAsyncTaskResultWrapper<BehanceSDKProjectDTO> result;
	private BehanceSDKGetProjectDetailsAsyncTaskParams taskParams;

	public BehanceSDKGetProjectDetailsAsyncTask(IBehanceSDKGetProjectDetailsAsyncTaskListener taskHandler) {
		this.taskHandler = taskHandler;
	}

	@Override
	protected BehanceSDKAsyncTaskResultWrapper<BehanceSDKProjectDTO> doInBackground(BehanceSDKGetProjectDetailsAsyncTaskParams... params) {
		this.result = new BehanceSDKAsyncTaskResultWrapper<BehanceSDKProjectDTO>();

		try {
			if (params.length != 1) {
				result.setException(new BehanceSDKException("Params are required"));
				result.setExceptionOccurred(true);
				return result;
			}

			taskParams = params[0];

			// we are sending a request to record a view for Project in parallel
			// to getting project details to save on time
			ExecutorService executorService = Executors.newFixedThreadPool(1);
			RecordProjectViewCallable recordProjectViewCallable = new RecordProjectViewCallable();
			Future<BehanceSDKProjectDTO> recordProjectViewFuture = executorService.submit(recordProjectViewCallable);

			BehanceSDKProjectDTO projectDTO = getProjectDetails(taskParams);
			if (result != null && result.isExceptionOccurred()) {
				return result;
			}

			if(projectDTO != null) {
				fixAndSaveHTML(projectDTO);
				if (result != null && result.isExceptionOccurred()) {
					return result;
				}
				BehanceSDKProjectDTO recordViewResponse = recordProjectViewFuture.get();
				projectDTO.setAppreciatedByUser(recordViewResponse.isAppreciatedByUser());
				projectDTO.setAppreciatedByUserOn(recordViewResponse.getAppreciatedByUserOn());
			}

			result.setResult(projectDTO);
		} catch (Exception e) {
			logger.error(e, "Problem getting Project details from server");
			result.setExceptionOccurred(true);
			result.setException(e);
		} catch (Throwable t) {
			logger.error(t, "Problem getting Project details from server");
			BehanceSDKException ex = new BehanceSDKException(t);
			result.setException(ex);
			result.setExceptionOccurred(true);
		}
		return result;
	}

	private BehanceSDKProjectDTO getProjectDetails(BehanceSDKGetProjectDetailsAsyncTaskParams taskParams) {
		try {

			Map<String, String> urlParams = new HashMap<String, String>();
			urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, BehanceSDKConstants.BEHANCE_API_CLIENT_ID);
			urlParams.put(BehanceSDKUrlUtil.KEY_PROJECT_ID, taskParams.getProjectID());

			String getProjectDetailsUrl = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.GET_PROJECT_API_URL, urlParams);

			String userAccessToken = taskParams.getUserAccessToken();
			if (userAccessToken != null) {
				getProjectDetailsUrl = BehanceSDKUrlUtil.appendQueryStringParam(getProjectDetailsUrl, BehanceSDKUrlUtil.PARAM_KEY_ACCESS_TOKEN, userAccessToken);
			}

			logger.debug("Get project details url - %s", getProjectDetailsUrl);

            BehanceConnectionResponse<String> response = BehanceHttpsConnection.getInstance().invokeHttpGetRequest(getProjectDetailsUrl);
            String jsonResponseString = response.getResponseObject();
			logger.debug("Get project details response: %s", jsonResponseString);
			JSONObject jsonObject = new JSONObject(jsonResponseString);
			int httpResponseCode = jsonObject.getInt("http_code");

			if (httpResponseCode == IBehanceHttpConnection.SC_OK) {
				BehanceSDKProjectDTOParser parser = new BehanceSDKProjectDTOParser();
				JSONObject projectJsonObject = jsonObject.optJSONObject("project");
				return parser.parseProjectWithDetails(projectJsonObject);
			} else if (httpResponseCode == IBehanceHttpConnection.SC_NOT_FOUND) {
				result.setException(new BehanceSDKException("Project not found"));
				result.setExceptionOccurred(true);
			} else {
				result.setException(new BehanceSDKException("Invalid server response code " + httpResponseCode));
				result.setExceptionOccurred(true);
			}

		} catch (Exception e) {
			logger.error(e, "Problem getting Project details from server");
			result.setExceptionOccurred(true);
			result.setException(e);
		} catch (Throwable t) {
			logger.error(t, "Problem getting Project details from server");
			BehanceSDKException ex = new BehanceSDKException(t);
			result.setException(ex);
			result.setExceptionOccurred(true);
		}
		return null;
	}

	private void fixAndSaveHTML(BehanceSDKProjectDTO projectDTO) {
		String fixedHtml = fixHTMLContent(projectDTO);
		if (fixedHtml != null && fixedHtml.length() > 0) {
			saveProjectHTML(fixedHtml, projectDTO);
		} else {
			result.setExceptionOccurred(true);
			result.setException(new BehanceSDKException("Unable to fix the HTML content"));
		}
	}

	private String fixHTMLContent(BehanceSDKProjectDTO projectDTO) {
		List<BehanceSDKAbstractProjectModuleDTO> modules = projectDTO.getModules();
		String projectHTML = projectDTO.getHTML();
		int imageNumber = 0;
		for (BehanceSDKAbstractProjectModuleDTO abstractProjectModuleDTO : modules) {
			if (BehanceSDKProjectModuleType.IMAGE.equals(abstractProjectModuleDTO.getType())) {
				projectHTML = fixImageModuleLayout(projectDTO.getId(), projectHTML, (BehanceSDKProjectModuleImageDTO) abstractProjectModuleDTO, imageNumber++);
			} else if (BehanceSDKProjectModuleType.EMBED == abstractProjectModuleDTO.getType()) {
				projectHTML = fixEmbedModuleLayout(projectHTML, (BehanceSDKProjectModuleEmbedDTO) abstractProjectModuleDTO);
			} else if (BehanceSDKProjectModuleType.VIDEO == abstractProjectModuleDTO.getType()) {
				BehanceSDKProjectModuleVideoDTO videoModule = (BehanceSDKProjectModuleVideoDTO) abstractProjectModuleDTO;
				projectHTML = fixVideoModuleLayout(projectDTO.getId(), projectHTML, videoModule);
			} else if (BehanceSDKProjectModuleType.AUDIO == abstractProjectModuleDTO.getType()) {
				projectHTML = fixAudioModuleLayout(projectHTML, (BehanceSDKProjectModuleAudioDTO) abstractProjectModuleDTO);
			}
		}
		return projectHTML;
	}

	private String fixEmbedModuleLayout(String projectHTML, BehanceSDKProjectModuleEmbedDTO embedModule) {
		String response;
		String embedSrc = "";
		String embedHTML = embedModule.getEmbedHTML();
		int indexOfSrc = embedHTML.indexOf("src=\"");
		if (indexOfSrc >= 0) {
			indexOfSrc = indexOfSrc + 5;
			int indexOfLastQuote = embedHTML.indexOf("\"", indexOfSrc);
			if (indexOfLastQuote > 0) {
				embedSrc = embedHTML.substring(indexOfSrc, indexOfLastQuote);
			}
		}
		//URL must start with http or webview looks locally
		if(!embedSrc.contains("https://")&&!embedSrc.contains("http://")) {
			String embedSrcFixed = "https://" + embedSrc;
			projectHTML = projectHTML.replace(embedSrc, embedSrcFixed);
			embedSrc = embedSrcFixed;
		}

		if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN) {
			response = projectHTML.replaceFirst(
					"(?s)(<li )(.*)(module-" + embedModule.getId() + ")([^>])(>)(.*?)(<iframe )([^>]*)(></iframe>)",
					"$1$2$3$4$5$6" + "<div class=\"embedModuleWrapperClass\" style=\"position: relative;\">" + "$7$8$9" + "</div>");
		} else {
			response = projectHTML.replaceFirst(
				"(?s)(<li )(.*)(module-" + embedModule.getId() + ")([^>])(>)(.*?)(<iframe )([^>]*)(></iframe>)",
				"$1$2$3$4$5$6" + "<div class=\"embedModuleWrapperClass\" style=\"position: relative;\">"
						+ "<div style=\"position: absolute; top: 0; left: 0; width:100%; height:100%\" onclick=\"alert('"
						+ BehanceSDKProjectModuleType.EMBED.name() + embedSrc + "'); return false\"></div>" + "$7$8$9" + "</div>");
		}
		return response;
	}

	private String fixAudioModuleLayout(String projectHTML, BehanceSDKProjectModuleAudioDTO audioModule) {
		String response = projectHTML;

		response = projectHTML.replaceFirst(
				"(?s)(<li )(.*)(module-" + audioModule.getId() + ")([^>])(>)(.*?)(<object )(.*?)(</object>)",
				"$1$2$3$4$5" + "<div class=\"audioModuleWrapperClass\" align=\"center\">"
						+ "<audio class=\"audioModuleWrapperClass\" style=\"width:80%;\" controls=\"controls\">" + "<source src=\""
						+ audioModule.getSrcURL() + "\" type=\"audio/mpeg\">" + "Problem loading audio</audio>" + "</div>");
		return response;
	}

	private String fixVideoModuleLayout(String projectId, String projectHTML, BehanceSDKProjectModuleVideoDTO videoModule) {
		String response = projectHTML;
		String embedHTML = videoModule.getEmbedHTML();

		String widthStr = "";
		int indexOfWidth = embedHTML.indexOf("width=\"");
		if (indexOfWidth >= 0) {
			indexOfWidth = indexOfWidth + 7;
			int indexOfWidthLastQuote = embedHTML.indexOf("\"", indexOfWidth);
			if (indexOfWidthLastQuote > 0) {
				widthStr = embedHTML.substring(indexOfWidth, indexOfWidthLastQuote);
			}
		}

		String heightStr = "";
		int indexOfHeight = embedHTML.indexOf("height=\"");
		if (indexOfHeight >= 0) {
			indexOfHeight = indexOfHeight + 8;
			int indexOfHeightLastQuote = embedHTML.indexOf("\"", indexOfHeight);
			if (indexOfHeightLastQuote > 0) {
				heightStr = embedHTML.substring(indexOfHeight, indexOfHeightLastQuote);
			}
		}

		String videoPosterImageSrc = videoModule.getImageSrc();
		response = projectHTML.replaceFirst(
				"(?s)(<li )(.*)(module-" + videoModule.getId() + ")([^>])(>)(.*?)(<object )(.*?)(</object>)",
				"$1$2$3$4$5" + "<a class=\"videoModuleWrapperClass\" href=\"#\" onClick=\"alert('" + BehanceSDKProjectModuleType.VIDEO.name()
						+ videoModule.getVideoSrc() + "'); return false\">" + "<div style=\"position: relative\"><img style=\"position: relative;\" "
						+ "width=\"" + widthStr + "\" height=\"" + heightStr + "\" src=\"" + videoPosterImageSrc
						+ "\"/><img style=\"position: absolute; top: 39%; left: 43%; z-index: 1;\" "
						+ "src='file:///android_asset/flash_play_button.png'/></div></a>");
		return response;
	}

	private String fixImageModuleLayout(String projectId, String projectHTML, BehanceSDKProjectModuleImageDTO imageModule, int imageNumber) {
		String srcUrl;
		if (taskParams.isDownloadHD())
			srcUrl = imageModule.getHDSrcUrl();
		else
			srcUrl = imageModule.getSrcUrl();
		//Set image left margin to -53px for full bleed
		String response = projectHTML.replaceFirst(
				"(?s)(<li )(.*?)(module-" + imageModule.getId() + ")([^>])(>)(<img )(.*?)(src=\\\"[^\\\"]*\")([^>]*?)(>)",
				"$1$2$3$4$5<a href=\"#\" onClick=\"alert('" + BehanceSDKProjectModuleType.IMAGE.name() + imageNumber + "'); return false\">" + "<img src=\""
						+ srcUrl + "\" " + (imageModule.isFullBleed() ? "style=\"float:none;min-width:725px;height:auto;margin-left:-53px;margin-right:0px\" $7" : "$7$9") + "></a>");
		return response;
	}

	private String createAppreciateDivInHTML() {
		String appreciateHTML = "";
		appreciateHTML += "<div id=\"appreciation\" style=\"height: 180px; width: 100%; text-align:center; padding-bottom:52px; padding-top:38px; clear:both;\"/>";
		return appreciateHTML;
	}

	private void saveProjectHTML(String projectHTML, BehanceSDKProjectDTO activeProject) {
		File htmlRootFolder = null;
		File htmlFile = new File(htmlRootFolder, activeProject.getLayoutHTMLPath());
		boolean projectFolderCreationStatus = false;
		try {
			String rootDir = BehanceSDKFileUtils.getAppStoragePath();
			logger.debug("Trying to create temp folder. [Project id - %s]", activeProject.getId());
			htmlRootFolder = new File(rootDir);
			if (!htmlRootFolder.exists()) {
				logger.debug("Creating HTML Root folder folder. [Folder Path is - %s]", htmlRootFolder.getAbsolutePath());
				projectFolderCreationStatus = htmlRootFolder.mkdirs();
				if (!projectFolderCreationStatus) {
					logger.error("Error Trying to create temp folder. [Path is - %s]", htmlRootFolder.getAbsolutePath());
					result.setExceptionOccurred(true);
					result.setException(new BehanceSDKException("Could not create temp folder"));
					return;
				}
			}

			if (null != projectHTML) {
				String content = "<html><head>"
						+ "<meta name=\"viewport\" content=\"width=100%\" />"
						+ "<link rel=\"stylesheet\" type=\"text/css\" href=\"file:///android_asset/final.css\"/>"
						+ "<style type=\"text/css\">"
						+ activeProject.getCustomCSS()
						+ "</style>"
						+ "<style type=\"text/css\">* {\n-webkit-tap-highlight-color: rgba(0, 92, 255, .4);\n-webkit-tap-highlight-color: #66005cff;\n}</style>"
						+ "<script type=\"text/javascript\" src=\"file:///android_asset/combined.js\"></script>"
						+ "<script type=\"text/javascript\">function pauseAllActiveAudio() { $('audio').each(function() {if(!this.paused){this.pause();}}); }</script>"
						+ "</head><body class=\"no-padding project-view project-styles\"><div id=\"site-container\"><div id=\"site-columns\" class=\"grid-wrap cfix\"><div id=\"site-left-column\"><div id=\"primary-content\" class=\"ui-corner-all no-level-2-nav cfix\"><div id=\"primary-project-content\" class=\"project-styles\">"
						+ projectHTML
						+ createAppreciateDivInHTML()
						+ "</div></div></div></div></div><script type=\"text/javascript\">$(document).ready(function() { if (-1 != window.location.href.indexOf('?connection=false')) { $('.embedModuleWrapperClass').css('visibility', 'hidden'); $('.videoModuleWrapperClass').css('visibility', 'hidden'); $('.audioModuleWrapperClass').css('visibility', 'hidden'); } });</script></body></html>";// <script
				byte[] contInBytes = content.getBytes();

				if (htmlFile.exists())
					htmlFile.delete();
				FileOutputStream fos = new FileOutputStream(htmlFile);
				fos.write(contInBytes);
				fos.flush();
				fos.close();
			}
		} catch (FileNotFoundException e) {
			result.setExceptionOccurred(true);
			result.setException(new BehanceSDKException("HTML File not found"));
			logger.error(e, "Problem saving project HTML. [project id - %s] [temp dir - %s]", activeProject.getId(), htmlRootFolder.getAbsolutePath());
		} catch (IOException e) {
			result.setExceptionOccurred(true);
			result.setException(new BehanceSDKException("Problem saving project HTML"));
			logger.error(e, "IOProblem saving project HTML. [project id - %s] [temp dir - %s]", activeProject.getId(),
					htmlRootFolder.getAbsolutePath());
		} catch (Exception e) {
			result.setExceptionOccurred(true);
			result.setException(new BehanceSDKException( "Problem saving project HTML"));
			logger.error(e, "Unkown problem saving project HTML. [project id - %s] [temp dir - %s]", activeProject.getId(),
					(htmlRootFolder != null ? htmlRootFolder.getAbsolutePath() : "null"));
		}
	}

	@Override
	protected void onPostExecute(BehanceSDKAsyncTaskResultWrapper<BehanceSDKProjectDTO> resultWrapper) {
		if (resultWrapper.isExceptionOccurred()) {
			taskHandler.onGetProjectDetailsAsyncTaskFailure(resultWrapper.getException());
		} else {
			taskHandler.onGetProjectDetailsAsyncTaskSuccess(resultWrapper.getResult());
		}
	}

	private class RecordProjectViewCallable implements Callable<BehanceSDKProjectDTO> {
		private RecordProjectViewCallable() {
		}

		@Override
		public BehanceSDKProjectDTO call() throws Exception {
			return recordAViewForProject();
		}

		private BehanceSDKProjectDTO recordAViewForProject() {
			BehanceSDKProjectDTO projectDTO = new BehanceSDKProjectDTO();
			try {
				Map<String, String> urlParams = new HashMap<String, String>();
				urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, BehanceSDKConstants.BEHANCE_API_CLIENT_ID);
				String projectID = taskParams.getProjectID();
				urlParams.put(BehanceSDKUrlUtil.KEY_PROJECT_ID, projectID);

				String recordViewForProjectUrl = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.VIEW_PROJECT_API_URL, urlParams);

				String userAccessToken = taskParams.getUserAccessToken();
				if (userAccessToken != null) {
					recordViewForProjectUrl = BehanceSDKUrlUtil
							.appendQueryStringParam(recordViewForProjectUrl, BehanceSDKUrlUtil.PARAM_KEY_ACCESS_TOKEN, userAccessToken);
				}

				BehanceConnectionResponse<String> response = BehanceHttpsConnection.getInstance().invokeHttpPostRequest(recordViewForProjectUrl);
				String jsonResponseString = response.getResponseObject();
				logger.debug("Record a view for Project response for [project id - %s ] : %s", projectID, jsonResponseString);
				int httpResponseCode = response.getResponseCode();
				if (httpResponseCode == IBehanceHttpConnection.SC_OK) {
					JSONObject jsonObject = new JSONObject(jsonResponseString);
					httpResponseCode = jsonObject.getInt("http_code");
					if (httpResponseCode == IBehanceHttpConnection.SC_OK) {
						projectDTO.setAppreciatedByUser(jsonObject.optBoolean("appreciated", false));
						projectDTO.setAppreciatedByUserOn(jsonObject.optLong("appreciated_on"));
					} else {
						projectDTO.setAppreciatedByUser(false);
					}
					return projectDTO;
				}
				logger.debug("Record a view for Project http response status code - %s", httpResponseCode);
			} catch (Exception e) {
				projectDTO.setAppreciatedByUser(false);
				logger.error(e, "Problem recording view for Project");
			} catch (Throwable t) {
				projectDTO.setAppreciatedByUser(false);
				logger.error(t, "Problem recording view for Project");
			}
			return projectDTO;
		}
	}
}
