/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.asynctasks;

/**
 * @author Sujit Reddy G
 *
 */

import android.os.AsyncTask;
import android.text.TextUtils;


import com.behance.sdk.asynctask.listeners.IBehanceSDKGetProjectsAsyncTaskListener;
import com.behance.sdk.asynctask.params.BehanceSDKGetProjectsAsyncTaskParams;
import com.behance.sdk.asynctasks.result.BehanceSDKAsyncTaskResultWrapper;
import com.behance.sdk.dto.location.BehanceSDKCityDTO;
import com.behance.sdk.dto.location.BehanceSDKStateDTO;
import com.behance.sdk.dto.parser.BehanceSDKProjectDTOParser;
import com.behance.sdk.dto.project.BehanceSDKProjectDTO;
import com.behance.sdk.enums.BehanceSDKProjectsSortOption;
import com.behance.sdk.enums.BehanceSDKProjectsTimeSpan;
import com.behance.sdk.exception.BehanceSDKException;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.network.BehanceConnectionResponse;
import com.behance.sdk.network.BehanceHttpsConnection;
import com.behance.sdk.util.BehanceSDKConstants;
import com.behance.sdk.util.BehanceSDKUrlUtil;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class BehanceSDKGetProjectsAsyncTask extends AsyncTask<BehanceSDKGetProjectsAsyncTaskParams, Void, BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKProjectDTO>>> {

	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKGetProjectsAsyncTask.class);

	private IBehanceSDKGetProjectsAsyncTaskListener taskHandler;
	private BehanceSDKGetProjectsAsyncTaskParams taskParams;

	public BehanceSDKGetProjectsAsyncTask(IBehanceSDKGetProjectsAsyncTaskListener taskHandler) {
		this.taskHandler = taskHandler;
	}

	@Override
	protected BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKProjectDTO>> doInBackground(BehanceSDKGetProjectsAsyncTaskParams... params) {

		BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKProjectDTO>> result = new BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKProjectDTO>>();

		try {

			Map<String, String> urlParams = new HashMap<String, String>();
			urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, BehanceSDKConstants.BEHANCE_API_CLIENT_ID);
			String url = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.GET_PROJECTS_API_URL, urlParams);

			if (params.length == 1) {
				BehanceSDKGetProjectsAsyncTaskParams projectParams = params[0];
				this.taskParams = projectParams;
				if (projectParams != null) {
					url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_PAGE_NUMBER, projectParams.getPageNumber());
					url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_PAGE_SIZE, projectParams.getPageSize());
					// set sort option
					BehanceSDKProjectsSortOption sortOption = projectParams.getSortOption();
					if(null != sortOption)
						url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_SORT, sortOption.toString());

					// set time span
					BehanceSDKProjectsTimeSpan timeSpan = projectParams.getTimeSpan();
					if (null != timeSpan) {
						url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_TIME, timeSpan.toString());
					}

					// set creative field
					String field = projectParams.getField();
					if (!TextUtils.isEmpty(field))
						url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_FIELD, field);

					// set country
					String country = projectParams.getCountry();
					if (!TextUtils.isEmpty(country))
						url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_COUNTRY, country);

					// set state
					String state = projectParams.getState();
					if (!TextUtils.isEmpty(state) && !state.equals(BehanceSDKStateDTO.ALL_STATES_NAME))
						url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_STATE, state);
					// set city
					String city = projectParams.getCity();
					if (!TextUtils.isEmpty(city) && !city.equals(BehanceSDKCityDTO.ALL_CITY_NAME))
						url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_CITY, city);

					// set search string
					String searchString = projectParams.getSearchString();
					if (!TextUtils.isEmpty(searchString))
						url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_SEARCH_FREE_TEXT_QUERY, searchString);

					// set access token
					String userAccessToken = projectParams.getUserAccessToken();
					if (userAccessToken != null) {
						url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_ACCESS_TOKEN, userAccessToken);
					}
				}
			}

			logger.debug("Get Projects async task request url - %s", url);

			BehanceConnectionResponse<String> response = BehanceHttpsConnection.getInstance().invokeHttpGetRequest(url);
			String jsonResponseString = response.getResponseObject();

			logger.debug("Get projects async task response: %s", jsonResponseString);
			BehanceSDKProjectDTOParser parser = new BehanceSDKProjectDTOParser();
			List<BehanceSDKProjectDTO> projectsList = parser.parseJustProjects(jsonResponseString);
			result.setResult(projectsList);
		} catch (Exception e) {
			logger.error(e, "Problem getting Projects from server");
			result.setExceptionOccurred(true);
			result.setException(e);
		} catch (Throwable t) {
			logger.error(t, "Problem getting Projects from server");
			BehanceSDKException ex = new BehanceSDKException(t);
			result.setException(ex);
			result.setExceptionOccurred(true);
		}
		return result;
	}

	@Override
	protected void onPostExecute(BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKProjectDTO>> resultWrapper) {
		if (resultWrapper.isExceptionOccurred()) {
			taskHandler.onGetProjectsAsyncTaskFailure(resultWrapper.getException(), taskParams);
		} else {
			taskHandler.onGetProjectsAsyncTaskSuccess(resultWrapper.getResult(), taskParams);
		}
	}
}
