package com.behance.sdk.asynctasks;

import android.os.AsyncTask;
import android.text.TextUtils;

import com.behance.sdk.BehanceSDKUserProfile;
import com.behance.sdk.asynctask.listeners.IBehanceSDKGetUserProfileAsyncTaskListener;
import com.behance.sdk.asynctask.params.BehanceSDKGetUserProfileTaskParams;
import com.behance.sdk.asynctasks.result.BehanceSDKGetUserProfileTaskResult;
import com.behance.sdk.dto.location.BehanceSDKCityDTO;
import com.behance.sdk.dto.location.BehanceSDKCountryDTO;
import com.behance.sdk.dto.location.BehanceSDKStateDTO;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.managers.BehanceSDKUserManager;
import com.behance.sdk.network.BehanceConnectionResponse;
import com.behance.sdk.network.BehanceHttpsConnection;
import com.behance.sdk.util.BehanceSDKUrlUtil;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * Created by forta on 9/15/14.
 */
public class BehanceSDKGetUserProfileAsyncTask extends AsyncTask<BehanceSDKGetUserProfileTaskParams, Void, BehanceSDKGetUserProfileTaskResult>{

    private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKGetUserProfileAsyncTask.class);

    private BehanceSDKGetUserProfileTaskParams getUserProfileTaskParams;

    private IBehanceSDKGetUserProfileAsyncTaskListener listener;

    public BehanceSDKGetUserProfileAsyncTask(IBehanceSDKGetUserProfileAsyncTaskListener listener) {
        this.listener = listener;
    }

    @Override
    protected BehanceSDKGetUserProfileTaskResult doInBackground(BehanceSDKGetUserProfileTaskParams... params) {
        BehanceSDKGetUserProfileTaskResult result = new BehanceSDKGetUserProfileTaskResult();
        if (params.length != 1) {
            result.setExceptionOccurred(true);
            return result;
        }

        getUserProfileTaskParams = params[0];
        String userId = String.valueOf(getUserProfileTaskParams.getUserId());

        try {
            String clientID = getUserProfileTaskParams.getClientId();
            Map<String, String> urlParams = new HashMap<String, String>();
            urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, clientID );
            urlParams.put(BehanceSDKUrlUtil.KEY_USER_ID, userId);
            String url = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.GET_USER_BASIC_DETAILS_API_URL, urlParams);

            BehanceSDKUserManager sdkUserManager = BehanceSDKUserManager.getInstance();
            String userAccessToken = sdkUserManager.checkExpiryAndGetAccessToken();
            if (userAccessToken != null) {
                url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_ACCESS_TOKEN, userAccessToken);
            }

            logger.debug("Get user details URL: %s", url);
            BehanceConnectionResponse<String> httpResponse = BehanceHttpsConnection.getInstance().invokeHttpGetRequest(url);

            String jsonResponseString = httpResponse.getResponseObject();
            logger.debug("Get user details response: %s", jsonResponseString);
            JSONObject jsonObject = new JSONObject(jsonResponseString);
            int httpResponseCode = jsonObject.getInt("http_code");

            if (httpResponseCode == BehanceHttpsConnection.SC_OK) {
                JSONObject rootJsonObject = new JSONObject(jsonResponseString);
                JSONObject userJsonObject = rootJsonObject.optJSONObject("user");
                if (userJsonObject != null) {
                    BehanceSDKUserProfile userProfile = new BehanceSDKUserProfile();
                    userProfile.setFirstName(userJsonObject.optString("first_name"));
                    userProfile.setLastName(userJsonObject.optString("last_name"));
                    userProfile.setCompany(userJsonObject.optString("company"));
                    userProfile.setOccupation(userJsonObject.optString("occupation"));
                    userProfile.setWebsite(userJsonObject.optString("website"));

                    BehanceSDKCountryDTO countryDTO = getCountryDTO(userJsonObject.optString("country"), clientID );
                    userProfile.setCountry(countryDTO);
                    String stateID = null;
                    if (countryDTO != null) {

                        if(BehanceSDKCountryDTO.COUNTRY_CODES_FOR_LOAD_STATES.contains(countryDTO.getId())) {
                            BehanceSDKStateDTO stateDTO = getStateDTO(userJsonObject.optString("state"), countryDTO.getId(), clientID);
                            userProfile.setState(stateDTO);
                            if (null != stateDTO)
                                stateID = stateDTO.getId();
                        }

                        BehanceSDKCityDTO cityDTO = getCityDTO(userJsonObject.optString("city"), countryDTO.getId(), stateID, clientID);
                        userProfile.setCity(cityDTO);
                    }

                    userProfile.setProfileImageUrl(getProfileImage(userJsonObject.optJSONObject("images")));
                    result.setBehanceUserProfile(userProfile);
                }else {
                    logger.error("Unexpected HTTP Response code when trying to fetch User details. [User ID - %s] [Response code - %d]", userId,
                            httpResponseCode);
                    result.setException(new Exception("Invalid server response code " + httpResponseCode));
                    result.setExceptionOccurred(true);
                }
            } else if (httpResponseCode == BehanceHttpsConnection.SC_NOT_FOUND) {
                logger.error("HTTP Response code 404 when trying to fetch User details. [User ID - %s]", userId);
                result.setException(new Exception("User not found"));
                result.setExceptionOccurred(true);
            } else {
                logger.error("Unexpected HTTP Response code when trying to fetch User details. [User ID - %s] [Response code - %d]", userId,
                        httpResponseCode);
                result.setException(new Exception("Invalid server response code " + httpResponseCode));
                result.setExceptionOccurred(true);
            }

        } catch (Exception e) {
            logger.error(e, "Unknown problem fetching User details [User ID - %s]", userId);
            result.setExceptionOccurred(true);
            result.setException(e);
        } catch (Throwable t) {
            logger.error(t, "Unknown problem fetching User details [User ID - %s]", userId);
            result.setException(new Exception(t.getMessage()));
            result.setExceptionOccurred(true);
        }
        return result;
    }

    private BehanceSDKCityDTO getCityDTO(String city, String countryId, String stateID, String clientId) {

        try {
            Map<String, String> urlParams = new HashMap<String, String>();
            urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, clientId);
            String url = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.GET_CITIES_API_URL, urlParams);

            if ( TextUtils.isEmpty(countryId) == false ) {
                url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_LOCATION_FILTER_COUNTRY, countryId);
            }

            if ( TextUtils.isEmpty(stateID) == false ) {
                url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_LOCATION_FILTER_STATE, stateID);
            }

            if ( TextUtils.isEmpty(city) == false ) {
                url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_LOCATION_FILTER_CITY, city);
            }

            logger.debug("Get Cities URL - %s", url);

            BehanceConnectionResponse<String> httpResponse = BehanceHttpsConnection.getInstance().invokeHttpGetRequest(url);

            String jsonResponseString = httpResponse.getResponseObject();

            logger.debug("Get cities response: %s", jsonResponseString);
            JSONArray rootJsonArray = new JSONArray(jsonResponseString);
            for (int counter = 0; counter < rootJsonArray.length(); counter++) {
                Object cityObject = rootJsonArray.opt(counter);
                if (cityObject != null) {
                    JSONObject cityJsonObject = (JSONObject) cityObject;
                    if(cityJsonObject.optString("n").equalsIgnoreCase(city)) {
                        BehanceSDKCityDTO cityDTO = new BehanceSDKCityDTO();
                        cityDTO.setId(cityJsonObject.optString("id"));
                        cityDTO.setDisplayName(city);
                        return cityDTO;
                    }
                }
            }

        } catch (Exception e) {
            logger.debug(e, "Problem getting Cities from server");

        } catch (Throwable t) {
            logger.debug(t, "Problem getting Cities from server");

        }
        return null;
    }


    private BehanceSDKStateDTO getStateDTO(String state, String countryId, String clientId) {
        BehanceSDKStateDTO stateDTO = new BehanceSDKStateDTO();
        stateDTO.setDisplayName(state);
        try {
            Map<String, String> urlParams = new HashMap<String, String>();
            urlParams.put(BehanceSDKUrlUtil.KEY_COUNTRY_ID, countryId);
            urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, clientId);
            String url = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.GET_STATES_API_URL, urlParams);

            logger.debug("Get States of Country URL - %s", url);

            BehanceConnectionResponse<String> httpResponse = BehanceHttpsConnection.getInstance().invokeHttpGetRequest(url);

            String jsonResponseString = httpResponse.getResponseObject();
            logger.debug("Get States of Country [Country id - %s ] response: %s", countryId, jsonResponseString);

            JSONObject rootJsonObject = new JSONObject(jsonResponseString);
            Iterator<?> keysIterator = rootJsonObject.keys();
            while (keysIterator.hasNext()) {
                String keyStr = (String) keysIterator.next();
                Object stateObject = rootJsonObject.optJSONObject(keyStr);
                if (stateObject != null) {
                    JSONObject stateJsonObject = (JSONObject) stateObject;
                    if(stateJsonObject.optString("n").equalsIgnoreCase(state)) {
                        stateDTO.setId(stateJsonObject.optString("id"));
                        return stateDTO;
                    }
                }
            }
        } catch (Exception e) {
            logger.debug(e);
        } catch (Throwable t) {
            logger.debug(t);
        }
        return null;
    }

    private BehanceSDKCountryDTO getCountryDTO(String country, String clientId) {
        BehanceSDKCountryDTO countryDTO = new BehanceSDKCountryDTO();
        countryDTO.setDisplayName(country);

        try {
            Map<String, String> urlParams = new HashMap<String, String>();
            urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, clientId);
            String url = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.GET_COUNTRIES_API_URL, urlParams);

            BehanceConnectionResponse<String> httpResponse = BehanceHttpsConnection.getInstance().invokeHttpGetRequest(url);

            String jsonResponseString = httpResponse.getResponseObject();
            logger.debug("Get Countries response: %s", jsonResponseString);

            JSONObject rootJsonObject = new JSONObject(jsonResponseString);
            Iterator<?> keysIterator = rootJsonObject.keys();
            while (keysIterator.hasNext()) {
                String keyStr = (String) keysIterator.next();
                Object countryObject = rootJsonObject.optJSONObject(keyStr);
                if (countryObject != null) {
                    JSONObject countryJsonObject = (JSONObject) countryObject;
                    if(countryJsonObject.optString("n").equalsIgnoreCase(country)) {
                        countryDTO.setId(countryJsonObject.optString("id"));
                        return countryDTO;
                    }
                }
            }

        } catch (IOException e) {
            logger.error(e);
        } catch (JSONException e) {
            logger.error(e);
        }
        return null;
    }

    private String getProfileImage(JSONObject images) {

        try {
            if(images.has("276"))
                return images.getString("276");
            else if(images.has("138"))
                return images.getString("138");
            else if(images.has("129"))
                return images.getString("129");
            else if(images.has("115"))
                return images.getString("115");
            else if(images.has("78"))
                return images.getString("78");
            else if(images.has("50"))
                return images.getString("50");
            else if(images.has("32"))
                return images.getString("32");
        } catch (JSONException e) {
            logger.error(e);
        }
        return null;
    }

    @Override
    protected void onPostExecute(BehanceSDKGetUserProfileTaskResult result) {
        if (result.isExceptionOccurred()) {
            listener.onGetUserProfileFailure(result.getException(), getUserProfileTaskParams);
        } else {
            listener.onGetUserProfileSuccess(result, getUserProfileTaskParams);
        }
    }
}
