/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.asynctasks;

import android.os.AsyncTask;

import com.behance.sdk.asynctask.listeners.IBehanceSDKGetUserProjectsListAsyncTaskListener;
import com.behance.sdk.asynctask.params.BehanceSDKGetUserProjectsListAsyncTaskParams;
import com.behance.sdk.asynctasks.result.BehanceSDKAsyncTaskResultWrapper;
import com.behance.sdk.dto.parser.BehanceSDKProjectDTOParser;
import com.behance.sdk.dto.project.BehanceSDKProjectDTO;
import com.behance.sdk.exception.BehanceSDKException;
import com.behance.sdk.exception.BehanceSDKHTTPStatusCodeNotOKException;
import com.behance.sdk.exception.BehanceSDKProjectParseException;
import com.behance.sdk.exception.BehanceSDKUserNotAuthenticatedException;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.network.BehanceConnectionResponse;
import com.behance.sdk.network.BehanceHttpsConnection;
import com.behance.sdk.network.IBehanceHttpConnection;
import com.behance.sdk.util.BehanceSDKConstants;
import com.behance.sdk.util.BehanceSDKUrlUtil;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class BehanceSDKGetUserProjectsListAsyncTask extends AsyncTask<BehanceSDKGetUserProjectsListAsyncTaskParams, Void, BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKProjectDTO>>> {

	private BehanceSDKGetUserProjectsListAsyncTaskParams taskParams;
	private IBehanceSDKGetUserProjectsListAsyncTaskListener taskHandler;
	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKGetUserProjectsListAsyncTask.class);

	public BehanceSDKGetUserProjectsListAsyncTask(IBehanceSDKGetUserProjectsListAsyncTaskListener taskHandler) {
		this.taskHandler = taskHandler;
	}

	@Override
	protected BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKProjectDTO>> doInBackground(BehanceSDKGetUserProjectsListAsyncTaskParams... params) {
		BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKProjectDTO>> result = new BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKProjectDTO>>();
		if (params.length != 1) {
			result.setException(new BehanceSDKException("User Id is required"));
			result.setExceptionOccurred(true);
			return result;
		}

		taskParams = params[0];
		String userId = String.valueOf(taskParams.getUserId());

		try {

			int projectPageNumber = 1;
			boolean getAllProjects = true;

			if(taskParams.getPageNumber() > 0) {
				getAllProjects = false;
				projectPageNumber = taskParams.getPageNumber();
			}
			List<BehanceSDKProjectDTO> totalUserProjectsList = new ArrayList<BehanceSDKProjectDTO>();
			do {
				List<BehanceSDKProjectDTO> projectsFromServer = getUserProjects(userId, projectPageNumber, taskParams.isIncludePrivateProjects(), result);
				if (projectsFromServer != null) {
					totalUserProjectsList.addAll(projectsFromServer);
				}

				if (!getAllProjects || projectsFromServer == null ||
						projectsFromServer.size() < BehanceSDKGetUserProjectsListAsyncTaskParams.DEFAULT_REQUEST_PAGE_SIZE ||
						result.isExceptionOccurred()) {
					break;
				}
				projectPageNumber++;
			} while (true);
			result.setResult(totalUserProjectsList);
		} catch (Exception e) {
			logger.error(e, "Unknown problem fetching Project List for User [User ID - %s]", userId);
			result.setExceptionOccurred(true);
			result.setException(e);
		} catch (Throwable t) {
			logger.error(t, "Unknown problem fetching Project List for User [User ID - %s]", userId);
			BehanceSDKException ex = new BehanceSDKException(t);
			result.setException(ex);
			result.setExceptionOccurred(true);
		}
		return result;
	}

	private List<BehanceSDKProjectDTO> getUserProjects(String userId, int pageNumber, boolean includePrivateProjects,
													   BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKProjectDTO>> result) throws IOException, JSONException, BehanceSDKProjectParseException,
			BehanceSDKHTTPStatusCodeNotOKException,BehanceSDKUserNotAuthenticatedException{
		Map<String, String> urlParams = new HashMap<String, String>();
		urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, BehanceSDKConstants.BEHANCE_API_CLIENT_ID);
		urlParams.put(BehanceSDKUrlUtil.KEY_USER_ID, userId);
		String url = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.GET_USER_PROJECTS_API_URL, urlParams);

		url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_PAGE_NUMBER, pageNumber);
		url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_PAGE_SIZE,
				BehanceSDKGetUserProjectsListAsyncTaskParams.DEFAULT_REQUEST_PAGE_SIZE);


			String userAccessToken = taskParams.getUserAccessToken();
			if (userAccessToken != null) {
				url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_ACCESS_TOKEN, userAccessToken);
			}


		logger.debug("Get user projects URL - %s", url);
		BehanceConnectionResponse<String> response = BehanceHttpsConnection.getInstance().invokeHttpGetRequest(url);
		String jsonResponseString = response.getResponseObject();
		logger.debug("Get user projects response: %s", jsonResponseString);
		JSONObject jsonObject = new JSONObject(jsonResponseString);
		int httpResponseCode = jsonObject.getInt("http_code");

		if (httpResponseCode == IBehanceHttpConnection.SC_OK) {
			BehanceSDKProjectDTOParser parser = new BehanceSDKProjectDTOParser();
			return parser.parseJustProjects(jsonResponseString, includePrivateProjects);
		} else {
			logger.error("Unexpected HTTP Response code when trying to fetch User Projects. [User ID - %s] [Response code - %d]", userId,
					httpResponseCode);
			result.setException(new BehanceSDKException(BehanceSDKHTTPStatusCodeNotOKException.GET_USER_PROJECT_LIST_GET_PROJECTS_UNEXPECTED_RESPONSE_CODE,
					"Invalid server response code " + httpResponseCode));
			result.setExceptionOccurred(true);
		}
		return null;
	}

	@Override
	protected void onPostExecute(BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKProjectDTO>> resultWrapper) {
		if (resultWrapper.isExceptionOccurred()) {
			taskHandler.onGetUserProjectsListFailure(resultWrapper.getException(), taskParams);
		} else {
			taskHandler.onGetUserProjectsListSuccess(resultWrapper.getResult(), taskParams);
		}
	}
}