/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.asynctasks;

import android.os.AsyncTask;


import com.behance.sdk.asynctask.listeners.IBehanceSDKRetrieveBehanceUserDetailsUsingAdobeIDTaskHandler;
import com.behance.sdk.asynctask.params.BehanceSDKRetrieveBehanceUserDetailsUsingAdobeIDTaskParams;
import com.behance.sdk.asynctasks.result.BehanceSDKRetrieveBehanceUserDetailsUsingAdobeIDTaskResponse;
import com.behance.sdk.dto.parser.BehanceUserDTOParser;
import com.behance.sdk.exception.BehanceSDKException;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.managers.BehanceSDKUserManager;
import com.behance.sdk.network.BehanceConnectionResponse;
import com.behance.sdk.network.BehanceHttpsConnection;
import com.behance.sdk.network.IBehanceHttpConnection;
import com.behance.sdk.util.BehanceSDKConstants;
import com.behance.sdk.util.BehanceSDKUrlUtil;


import org.json.JSONObject;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class BehanceSDKRetrieveBehanceUserDetailsUsingAdobeIDTask extends
		AsyncTask<BehanceSDKRetrieveBehanceUserDetailsUsingAdobeIDTaskParams, Void, BehanceSDKRetrieveBehanceUserDetailsUsingAdobeIDTaskResponse> {

	private static final String NOT_LINKED = "not-linked";
	private static final String COM_ADOBE_AUTH_LINKING_WORKFLOW_URL = "com.adobe.auth.linking-workflow-url";
	private static final String COM_ADOBE_AUTH_LINKING_STATUS = "com.adobe.auth.linking-status";

	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKRetrieveBehanceUserDetailsUsingAdobeIDTask.class);

	private IBehanceSDKRetrieveBehanceUserDetailsUsingAdobeIDTaskHandler caller;

	public BehanceSDKRetrieveBehanceUserDetailsUsingAdobeIDTask(IBehanceSDKRetrieveBehanceUserDetailsUsingAdobeIDTaskHandler caller) {
		this.caller = caller;
	}

	@Override
	protected BehanceSDKRetrieveBehanceUserDetailsUsingAdobeIDTaskResponse doInBackground(BehanceSDKRetrieveBehanceUserDetailsUsingAdobeIDTaskParams... params) {
		BehanceSDKRetrieveBehanceUserDetailsUsingAdobeIDTaskResponse response = new BehanceSDKRetrieveBehanceUserDetailsUsingAdobeIDTaskResponse();

		BehanceSDKUserManager behanceUserManager = BehanceSDKUserManager.getInstance();

        String userID = behanceUserManager.getAdobeUserId();
        String accessToken = behanceUserManager.getAccessToken();


        try {
			Map<String, String> urlParams = new HashMap<String, String>();
			urlParams.put(BehanceSDKUrlUtil.KEY_USER_ID, userID);
			urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, BehanceSDKConstants.BEHANCE_API_CLIENT_ID);
			urlParams.put(BehanceSDKUrlUtil.KEY_USER_TOKEN, accessToken);

			String url = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.GET_BEHANCE_USER_PROFILE_WITH_IMS_TOKEN_API_URL, urlParams);
			BehanceConnectionResponse<String> connectionResponse;
			logger.debug("Get user detail URL: %s", url);

			connectionResponse = BehanceHttpsConnection.getInstance().invokeHttpGetRequest(url);
			String jsonResponseString = connectionResponse.getResponseObject();

			logger.debug("Get user detail response: %s", jsonResponseString);
			JSONObject jsonObject = new JSONObject(jsonResponseString);
			int httpResponseCode = jsonObject.getInt("http_code");
			if (httpResponseCode == IBehanceHttpConnection.SC_OK) {
				BehanceUserDTOParser parser = new BehanceUserDTOParser();
				JSONObject userJSONObject = jsonObject.optJSONObject("user");
				response.setUserDTO(parser.parse(userJSONObject));
			} else if (httpResponseCode == IBehanceHttpConnection.SC_FORBIDDEN) {
				logger.error("HTTP 403 Response received when trying to fetch User Details. [User ID - %s] [Response code - %d]", userID,
						httpResponseCode);
				Map<String,List<String>> headers = connectionResponse.getHeaders();
				List<String> linkStatusHeader =  null;
				if(headers!= null) {
					linkStatusHeader = headers.get(COM_ADOBE_AUTH_LINKING_STATUS);
				}
				if (linkStatusHeader != null) {
					if (linkStatusHeader.get(0).equalsIgnoreCase(NOT_LINKED)) {
						List<String> linkURLHeader = headers.get(COM_ADOBE_AUTH_LINKING_WORKFLOW_URL);
						if(linkURLHeader!= null) {
							if (linkURLHeader.get(0) != null) {
								response.setException(new BehanceSDKException(
										"User Account Not Linked " + httpResponseCode + linkURLHeader.get(0)));
								logger.debug("Link URL for the account is. [URL - %s]", linkURLHeader.get(0));
								return response;
							}
						}
					}
				}
				logger.error("Header Values for the link status/link-url is empty");
				response.setException(new BehanceSDKException(
						"Header Values for the link status/link-url is empty"));
			} else {
				logger.error("Unexpected HTTP Response code when trying to fetch User Details. [User ID - %s] [Response code - %d]", userID,
						httpResponseCode);
				response.setException(new BehanceSDKException("Invalid server response code " + httpResponseCode));
			}
		} catch (Exception ex) {
			logger.error(ex, "Problem retrieving Behance User details using Adobe ID from server");
			logger.error(ex);
			response.setException(ex);
		} catch (Throwable t) {
			logger.error(t, "Problem retrieving Behance User details using Adobe ID from server");
			BehanceSDKException ex = new BehanceSDKException(t);
			response.setException(ex);
		}
		return response;
	}

	@Override
	protected void onCancelled() {
		super.onCancelled();
		this.cancel(true);
	}

	@Override
	protected void onPostExecute(BehanceSDKRetrieveBehanceUserDetailsUsingAdobeIDTaskResponse result) {
		if (result.getException() == null) {
			logger.debug("RetrieveBehanceUserDetailsUsingAdobeIDTask onPost executed. Caller success is being invoked.");
			caller.onRetrieveBehanceUserDetailsSuccess(result.getUserDTO());
		} else {
			logger.debug("RetrieveBehanceUserDetailsUsingAdobeIDTask onPost executed. Caller failure is being invoked.");
			caller.onRetrieveBehanceUserDetailsFailure(result.getException());
		}
	}
}