/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.asynctasks;

/**
 * @author Sujit Reddy G
 *
 */

import android.os.AsyncTask;
import android.text.TextUtils;


import com.behance.sdk.IBehanceSDKSearchProjectListener;
import com.behance.sdk.asynctask.params.BehanceSDKSearchProjectsTaskParams;
import com.behance.sdk.asynctasks.result.BehanceSDKAsyncTaskResultWrapper;
import com.behance.sdk.dto.location.BehanceSDKCityDTO;
import com.behance.sdk.dto.location.BehanceSDKStateDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectDTO;
import com.behance.sdk.dto.parser.BehanceSDKProjectDTOParser;
import com.behance.sdk.enums.BehanceSDKProjectsSortOption;
import com.behance.sdk.enums.BehanceSDKProjectsTimeSpan;
import com.behance.sdk.exception.BehanceSDKException;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.managers.BehanceSDKUserManager;
import com.behance.sdk.network.BehanceConnectionResponse;
import com.behance.sdk.network.BehanceHttpsConnection;
import com.behance.sdk.util.BehanceSDKUrlUtil;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class BehanceSDKSearchProjectsAsyncTask extends AsyncTask<BehanceSDKSearchProjectsTaskParams, Void, BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKProjectDTO>>> {

	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKSearchProjectsAsyncTask.class);

	private IBehanceSDKSearchProjectListener taskHandler;
	private BehanceSDKSearchProjectsTaskParams taskParams;

	public BehanceSDKSearchProjectsAsyncTask(IBehanceSDKSearchProjectListener taskHandler) {
		this.taskHandler = taskHandler;
	}

	@Override
	protected BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKProjectDTO>> doInBackground(BehanceSDKSearchProjectsTaskParams... params) {

		BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKProjectDTO>> result = new BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKProjectDTO>>();

		try {

			if (params.length == 1 && params[0] != null) {

					BehanceSDKSearchProjectsTaskParams projectParams = params[0];
					this.taskParams = projectParams;

					Map<String, String> urlParams = new HashMap<String, String>();
					urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, projectParams.getClientId());
					String url = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.GET_PROJECTS_API_URL, urlParams);

					url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_PAGE_NUMBER, projectParams.getPageNumber());
					url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_PAGE_SIZE, projectParams.getPageSize());
					// set sort option
					BehanceSDKProjectsSortOption sortOption = projectParams.getSortOption();
					if (null == sortOption)
						sortOption = BehanceSDKProjectsSortOption.FEATURED_DATE;
					url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_SORT, sortOption.toString());

					// set time span
					BehanceSDKProjectsTimeSpan timeSpan = projectParams.getTimeSpan();
					if (null != timeSpan) {
						url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_TIME, timeSpan.toString());
					}

					// set creative field
					String field = projectParams.getField();
					if (false == TextUtils.isEmpty(field))
						url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_FIELD, field);

					// set country
					String country = projectParams.getCountry();
					if (false == TextUtils.isEmpty(country))
						url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_COUNTRY, country);

					// set state
					String state = projectParams.getState();
					if (false == TextUtils.isEmpty(state) && false == state.equals(BehanceSDKStateDTO.ALL_STATES_NAME))
						url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_STATE, state);
					// set city
					String city = projectParams.getCity();
					if (false == TextUtils.isEmpty(city) && false == city.equals(BehanceSDKCityDTO.ALL_CITY_NAME))
						url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_CITY, city);

					// set search string
					String searchString = projectParams.getSearchString();
					if (false == TextUtils.isEmpty(searchString))
						url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_FREE_TEXT_QUERY, searchString);

					// set access token
					BehanceSDKUserManager sdkUserManager = BehanceSDKUserManager.getInstance();
					String userAccessToken = sdkUserManager.checkExpiryAndGetAccessToken();
					if (userAccessToken != null) {
						url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_ACCESS_TOKEN, userAccessToken);
					}

					logger.debug("Get Projects async task request url - %s", url);

					BehanceConnectionResponse<String> httpResponse = BehanceHttpsConnection.getInstance().invokeHttpGetRequest(url);
					String jsonResponseString = httpResponse.getResponseObject();


					logger.debug("Get projects async task response: %s", jsonResponseString);
					BehanceSDKProjectDTOParser parser = new BehanceSDKProjectDTOParser();
					List<BehanceSDKProjectDTO> projectsList = parser.parseJustProjects(jsonResponseString);
					result.setResult(projectsList);

			} else {
				result.setExceptionOccurred(true);
				result.setException(new BehanceSDKException("BehanceSDKSearchProjectsTaskParams cannot be null"));
			}


		} catch (Exception e) {
			logger.error(e, "Problem getting Projects from server");
			result.setExceptionOccurred(true);
			result.setException(e);
		} catch (Throwable t) {
			logger.error(t, "Problem getting Projects from server");
			BehanceSDKException ex = new BehanceSDKException(t);
			result.setException(ex);
			result.setExceptionOccurred(true);
		}
		return result;
	}

	@Override
	protected void onPostExecute(BehanceSDKAsyncTaskResultWrapper<List<BehanceSDKProjectDTO>> resultWrapper) {
		if (resultWrapper.isExceptionOccurred()) {
			taskHandler.onSearchProjectsFailure(resultWrapper.getException());
		} else {
			taskHandler.onSearchProjectsSuccess(resultWrapper.getResult());
		}
	}
}
