/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.dto.parser;

import android.graphics.Color;


import com.behance.sdk.dto.project.BehanceSDKAbstractProjectModuleDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectCopyrightDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectCoversDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectFeaturedDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectStatsDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectStylesDTO;
import com.behance.sdk.dto.project.BehanceSDKProjectToolDTO;
import com.behance.sdk.exception.BehanceSDKProjectParseException;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.List;

public class BehanceSDKProjectDTOParser {

	private static final float ProjectTextModuleDefaultFontSize = 12.0f;
	private static final float ProjectTextModuleDefaultFontSizeV5 = 14.0f;

	private static final float ProjectTextModuleCaptionDefaultFontSize = 13.0f;
	private static final float ProjectTextModuleCaptionDefaultFontSizeV5 = 11.0f;

	private static final String ProjectTextModuleDefaultLineHeight = "1.4";

	public List<BehanceSDKProjectDTO> parseJustProjects(String projectsJSON) throws BehanceSDKProjectParseException {
		return parseJustProjects(projectsJSON, true);
	}

	public List<BehanceSDKProjectDTO> parseJustProjects(String projectsJSON, boolean includePrivateProjects) throws BehanceSDKProjectParseException {
		try {
			JSONObject jsonObject = new JSONObject(projectsJSON);
			JSONArray projectsArray = jsonObject.getJSONArray("projects");
			return parseJustProjects(projectsArray, includePrivateProjects);
		} catch (JSONException e) {
			throw new BehanceSDKProjectParseException(e.getMessage(), e);
		}
	}

	public BehanceSDKProjectDTO parseProjectWithDetails(JSONObject projectJsonObject) throws BehanceSDKProjectParseException {
		try {
			return populateProjectDetails(projectJsonObject, true);
		} catch (JSONException e) {
			throw new BehanceSDKProjectParseException(e.getMessage(), e);
		}
	}

	private List<BehanceSDKProjectDTO> parseJustProjects(JSONArray projectsArray, boolean includePrivateProjects) throws BehanceSDKProjectParseException {

		try {
			List<BehanceSDKProjectDTO> projects = new ArrayList<BehanceSDKProjectDTO>(projectsArray.length());
			for (int i = 0; i < projectsArray.length(); i++) {
				JSONObject projectJsonObject = projectsArray.getJSONObject(i);
				if (projectJsonObject != null) {
					if (isProjectPublic(projectJsonObject) || includePrivateProjects) {
						BehanceSDKProjectDTO projectDTO = populateProjectDetails(projectJsonObject, false);
						if(projectDTO != null && !projectDTO.getName().equals("[Invalid Project]"))
							projects.add(projectDTO);
					}
				}
			}
			return projects;
		} catch (JSONException e) {
			throw new BehanceSDKProjectParseException(e.getMessage(), e);
		}

	}

	private boolean isProjectPublic(JSONObject projectJsonObject) {
		String projectPrivacy = projectJsonObject.optString("privacy");
		if (BehanceSDKProjectDTO.PUBLIC_PROJECT.equalsIgnoreCase(projectPrivacy)) {
			return true;
		} else {
			return false;
		}
	}

	private BehanceSDKProjectDTO populateProjectDetails(JSONObject projectJsonObject, boolean parseFullProjectDetails) throws JSONException {
		if (projectJsonObject != null) {
			BehanceSDKProjectDTO projectDTOObject = new BehanceSDKProjectDTO();
			projectDTOObject.setId(projectJsonObject.optString("id"));
			projectDTOObject.setName(projectJsonObject.optString("name"));
			projectDTOObject.setPublishedDate(projectJsonObject.optLong("published_on"));
			projectDTOObject.setCreatedDate(projectJsonObject.optLong("created_on"));
			projectDTOObject.setModifiedDate(projectJsonObject.optLong("modified_on"));
			projectDTOObject.setUrl(projectJsonObject.optString("url"));
			projectDTOObject.setPrivacy(projectJsonObject.optString("privacy"));
			populateProjectCovers(projectJsonObject.optJSONObject("covers"), projectDTOObject);
			populateProjectStats(projectJsonObject.optJSONObject("stats"), projectDTOObject);
			populateProjectOwners(projectJsonObject.optJSONArray("owners"), projectDTOObject);
			populateProjectFields(projectDTOObject, projectJsonObject);
			if (parseFullProjectDetails) {
				projectDTOObject.setMatureAccess(projectJsonObject.optString("mature_access"));
				int matureContent = projectJsonObject.optInt("mature_content");
				projectDTOObject.setMatureContent(matureContent != 0);
				projectDTOObject.setDescription(projectJsonObject.optString("description"));
				projectDTOObject.setCustomCSS(projectJsonObject.optString("custom_css"));
				projectDTOObject.setShortUrl(projectJsonObject.optString("short_url"));
				projectDTOObject.setHTML(projectJsonObject.optString("html"));
				projectDTOObject.setCanvasWidth(projectJsonObject.optInt("canvas_width"));
				projectDTOObject.setAllowComments(projectJsonObject.optInt("allow_comments") != 0);

				JSONArray projectTags = projectJsonObject.optJSONArray("tags");
				if (projectTags != null) {
					for (int tagsCount = 0; tagsCount < projectTags.length(); tagsCount++) {
						projectDTOObject.addTag(projectTags.getString(tagsCount));
					}
				}

				populateProjectModules(projectJsonObject.optJSONArray("modules"), projectDTOObject);
				populateProjectTools(projectJsonObject.optJSONArray("tools"),projectDTOObject);
				populateProjectFeaturedDetails(projectJsonObject.optJSONArray("features"), projectDTOObject);
				populateProjectCopyright(projectJsonObject.optJSONObject("copyright"), projectDTOObject);

				projectDTOObject.setProjectStyles(buildProjectStylesDTO(projectJsonObject.optJSONObject("styles"), projectJsonObject.optInt("editor_version")));
			}
			return projectDTOObject;
		}
		return null;
	}

	private void populateProjectFields(BehanceSDKProjectDTO projectDTOObject, JSONObject project) {
		JSONArray fields = project.optJSONArray("fields");
		if (fields != null) {
			List<String> fieldsList = new ArrayList<String>(fields.length());
			for (int j = 0; j < fields.length(); j++) {
				String field = fields.optString(j);
				fieldsList.add(field);
			}
			projectDTOObject.setFields(fieldsList);
		}
	}

	private void populateProjectModules(JSONArray modulesJSON, BehanceSDKProjectDTO projectDTOObject) throws JSONException {
		if (null != modulesJSON) {
			for (int i = 0; i < modulesJSON.length(); i++) {
				JSONObject module = modulesJSON.optJSONObject(i);
				BehanceSDKAbstractProjectModuleDTO projectModuleInstance = BehanceSDKProjectModuleFactory.getProjectModuleInstance(module);
				if (null != projectModuleInstance) {
					projectDTOObject.addModule(projectModuleInstance);
				}
			}
		}
	}

	private void populateProjectTools(JSONArray toolsJSON, BehanceSDKProjectDTO projectDTO) throws JSONException {
		if(toolsJSON == null)
			return;
		List<BehanceSDKProjectToolDTO> tools = new ArrayList<>();
		for(int i = 0; i < toolsJSON.length(); i++) {
			JSONObject toolOb = toolsJSON.getJSONObject(i);
			BehanceSDKProjectToolDTO toolDTO = new BehanceSDKProjectToolDTO();
			toolDTO.setId(toolOb.optInt("id"));
			toolDTO.setTitle(toolOb.optString("title"));
			JSONObject synonym = toolOb.optJSONObject("synonym");
			if(synonym != null) {
				toolDTO.setUrl(synonym.optString("url"));
				toolDTO.setIconUrl(synonym.optString("icon_url"));
			}
			tools.add(toolDTO);
		}
		projectDTO.setTools(tools);
	}

	private void populateProjectStats(JSONObject statsJSON, BehanceSDKProjectDTO projectDTOObject) throws JSONException {
		if (null != statsJSON) {
			BehanceSDKProjectStatsDTO stats = new BehanceSDKProjectStatsDTO();
			stats.setAppreciationsCount(statsJSON.optInt("appreciations"));
			stats.setViewsCount(statsJSON.optInt("views"));
			stats.setCommentsCount(statsJSON.optInt("comments"));
			projectDTOObject.setStats(stats);
		}
	}

	private void populateProjectOwners(JSONArray ownerArray, BehanceSDKProjectDTO projectDTOObject) throws JSONException {
		if (null != ownerArray) {
			BehanceUserDTOParser userParser = new BehanceUserDTOParser();
			for (int i = 0; i < ownerArray.length(); i++) {
				JSONObject userJsonObject = ownerArray.optJSONObject(i);
				projectDTOObject.addOwner(userParser.parse(userJsonObject));
			}
		}
	}

	private void populateProjectCovers(JSONObject coversJSON, BehanceSDKProjectDTO projectDTOObject) throws JSONException {
		if (null != coversJSON) {
			BehanceSDKProjectCoversDTO covers = new BehanceSDKProjectCoversDTO();
			covers.addCoverImages(BehanceSDKProjectCoversDTO.SIZE_115, coversJSON.optString(String.valueOf(BehanceSDKProjectCoversDTO.SIZE_115)));
			covers.addCoverImages(BehanceSDKProjectCoversDTO.SIZE_202, coversJSON.optString(String.valueOf(BehanceSDKProjectCoversDTO.SIZE_202)));
			covers.addCoverImages(BehanceSDKProjectCoversDTO.SIZE_230, coversJSON.optString(String.valueOf(BehanceSDKProjectCoversDTO.SIZE_230)));
			covers.addCoverImages(BehanceSDKProjectCoversDTO.SIZE_404, coversJSON.optString(String.valueOf(BehanceSDKProjectCoversDTO.SIZE_404)));
			projectDTOObject.setCovers(covers);
		}
	}

	private void populateProjectFeaturedDetails(JSONArray projectFeatures, BehanceSDKProjectDTO projectDTOObject) throws JSONException {
		if (projectFeatures != null) {
			for (int featureCount = 0; featureCount < projectFeatures.length(); featureCount++) {
				JSONObject feature = projectFeatures.getJSONObject(featureCount);
				BehanceSDKProjectFeaturedDTO featureDTO = new BehanceSDKProjectFeaturedDTO();
				featureDTO.setFeaturedDate(feature.optLong("featured_on"));
				featureDTO.setProjectUrl(feature.optString("url"));

				JSONObject featureSite = feature.optJSONObject("site");
				if (featureSite != null) {
					featureDTO.setSiteDomain(featureSite.optString("domain"));
					featureDTO.setSiteIcon(featureSite.optString("icon"));
					featureDTO.setSiteId(featureSite.optString("id"));
					featureDTO.setSiteKey(featureSite.optString("key"));
					featureDTO.setSiteName(featureSite.optString("name"));
					featureDTO.setSiteUrl(featureSite.optString("url"));
					featureDTO.setNetworkId(featureSite.optString("network_id"));
					JSONObject featureSiteRibbon = featureSite.optJSONObject("ribbon");
					if (featureSiteRibbon != null) {
						featureDTO.setSiteRibbonImage(featureSiteRibbon.optString("image"));
						featureDTO.setSiteRibbonBiggerImage(featureSiteRibbon.optString("image_2x"));
					}
					projectDTOObject.addFeature(featureDTO);
				}
			}
		}
	}

	private void populateProjectCopyright(JSONObject copyrightJSONObject, BehanceSDKProjectDTO projectDTOObject) throws JSONException {
		if (copyrightJSONObject != null) {
			BehanceSDKProjectCopyrightDTO copyrightDTO = new BehanceSDKProjectCopyrightDTO();
			copyrightDTO.setLicenseType(copyrightJSONObject.optString("license"));
			copyrightDTO.setLicenseDescription(copyrightJSONObject.optString("description"));
			projectDTOObject.setCopyright(copyrightDTO);
		}
	}

	private BehanceSDKProjectStylesDTO buildProjectStylesDTO(JSONObject stylesJSONOb, int editorVersion) {
		if(stylesJSONOb == null)
			return null;
		BehanceSDKProjectStylesDTO stylesDTO = new BehanceSDKProjectStylesDTO();
		stylesDTO.setBgColor(Color.parseColor("#"+stylesJSONOb.optJSONObject("background").optString("color")));
		JSONObject backgroundImageOb = stylesJSONOb.optJSONObject("background").optJSONObject("image");
		if(backgroundImageOb != null) {
			stylesDTO.setBgImageUrl(backgroundImageOb.optString("url"));
			stylesDTO.setBgImageRepeat(backgroundImageOb.optString("repeat").equalsIgnoreCase("repeat"));
		}
		String cssBlock = "";
		JSONObject textStylesOb = stylesJSONOb.optJSONObject("text");
		if(textStylesOb != null) {
			float textSizeFallback = editorVersion >=5 ? ProjectTextModuleDefaultFontSizeV5 : ProjectTextModuleDefaultFontSize;
			float captionTextSizeFallback = editorVersion >=5 ? ProjectTextModuleCaptionDefaultFontSizeV5 : ProjectTextModuleCaptionDefaultFontSize;
			cssBlock += buildCSSProperty(".title", textStylesOb.optJSONObject("title"), textSizeFallback, true) + "\r\n";
			cssBlock += buildCSSProperty(".sub-title", textStylesOb.optJSONObject("subtitle"), textSizeFallback, true) + "\r\n";
			cssBlock += buildCSSProperty("div, p, .main-text", textStylesOb.optJSONObject("paragraph"), textSizeFallback, true) + "\r\n";
			cssBlock += buildCSSProperty(".caption", textStylesOb.optJSONObject("caption"), captionTextSizeFallback, true) + "\r\n";
			cssBlock += buildCSSProperty("a", textStylesOb.optJSONObject("link"), textSizeFallback, false) + "\r\n";
			cssBlock += ".underline {\ndisplay: inline;\ntext-decoration: underline;\n}" + "\r\n";
			cssBlock += ".italic {\ndisplay: inline;\nfont-weight: italic;\n}" + "\r\n";
			cssBlock += ".bold {\ndisplay: inline;\nfont-weight: bold;\n}" + "\r\n";
			cssBlock += "body {\nmargin: 0;\n}";

			stylesDTO.setTitleCSS(buildCSSProperty(textStylesOb.optJSONObject("title"), textSizeFallback));
			stylesDTO.setSubTitleCSS(buildCSSProperty(textStylesOb.optJSONObject("subtitle"), textSizeFallback));
			stylesDTO.setMainTextCSS(buildCSSProperty(textStylesOb.optJSONObject("paragraph"), textSizeFallback));
			stylesDTO.setCaptionCSS(buildCSSProperty(textStylesOb.optJSONObject("caption"), captionTextSizeFallback));
			stylesDTO.setLinkCSS(buildCSSProperty(textStylesOb.optJSONObject("link"), textSizeFallback));
			stylesDTO.setUnderlineCSS( "{\ndisplay: block;\ntext-decoration: underline;\n}");
		}
		stylesDTO.setCustomCSS(cssBlock);
		stylesDTO.setDividerCSS(buildDividerCSS(stylesJSONOb.optJSONObject("dividers")));
		JSONObject spacingOb = stylesJSONOb.optJSONObject("spacing");
		stylesDTO.setProjectMarginTop(spacingOb.optJSONObject("project").optInt("top_margin"));
		stylesDTO.setModuleMarginBottom(spacingOb.optJSONObject("modules").optInt("bottom_margin"));
		return stylesDTO;
	}

	private String buildCSSProperty(String name, JSONObject textStyleOb, float fallback, boolean forceBlock) {
		if(textStyleOb == null)
			return null;
		String css = name + " {\n";
		css += "font-family: " + textStyleOb.optString("font_family") + ";\n";
		css += "font-weight: " + textStyleOb.optString("font_weight") + ";\n";
		css += "color: " + textStyleOb.optString("color") + ";\n";
		if(!textStyleOb.optString("text_align").equalsIgnoreCase("left"))
			css += "text-align: " + textStyleOb.optString("text_align") + ";\n";
		if(!textStyleOb.optString("line_height").replace("em","").equalsIgnoreCase(ProjectTextModuleDefaultLineHeight))
			css += "line-height: " + textStyleOb.optString("line_height", String.valueOf(ProjectTextModuleDefaultLineHeight)).replace("em","") + ";\n";
		if(!textStyleOb.optString("font_size").equalsIgnoreCase(String.valueOf(fallback)))
			css += "font-size: " + textStyleOb.optString("font_size", String.valueOf(fallback)) + ";\n";
		css += "text-decoration: " + textStyleOb.optString("text_decoration") + ";\n";
		css += "font-style:" + textStyleOb.optString("font_style") + ";\n";
		css += "display: " + (forceBlock ? "block;\n" : textStyleOb.optString("display") + ";\n");
		css += "}";
		return css;
	}

	private String buildCSSProperty(JSONObject textStyleOb, float fallback) {
		if(textStyleOb == null)
			return null;
		String css = "font-family: " + textStyleOb.optString("font_family") + ";\n";
		css += "font-weight: " + textStyleOb.optString("font_weight") + ";\n";
		css += "color: " + textStyleOb.optString("color") + ";\n";
		css += "text-align: " + textStyleOb.optString("text_align") + ";\n";
		css += "line-height: " + textStyleOb.optString("line_height", String.valueOf(ProjectTextModuleDefaultLineHeight)).replace("em","") + ";\n";
		css += "font-size: " + textStyleOb.optString("font_size", String.valueOf(fallback)) + ";\n";
		css += "text-decoration: " + textStyleOb.optString("text_decoration") + ";\n";
		css += "font-style:" + textStyleOb.optString("font_style") + ";\n";
		css += "display: " + "block;\n";//textStyleOb.optString("display") + ";";
		return css;
	}

	private String buildDividerCSS(JSONObject dividerOb) {
		if(dividerOb == null || dividerOb.optString("display").equalsIgnoreCase("none"))
			return null;
		String css = "p.divider {\n";
		css += "font-size: " + dividerOb.optString("font_size") + ";\n";
		css += "line-height: " + dividerOb.optString("line_height") + ";\n";
		css += "height: " + dividerOb.optString("height") + ";\n";
		css += "border-color: " + dividerOb.optString("border_color") + ";\n";
		css += "margin: " + dividerOb.optString("margin") + ";\n";
		css += "position: " + dividerOb.optString("position") + ";\n";
//		css += "top:" + dividerOb.optString("top") + ";";
		css += "border-width: " + dividerOb.optString("border_width") + ";\n";
		css += "border-style: " + dividerOb.optString("border_style") + " none none none;\n";
		css += "}";
		return css;
	}
}
