/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.behance.sdk.managers;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import android.app.Activity;

import com.behance.sdk.IBehanceSDKUserCredentials;
import com.behance.sdk.dto.BehanceSDKCreativeFieldDTO;
import com.behance.sdk.enums.BehanceSDKCopyrightOption;
import com.behance.sdk.project.NewBehanceProject;
import com.behance.sdk.project.modules.CoverImage;
import com.behance.sdk.project.modules.ImageModule;
import com.behance.sdk.project.modules.ProjectModule;
import com.behance.sdk.project.modules.ProjectModuleTypes;
import com.behance.sdk.util.BehanceSDKConstants;

/**
 * @author ugs
 *
 */
public class BehanceSDKProjectPublishWFManager {
	
	private static final BehanceSDKProjectPublishWFManager INSTANCE = new BehanceSDKProjectPublishWFManager();
	
	private boolean hideTwitterSharing;
	
	private boolean hideFacebookSharing;
	
	private List<ImageModule> selectedImagesForCover;
	
	private NewBehanceProject currentBehanceProject;

	private IBehanceSDKUserCredentials userCredentials;
	
	private List<ProjectModuleChangeListener> projectModuleChangeListeners;
	
	private List<CoverImageChangeListener> coverImageChangeListeners;
	
	private Class<? extends Activity> notificationHandlerActivityClass;

	public interface ProjectModuleChangeListener {
		public void projectModulesChanged(List<ProjectModule> modules);
		
		public void rotateImage(ImageModule module);
	}
	
	public interface CoverImageChangeListener {
		public void coverImageChanges(CoverImage coverImage);
	}

	public static BehanceSDKProjectPublishWFManager getInstance() {
		return INSTANCE;
	}
	
	private BehanceSDKProjectPublishWFManager(){
		projectModuleChangeListeners = new ArrayList<ProjectModuleChangeListener>(2);
		coverImageChangeListeners = new ArrayList<CoverImageChangeListener>(2);
		
	}
	
	
	
	public void addProjectModuleChangeListener(ProjectModuleChangeListener listener) {
		if(false ==  projectModuleChangeListeners.contains(listener))
			projectModuleChangeListeners.add(listener);
	}
	
	public void removeProjectModuleChangeListener(ProjectModuleChangeListener listener) {
		projectModuleChangeListeners.remove(listener);
	}
	
	public void addCoverImageChangeListener(CoverImageChangeListener listener) {
		if(false ==  coverImageChangeListeners.contains(listener))
			coverImageChangeListeners.add(listener);
	}
	
	public void removeCoverImageChangeListener(CoverImageChangeListener listener) {
		coverImageChangeListeners.remove(listener);
	}
	
	public void addImagesSelectedForCover(List<ImageModule> coverImageSelection) {
		if(null != coverImageSelection && false == coverImageSelection.isEmpty()){
			if(null == selectedImagesForCover)
				selectedImagesForCover = new ArrayList<ImageModule>(coverImageSelection.size());
			selectedImagesForCover.addAll(coverImageSelection);
		}
	}
	
	
	public List<ImageModule> getImagesSelectedForCover() {
		List<ImageModule> imagesForCover = new ArrayList<ImageModule>();
		if(null != currentBehanceProject){
			List<ImageModule> imageModules = currentBehanceProject.getImageModules();
			if(null != imageModules && false  == imageModules.isEmpty())
				imagesForCover.addAll(imageModules);
		}

		if(null != selectedImagesForCover && false == selectedImagesForCover.isEmpty()){
			imagesForCover.addAll(selectedImagesForCover);
		}
		return imagesForCover;
	}
	
	public List<ImageModule> getImagesSelectedOnlyForCover() {
		return selectedImagesForCover;
	}
	
	
	public void addImagesSelectedForProject(List<ImageModule> projectImageSelection) {
		if(null != projectImageSelection && false == projectImageSelection.isEmpty()){
			if(null == currentBehanceProject){
				initializeProject();
			}
			currentBehanceProject.addImagesToProject(projectImageSelection);
			notifyListeners();
		}else{
			if(null != currentBehanceProject){
				currentBehanceProject.removeAllImageModules();
				notifyListeners();
			}
		}
	}


	public void initializeProject() {
		if(null == currentBehanceProject)
			currentBehanceProject = new NewBehanceProject();
	}
	
	private void notifyListeners() {
		for (ProjectModuleChangeListener listener : projectModuleChangeListeners) {
			listener.projectModulesChanged(currentBehanceProject.getProjectModules());
		}
	}
	
	public void clearImagesSelectedForCover() {
		if(null != selectedImagesForCover){
			List<ImageModule> imageModules = currentBehanceProject.getImageModules();
			for(ProjectModule module : selectedImagesForCover){
				if(module.getType() == ProjectModuleTypes.CREATIVECLOUD_ASSET &&
						false == imageModules.contains(module)){
					module.deleteFromFileSystem();
				}
			}

			selectedImagesForCover.clear();
			selectedImagesForCover = null;
		}
	}
	
	public void removeAllModuleChangeListeners() {
		if(null != projectModuleChangeListeners){
			projectModuleChangeListeners.clear();
		}
	}
	
	
	public List<ImageModule> getImagesSelectedForProject() {
		List<ImageModule> selectedImagesForProject = null;
		if(null != currentBehanceProject)
				selectedImagesForProject = currentBehanceProject.getImageModules();
		return selectedImagesForProject ;
	}
	
	
	public List<ProjectModule> getProjectModules() {
		if(null == currentBehanceProject)
			return null;
		return currentBehanceProject.getProjectModules();
	}

	/**
	 * @param imageFile
	 */
	public void addImageSelectedForProject(File imageFile) {
		if(null == currentBehanceProject)
					initializeProject();
		ImageModule module = new ImageModule(imageFile);
		currentBehanceProject.addProjectModule(module);
		notifyListeners();
	}

	public void reorderSelectedModules(int fromPosition, int toPosition) {
		if(null != currentBehanceProject)
			currentBehanceProject.reorderSelectedModules(fromPosition, toPosition);
		notifyListeners();
	}

	public CoverImage getSelectedCoverImage() {
		if(null != currentBehanceProject)
			return currentBehanceProject.getCoverImage();
		return null;
	}

	public void setSelectedCoverImage(CoverImage cover) {
		//by now the project should get created so not checking for null
		currentBehanceProject.setCoverImage(cover);
		if(null != coverImageChangeListeners){
			for (CoverImageChangeListener listener : coverImageChangeListeners) {
				listener.coverImageChanges(cover);
			}
		}
	}

	public void addModuleToCurrentProject(ProjectModule projectModule) {
		if(null == currentBehanceProject)
			initializeProject();
		currentBehanceProject.addProjectModule(projectModule);
		notifyListeners();
	}

	public void setProjectTitle(String projectTitle) {
		if(null != currentBehanceProject)
			currentBehanceProject.setTitle(projectTitle);
	}
	public String getProjectTitle() {
		if(null != currentBehanceProject)
			return currentBehanceProject.getTitle();
		return BehanceSDKConstants.EMPTY_STRING;
	}

	public void setProjectTags(String projectTags) {
		if(null != currentBehanceProject)
			currentBehanceProject.setTags(projectTags);
	}
	public String getProjectTags() {
		if(null != currentBehanceProject)
			return currentBehanceProject.getTags();
		return null;
	}

	public void setProjectFields(List<BehanceSDKCreativeFieldDTO> projectFields) {
		if(null != currentBehanceProject)
			currentBehanceProject.setFields(projectFields);
	}
	
	public List<BehanceSDKCreativeFieldDTO> getProjectFields() {
		if(null != currentBehanceProject)
			return currentBehanceProject.getFields();
		return null;
	}

	public void setProjectDescription(String projectDesc) {
		if(null != currentBehanceProject)
			currentBehanceProject.setDescription(projectDesc);
	}
	
	public String getProjectDescription() {
		if(null != currentBehanceProject)
			return currentBehanceProject.getDescription();
		return BehanceSDKConstants.EMPTY_STRING;
	}

	public IBehanceSDKUserCredentials getUserCredentials() {
		return userCredentials;
	}

	public void setUserCredentials(IBehanceSDKUserCredentials userCredentials) {
		this.userCredentials = userCredentials;
	}

	public void setCopyrightOption(BehanceSDKCopyrightOption selectedCopyRight) {
		if(null != currentBehanceProject)
			currentBehanceProject.setCopyRight(selectedCopyRight);
	}
	
	public BehanceSDKCopyrightOption getCopyrightOption() {
		if(null != currentBehanceProject)
			return currentBehanceProject.getCopyRight();
		return BehanceSDKCopyrightOption.BY_NC;
	}

	public void setContainsAdultContent(boolean containsAdultContent) {
		if(null != currentBehanceProject)
			currentBehanceProject.markProjectForAdultContent(containsAdultContent);
	}
	
	public boolean isProjectContainsAdultContent() {
		if(null != currentBehanceProject)
			return currentBehanceProject.isProjectContainsAdultContent();
		return false;
	}

	public void cancelWorkflow() {
		List<ImageModule> imagesSelectedForProject = getImagesSelectedForProject();
		if(null != imagesSelectedForProject && false == imagesSelectedForProject.isEmpty()) {
			for (ImageModule imageModule : imagesSelectedForProject) {
				if(ProjectModuleTypes.CREATIVECLOUD_ASSET == imageModule.getType()){
					imageModule.deleteFromFileSystem();
				}
			}
		}
		finishWorkflow();

	}

	public void finishWorkflow() {
		clearImagesSelectedForCover();
		removeAllModuleChangeListeners();
		setUserCredentials(null);
		if(null != currentBehanceProject){
			currentBehanceProject.clear();
			currentBehanceProject = null;
		}
	}

	public void removeProjectModule(ProjectModule projectModule) {
		if(null != currentBehanceProject)
			currentBehanceProject.removeProjectModules(projectModule);
		if(projectModule.getType() == ProjectModuleTypes.IMAGE){
				((ImageModule)projectModule).recycleImageBitmap();
		}

		notifyListeners();
	}

	public boolean isEmptyProject() {
		if(null == currentBehanceProject || currentBehanceProject.isEmpty()){
			return true;
		}
		return false;
	}
	
	public void setShareOnFaceBook(boolean shareOnFaceBook) {
		if(null != currentBehanceProject)
			currentBehanceProject.setShareOnFaceBook(shareOnFaceBook);
	}

	public void setShareOnTwitter(boolean shareOnTwitter) {
		if(null != currentBehanceProject)
			currentBehanceProject.setShareOnTwitter(shareOnTwitter);
	}
	
	public boolean getShareOnFaceBook() {
		if(null != currentBehanceProject)
			return currentBehanceProject.getShareOnFaceBook();
		return false;
	}

	public boolean getShareOnTwitter() {
		if(null != currentBehanceProject)
			return currentBehanceProject.getShareOnTwitter();
		return false;
	}

	public Class<? extends Activity> getNotificationHandlerActivityClass() {
		return notificationHandlerActivityClass;
	}

	public void setNotificationHandlerActivityClass(Class<? extends Activity> notificationHandlerActivityClass) {
		this.notificationHandlerActivityClass = notificationHandlerActivityClass;
	}

	public boolean isHideTwitterSharing() {
		return hideTwitterSharing;
	}

	public void setHideTwitterSharing(boolean hideTwitterSharing) {
		this.hideTwitterSharing = hideTwitterSharing;
	}

	public boolean isHideFacebookSharing() {
		return hideFacebookSharing;
	}

	public void setHideFacebookSharing(boolean hideFacebookSharing) {
		this.hideFacebookSharing = hideFacebookSharing;
	}

	public void rotateImage(ImageModule projectModule) {
		projectModule.rotateImage();
		for (ProjectModuleChangeListener listener : projectModuleChangeListeners) {
			listener.rotateImage(projectModule);
		}
	}
	
}
