/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2013 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.behance.sdk.project.modules;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Matrix;
import android.media.ExifInterface;
import android.net.Uri;
import android.os.Environment;
import android.provider.MediaStore;
import android.view.View;
import android.widget.ImageView;

import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.util.BehanceSDKConstants;
import com.behance.sdk.util.BehanceSDKImageLoaderUtils;
import com.nostra13.universalimageloader.core.DisplayImageOptions;
import com.nostra13.universalimageloader.core.assist.FailReason;
import com.nostra13.universalimageloader.core.listener.ImageLoadingListener;

/**
 * @author ugs
 * 
 */
public class ImageModule extends BaseModule {

	private static final long serialVersionUID = 5636447312019606595L;
	private static final ILogger logger = LoggerFactory.getLogger(ImageModule.class);

	private long id = -1;
	private int width;
	private int height;
	private long sizeInBytes;
	private File file;
	private String albumName;
	private transient Bitmap thumbnail;
	private int rotation = 0;
	private boolean defaultThumbnailRotationApplied = false;
	private transient Bitmap bitmap;

	public ImageModule(long id, String filePath) {
		this.id = id;
		setFile(new File(filePath));
		computeImageDimensions();
		getDefaultOrientation();
	}

	public ImageModule(File imageFile) {
		setFile(imageFile);
		computeImageDimensions();
		getDefaultOrientation();
	}

	private void computeImageDimensions() {
		if (file != null && file.exists() && file.isFile()) {
			BitmapFactory.Options options = new BitmapFactory.Options();
			options.inJustDecodeBounds = true;
			BitmapFactory.decodeFile(file.getAbsolutePath(), options);
			setWidth(options.outWidth);
			setHeight(options.outHeight);
			setSizeInBytes(file.length());
		}
	}

	@Override
	public void clear() {
		//clear rotation will recycle the thumbnail also
		clearRotation();
	}

	@Override
	public ProjectModuleTypes getType() {
		return ProjectModuleTypes.IMAGE;
	}

	public long getId() {
		return id;
	}

	public void setId(long id) {
		this.id = id;
	}

	public File getFile() {
		return file;
	}

	private String getFilePath() {
		return file.getAbsolutePath();
	}
	
	
	

	/**
	 * @param file
	 *            the file to set
	 */
	private void setFile(File file) {
		this.file = file;
	}
	
	public Bitmap getThumbNail(Context context) {

		if (null == thumbnail || thumbnail.isRecycled()) {
			Bitmap tempThumbnail = null;
			if (id != -1) {
				tempThumbnail = MediaStore.Images.Thumbnails.getThumbnail(context.getContentResolver(), id, MediaStore.Images.Thumbnails.MINI_KIND, null);
			} else if (null != file) {
				tempThumbnail = getThumbNailFromFile();
			}
			thumbnail = tempThumbnail;
			updateThumbnailForDefaultRotation();
		}
		return thumbnail;
	}

	private void getDefaultOrientation() {
		if (!defaultThumbnailRotationApplied || thumbnail == null) {
				try {
					ExifInterface exif = new ExifInterface(file.getAbsolutePath());
					int exifOrientation = exif.getAttributeInt(ExifInterface.TAG_ORIENTATION, ExifInterface.ORIENTATION_NORMAL);
					switch (exifOrientation) {
					case ExifInterface.ORIENTATION_NORMAL:
						rotation = 0;
						break;
					case ExifInterface.ORIENTATION_ROTATE_90:
						rotation = 1;
						break;
					case ExifInterface.ORIENTATION_ROTATE_180:
						rotation = 2;
						break;
					case ExifInterface.ORIENTATION_ROTATE_270:
						rotation = 3;
						break;
					}
					defaultThumbnailRotationApplied = true;
				} catch (IOException e) {
					logger.error("Exception while getting the default orientation", e);
				}
		}
	}
	
	private void updateThumbnailForDefaultRotation() {
		if(rotation > 0){
			Matrix matrix = new Matrix();
			// setup rotation degree
			matrix.postRotate(90 * rotation);
			Bitmap updatedThumbnail = Bitmap.createBitmap(thumbnail, 0, 0, thumbnail.getWidth(), thumbnail.getHeight(), matrix, true);
			thumbnail.recycle();
			thumbnail = updatedThumbnail;
		}
	}


	private Bitmap updateThumbnailForRotation(Bitmap thumbnailBitmap) {
		Matrix matrix = new Matrix();
		// setup rotation degree
		matrix.postRotate(90);
		Bitmap updatedThumbnail = Bitmap.createBitmap(thumbnailBitmap, 0, 0, thumbnailBitmap.getWidth(), thumbnailBitmap.getHeight(), matrix, true);
		return updatedThumbnail;
	}
	

	private Bitmap getThumbNailFromFile() {
		// Get the dimensions of the View
		int targetW = 400;
		int targetH = 400;

		// Get the dimensions of the bitmap
		BitmapFactory.Options bmOptions = new BitmapFactory.Options();
		bmOptions.inJustDecodeBounds = true;
		String absolutePath = getFilePath();

		BitmapFactory.decodeFile(absolutePath, bmOptions);
		int photoW = bmOptions.outWidth;
		int photoH = bmOptions.outHeight;

		// Determine how much to scale down the image
		int scaleFactor = Math.min(photoW / targetW, photoH / targetH);

		// Decode the image file into a Bitmap sized to fill the View
		bmOptions.inJustDecodeBounds = false;
		bmOptions.inSampleSize = scaleFactor;
		bmOptions.inPurgeable = true;

		Bitmap bitmap = BitmapFactory.decodeFile(absolutePath, bmOptions);
		return bitmap;
	}

	public void rotateImage() {
		if (rotation == 3)
			rotation = 0;
		else
			rotation++;
		Bitmap thumbnailForRotation = updateThumbnailForRotation(thumbnail);
		recycleBitmaps();
		thumbnail = thumbnailForRotation;
	}

	/**
	 * @return
	 */
	public String getName() {
		return file.getName();
	}

	/**
	 * @param albumName
	 */
	public void setAlbumName(String albumName) {
		this.albumName = albumName;
	}

	/**
	 * @param albumName
	 */
	public String getAlbumName() {
		return albumName;

	}

	@Override
	public boolean equals(Object object) {
		if (this == object)
			return true;
		if (null == object || getClass() != object.getClass())
			return false;
		ImageModule objLocalImage = (ImageModule) object;
		return id == objLocalImage.id && equalObject(file, objLocalImage.file) && equalObject(albumName, objLocalImage.albumName)
				&& rotation == objLocalImage.rotation && width == objLocalImage.width && height == objLocalImage.height;
	}

	private boolean equalObject(Object first, Object second) {
		if (first == second) {
			return true;
		} else if ((null != first && null == second) || (null == first && null != second)) {
			return false;
		}
		return first.equals(second);
	}

	public void recycleBitmaps() {
		recycleThumbnailBitmap();
		recycleImageBitmap();

	}
	
	public void recycleImageBitmap() {
		if (null != bitmap && false == bitmap.isRecycled()) {
			bitmap.recycle();
			bitmap = null;
		}
	}

	public void recycleThumbnailBitmap() {
		if (null != thumbnail && false == thumbnail.isRecycled()) {
			thumbnail.recycle();
			thumbnail = null;
		}
	}

	public int getRotation() {
		return rotation;
	}

	public void clearRotation() {
		rotation = 0;
		defaultThumbnailRotationApplied =false;
		getDefaultOrientation();
		recycleBitmaps();
	}
	
	public File getFileForUpload() throws IOException{
		File finalImageFile = file;
		if(rotation > 0){
 			Bitmap decodeFile = BehanceSDKImageLoaderUtils.getImageBitmap(getImageUrl());
 			Bitmap updateBitmapForRotation = BehanceSDKImageLoaderUtils.updateBitmapForRotation(decodeFile, this);
 			finalImageFile = savebitmap(updateBitmapForRotation, file.getName());
 			decodeFile.recycle();
 			updateBitmapForRotation.recycle();
		 }
		 return finalImageFile;
	}

	private File savebitmap(Bitmap bmp, String fileName) throws IOException {
		String extStorageDirectory = Environment.getExternalStorageDirectory()
				.toString();
		OutputStream outStream = null;
		File newFile = new File(extStorageDirectory, fileName + "_temp");
		if (newFile.exists()) {
			newFile.delete();
		}
		outStream = new FileOutputStream(newFile);
		bmp.compress(Bitmap.CompressFormat.PNG, 100, outStream);
		outStream.flush();
		outStream.close();
		return newFile;

	}

	public int getWidth() {
		return width;
	}

	private void setWidth(int width) {
		this.width = width;
	}

	public int getHeight() {
		return height;
	}

	private void setHeight(int height) {
		this.height = height;
	}

	public long getSizeInBytes() {
		return sizeInBytes;
	}

	public void setSizeInBytes(long sizeInBytes) {
		this.sizeInBytes = sizeInBytes;
	}
	
	public void displayImage(final ImageView imageView, final ImageLoadingListener listener) {
		displayImage(imageView, listener, BehanceSDKImageLoaderUtils.getDefaultImageLoaderOptions());
	}

	private String getImageUrl() {
		Uri fromFile = Uri.fromFile(file);
		String imageUrl = Uri.decode(fromFile.toString());
		return imageUrl;
	}

	public void displayImage(final ImageView imageView, final ImageLoadingListener listener, DisplayImageOptions options) {
		
		String imageUrl = getImageUrl();
		if( null == bitmap){
			final ImageModule module =this;
			BehanceSDKImageLoaderUtils.displayImageFromCacheOrLoadFromServer(imageUrl, imageView, new ImageLoadingListener() {
	
				@Override
				public void onLoadingStarted(String arg0, View arg1) {
					if(null != listener)
						listener.onLoadingStarted(arg0, arg1);
				}
				
				@Override
				public void onLoadingFailed(String arg0, View arg1, FailReason arg2) {
					if(null != listener)
						listener.onLoadingFailed(arg0, arg1, arg2);
				}
				
				@Override
				public void onLoadingComplete(String arg0, View arg1, Bitmap loadedBitmap) {
					bitmap = BehanceSDKImageLoaderUtils.updateBitmapForRotation(loadedBitmap, module);
					setImageViewLayerType(imageView, loadedBitmap);
					imageView.setImageBitmap(bitmap);
					if(null != listener)
						listener.onLoadingComplete(arg0, arg1, bitmap);
				}
				
				@Override
				public void onLoadingCancelled(String arg0, View arg1) {
					if(null != listener)
						listener.onLoadingCancelled(arg0, arg1);
					
				}
			}, options);
		}else{
			setImageViewLayerType(imageView, bitmap);
			imageView.setImageBitmap(bitmap);
			if(null != listener)
				listener.onLoadingComplete(imageUrl, imageView, bitmap);
		}

	}
	
	private void setImageViewLayerType(final ImageView imageView, Bitmap loadedBitmap) {
		if (loadedBitmap.getWidth() > BehanceSDKConstants.HARDWARE_RENDERING_MAX_SIZE || 
				loadedBitmap.getHeight() > BehanceSDKConstants.HARDWARE_RENDERING_MAX_SIZE) {
			imageView.setLayerType(View.LAYER_TYPE_SOFTWARE, null);
		} else {
			imageView.setLayerType(View.LAYER_TYPE_HARDWARE, null);
		}
	}
}
