/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property 
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/

package com.behance.sdk.services;

import android.app.*;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Bundle;
import android.os.IBinder;
import android.support.v4.app.NotificationCompat;
import android.support.v4.content.LocalBroadcastManager;
import android.text.TextUtils;
import com.adobe.creativesdk.foundation.internal.analytics.AdobeAnalyticsSDKReporter;
import com.behance.sdk.BehanceSDK;
import com.behance.sdk.BehanceSDKSocialAccountManager;
import com.behance.sdk.IBehanceSDKPublishProjectResultIntentExtras;
import com.behance.sdk.R;
import com.behance.sdk.asynctasks.result.BehanceSDKPublishProjectTaskResult;
import com.behance.sdk.asynctasks.result.BehanceSDKPublishProjectTaskResult.PublishStatus;
import com.behance.sdk.dto.BehanceSDKPublishProjectServiceParamsDTO;
import com.behance.sdk.dto.BehanceSDKPublishProjectStatusDTO;
import com.behance.sdk.dto.BehanceSDKPublishedProjectDTO;
import com.behance.sdk.dto.BehanceSDKSocialAccountDTO;
import com.behance.sdk.enums.BehanceSDKPublishProjectProgressState;
import com.behance.sdk.enums.BehanceSDKSocialAccountType;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.managers.BehanceSDKUserManager;
import com.behance.sdk.network.BehanceConnectionResponse;
import com.behance.sdk.network.BehanceHttpsConnection;
import com.behance.sdk.network.BehanceMultipartEntity;
import com.behance.sdk.network.BehanceMultipartEntityWrapper;
import com.behance.sdk.network.BehanceRequestCancellationWrapper;
import com.behance.sdk.project.modules.EmbedModule;
import com.behance.sdk.project.modules.ImageModule;
import com.behance.sdk.project.modules.ProjectModule;
import com.behance.sdk.project.modules.ProjectModuleTypes;
import com.behance.sdk.services.binders.BehanceSDKPublishProjectBinder;
import com.behance.sdk.util.*;
import com.facebook.*;
import oauth.signpost.OAuthConsumer;
import oauth.signpost.basic.DefaultOAuthConsumer;
import org.json.JSONObject;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.Serializable;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Sujit Reddy G
 * 
 */

public class BehanceSDKPublishProjectService extends IntentService {

	private static final String TWITTER_UPDATE_API_URL = "https://api.twitter.com/1.1/statuses/update.json?status=";

	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKPublishProjectService.class);

	public static final String INTENT_EXTRA_PARAMS = "INTENT_EXTRA_PARAMS";

	private static final String PROJECTID_KEY = "{PROJECTID}";
	private static final String LICENSE_KEY = "license";
	private static final String MODULE_EMBED_KEY = "html";
	private static final String MODULE_TYPE_EMBED_VALUE = "embed";
	private static final String MODULE_TYPE_IMAGE_VALUE = "image";
	private static final String MODULE_IMAGE_KEY = MODULE_TYPE_IMAGE_VALUE;
	private static final String MODULE_TYPE_KEY = "type";
	private static final String DESCRIPTION = "description";
	private static final String TAGS_KEY = "tags";
	private static final String FIELDS_KEY = "fields";
	private static final String TITLE_KEY = "title";
	private static final String MATURE_CONTENT_KEY = "mature_content";
	private static final String PUBLISHED_KEY = "published";
	private static final String PUBLISHED = "1";

    private BehanceSDKPublishProjectBinder projectBinder;
    private String activePublishRequestId;
    private BehanceSDKUserManager sdkUserManager;
	private BehanceSDK behanceSDK;

    public BehanceSDKPublishProjectService() {
		super("Behance Publish Project Service");
	}

    @Override
    public IBinder onBind(Intent intent) {
        if (projectBinder == null) {
            projectBinder = new BehanceSDKPublishProjectBinder();
        }
        return projectBinder;
    }

    @Override
    public boolean onUnbind(Intent intent) {
        return super.onUnbind(intent);
    }

    private boolean isActiveUploadCancelled() {
        if (projectBinder != null) {
            return projectBinder.isUploadCancelled(activePublishRequestId);
        }
        return false;
    }

	@Override
	protected void onHandleIntent(Intent intent) {

        //generate id for this request
        activePublishRequestId = String.valueOf(System.currentTimeMillis());

        sdkUserManager = BehanceSDKUserManager.getInstance();
		behanceSDK = BehanceSDK.getInstance();
        NotificationCompat.Builder mBuilder = new NotificationCompat.Builder(this);
		mBuilder.setSmallIcon(R.drawable.bsdk_anim_list_publish_in_progress_indicator);

		//Allowing other apps to override Behance Title in notification.
		if(behanceSDK.useBehanceTheme())
		mBuilder.setContentTitle(getString(R.string.bsdk_publish_project_service_in_progress_notification_title));
		else {
			mBuilder.setContentTitle(getString(behanceSDK.getBehanceSDKCustomResourcesOptions().getAppTitle()));
			int color = behanceSDK.getBehanceSDKCustomResourcesOptions().getAppColor();
			if(color != 0)
				mBuilder.setColor(getResources().getColor(color));
		}


		mBuilder.setContentText(getString(R.string.bsdk_publish_project_service_in_progress_notification_text));
		mBuilder.setStyle(new NotificationCompat.BigTextStyle().bigText(getString(R.string.bsdk_publish_project_service_in_progress_notification_text)));
		mBuilder.setProgress(0, 0, true);
		Notification notification = mBuilder.build();
		startForeground(BehanceSDKConstants.NOTIFICATION_ID_PUBLISH_PROJECT_IN_PROGRESS, notification);

		Serializable params = intent.getSerializableExtra(INTENT_EXTRA_PARAMS);
		if (params instanceof BehanceSDKPublishProjectServiceParamsDTO) {
			BehanceSDKPublishProjectServiceParamsDTO serviceParamsDTO = (BehanceSDKPublishProjectServiceParamsDTO) params;
			BehanceSDKPublishProjectStatusDTO statusDTO = new BehanceSDKPublishProjectStatusDTO();

            if (isActiveUploadCancelled()) {
                broadcastUploadCancelledStatusUpdate();
                return;
            }
			// publish on server
			BehanceSDKPublishProjectTaskResult publishProjectResult = publishProjectOnServer(serviceParamsDTO, statusDTO);

			// check result and display Notification
			BehanceSDKPublishedProjectDTO publishedProject = publishProjectResult.getPublishedProject();
			Class<? extends Activity> notificationHandlerActivity = serviceParamsDTO.getNotificationHandlerActivity();

			if (publishedProject != null && publishProjectResult.getStatus() == BehanceSDKPublishProjectTaskResult.PublishStatus.SUCCESS) {

				if (!isActiveUploadCancelled() && serviceParamsDTO.isShareOnFacebook()) {
					publishOnFacebook(publishedProject, serviceParamsDTO, statusDTO);
				}

				if (!isActiveUploadCancelled() && serviceParamsDTO.isShareOnTwitter()) {
					publishOnTwitter(serviceParamsDTO, statusDTO);
							
				}

                if(isActiveUploadCancelled()) {
                    handleCancelAfterSuccessfulUpload(publishedProject, serviceParamsDTO);
                    return;
                }

				//Allowing other apps to override Behance title in notification.
				if(behanceSDK.useBehanceTheme()) {
					mBuilder.setContentTitle(getString(R.string.bsdk_publish_project_service_success_notification_title));
					mBuilder.setContentText(getString(R.string.bsdk_publish_project_service_success_notification_text));

				}
				else {
					mBuilder.setContentTitle(getString(behanceSDK.getBehanceSDKCustomResourcesOptions().getAppTitle()));
					mBuilder.setContentText(getString(R.string.bsdk_publish_project_service_success_notification_text_behance));
				}
				mBuilder.setTicker(getString(R.string.bsdk_publish_project_service_success_notification_ticker));
				mBuilder.setStyle(new NotificationCompat.BigTextStyle().bigText(getString(R.string.bsdk_publish_project_service_success_notification_text)));

				if (notificationHandlerActivity != null) {
					Intent notifyIntent = createNotifyIntent(notificationHandlerActivity);
					notifyIntent.putExtra(IBehanceSDKPublishProjectResultIntentExtras.INTENT_BOOL_EXTRA_PUBLISHED_PROJECT_SUCCESSFULLY, true);
					notifyIntent.putExtra(IBehanceSDKPublishProjectResultIntentExtras.INTENT_STR_EXTRA_PUBLISHED_PROJECT_ID,
							publishedProject.getProjectId());
					notifyIntent.putExtra(IBehanceSDKPublishProjectResultIntentExtras.INTENT_STR_EXTRA_PUBLISHED_PROJECT_TITLE,
							publishedProject.getProjectTitle());
					PendingIntent resultPendingIntent = createPendingIntent(notifyIntent);

					mBuilder.setContentIntent(resultPendingIntent);
				}
                statusDTO.setProgressState(BehanceSDKPublishProjectProgressState.PUBLISH_SUCCESSFUL);
                statusDTO.setProgressPercentage(getProgressPercentage(BehanceSDKPublishProjectProgressState.PUBLISH_SUCCESSFUL, serviceParamsDTO));
                broadcastProgressUpdate(statusDTO);

			} else {
                if(isActiveUploadCancelled()) {
                    return;
                }

				//Allowing other apps to override Behance Title in notification.
				if(behanceSDK.useBehanceTheme())
				mBuilder.setContentTitle(getString(R.string.bsdk_publish_project_service_failure_notification_title));
				else
					mBuilder.setContentTitle(getString(behanceSDK.getBehanceSDKCustomResourcesOptions().getAppTitle()));


				mBuilder.setTicker(getString(R.string.bsdk_publish_project_service_failure_notification_ticker));
				String msgToDisplay = "";
				String reasonPhrase = publishProjectResult.getReasonPhrase();
				if (reasonPhrase != null) {
					msgToDisplay = getString(R.string.bsdk_publish_project_service_failure_notification_msg, reasonPhrase);
				} else {
					msgToDisplay = getString(R.string.bsdk_publish_project_service_unknown_failure_notification_msg);
				}
				mBuilder.setContentText(msgToDisplay);
				mBuilder.setStyle(new NotificationCompat.BigTextStyle().bigText(msgToDisplay));

				if (notificationHandlerActivity != null) {
					Intent notifyIntent = createNotifyIntent(notificationHandlerActivity);
					notifyIntent.putExtra(IBehanceSDKPublishProjectResultIntentExtras.INTENT_BOOL_EXTRA_PUBLISHED_PROJECT_SUCCESSFULLY, false);
					notifyIntent.putExtra(IBehanceSDKPublishProjectResultIntentExtras.INTENT_STR_EXTRA_PUBLISHED_PROJECT_ID,
							publishedProject.getProjectId());
					notifyIntent.putExtra(IBehanceSDKPublishProjectResultIntentExtras.INTENT_STR_EXTRA_PUBLISHED_PROJECT_TITLE,
							publishedProject.getProjectTitle());
					notifyIntent.putExtra(IBehanceSDKPublishProjectResultIntentExtras.INTENT_STR_EXTRA_PUBLISH_PROJECT_FAILURE_MSG, msgToDisplay);
					PendingIntent resultPendingIntent = createPendingIntent(notifyIntent);

					mBuilder.setContentIntent(resultPendingIntent);
				}
			}
			if(!isActiveUploadCancelled()) {


				//Allowing other apps to override Behance Icon in notification.
				if(behanceSDK.useBehanceTheme()){
					mBuilder.setSmallIcon(R.drawable.bsdk_icon_notification_publish_progress);
				} else{
					mBuilder.setSmallIcon(behanceSDK.getBehanceSDKCustomResourcesOptions().getAppSmallIcon());
				}
                mBuilder.setProgress(0, 0, false);
                mBuilder.setAutoCancel(true);
                notification = mBuilder.build();
                NotificationManager mNotificationManager = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);
                mNotificationManager.notify(BehanceSDKConstants.NOTIFICATION_ID_PUBLISH_PROJECT_COMPLETE, notification);
            }
		}
	}

    @Override
    public void onDestroy() {
        super.onDestroy();
        projectBinder = null;
    }

    private PendingIntent createPendingIntent(Intent notifyIntent) {
		return PendingIntent.getActivity(this, 0, notifyIntent, PendingIntent.FLAG_UPDATE_CURRENT);
	}

	private Intent createNotifyIntent(Class<? extends Activity> notificationHandlerActivity) {
		Intent notifyIntent = new Intent(this, notificationHandlerActivity);
		notifyIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
		return notifyIntent;
	}

	private BehanceSDKPublishProjectTaskResult publishProjectOnServer(BehanceSDKPublishProjectServiceParamsDTO serviceParams,
								BehanceSDKPublishProjectStatusDTO statusDTO) {
		
		BehanceSDKPublishProjectTaskResult result = new BehanceSDKPublishProjectTaskResult();
		BehanceSDKPublishedProjectDTO publishedProjectDTO = new BehanceSDKPublishedProjectDTO();
		String projectTitle = serviceParams.getProjectTitle();
		publishedProjectDTO.setProjectTitle(projectTitle);
		result.setPublishedProject(publishedProjectDTO);

        List<ProjectModule> projectModules = serviceParams.getProjectModules();

		//Send Status PUBLISH_STARTED
		statusDTO.setProjectTitle(projectTitle);
		statusDTO.setProgressState(BehanceSDKPublishProjectProgressState.PUBLISH_STARTED);
		statusDTO.setProgressPercentage(getProgressPercentage(BehanceSDKPublishProjectProgressState.PUBLISH_STARTED, serviceParams));
		broadcastProgressUpdate(statusDTO);

		String projectId = null;
		Map<String, String> urlParams = new HashMap<String, String>();
		urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, serviceParams.getAppClientId());
		try {
			String createProjectDraftUrl = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.PROJECT_CREATE_URL, urlParams);

            BehanceSDKUserManager sdkUserManager = BehanceSDKUserManager.getInstance();
            String userAccessToken = sdkUserManager.checkExpiryAndGetAccessToken();
			if (userAccessToken != null) {
				createProjectDraftUrl = BehanceSDKUrlUtil.appendQueryStringParam(createProjectDraftUrl, BehanceSDKUrlUtil.PARAM_KEY_ACCESS_TOKEN,
						userAccessToken);
			}

			BehanceMultipartEntity createProjectDraftEntity = new BehanceMultipartEntity();
			createProjectDraftEntity.setChunked(true);
			ByteArrayInputStream coverImageInputStream = new ByteArrayInputStream(serviceParams.getProjectCoverImageByteArray());
			createProjectDraftEntity.addImagePart(MODULE_IMAGE_KEY, serviceParams.getProjectCoverImageFileName(),
					BehanceMultipartEntity.CONTENT_TYPE_APPLICATION_OCTETSTREAM, BehanceMultipartEntity.CONTENT_TRANSFER_BINARY, coverImageInputStream);

			createProjectDraftEntity.addTextPart(TITLE_KEY, BehanceMultipartEntity.CONTENT_TYPE_TEXT_UTF8,
					BehanceMultipartEntity.CONTENT_TRANSFER_8BIT, projectTitle.getBytes(BehanceMultipartEntity.CHARSET_UTF8));

			BehanceConnectionResponse<String> createProjectDraftHttpResponse = BehanceHttpsConnection.getInstance().invokeHttpPostRequest(createProjectDraftUrl,createProjectDraftEntity,null);

			if (createProjectDraftHttpResponse.getResponseCode() != BehanceHttpsConnection.SC_CREATED) {
				//Send Status PUBLISH_FAILED
				statusDTO.setProgressState(BehanceSDKPublishProjectProgressState.PUBLISH_FAILED);
				statusDTO.setReasonPhrase(createProjectDraftHttpResponse.getResponseObject());
				broadcastProgressUpdate(statusDTO);

				cleanupCreativeCloudResources(projectModules);
				result.setStatus(PublishStatus.CREATION_FAILED);
				result.setReasonPhrase(createProjectDraftHttpResponse.getResponseObject());
				return result;
			}

			// get Project details from response and set to DTO
			String draftProjectResponseJsonString = createProjectDraftHttpResponse.getResponseObject();
			JSONObject draftProjectResponseRootJsonObject = new JSONObject(draftProjectResponseJsonString);
			JSONObject draftProjectJsonObject = (JSONObject) draftProjectResponseRootJsonObject.get("project");
			projectId = draftProjectJsonObject.optString("id");
			publishedProjectDTO.setProjectId(projectId);
			String projectUrl = draftProjectJsonObject.optString("url");
			publishedProjectDTO.setProjectUrl(projectUrl);

			statusDTO.setProjectId(projectId);
			statusDTO.setProjectUrl(projectUrl);

            if(isActiveUploadCancelled()) {
                handleCancelAfterSuccessfulUpload(publishedProjectDTO, serviceParams);
                result.setStatus(PublishStatus.CANCELLED);
                return result;
            }

            statusDTO.setProgressState(BehanceSDKPublishProjectProgressState.PROJECT_DRAFT_CREATION_SUCCESSFUL);
            statusDTO.setProgressPercentage(getProgressPercentage(BehanceSDKPublishProjectProgressState.PROJECT_DRAFT_CREATION_SUCCESSFUL, serviceParams));
            broadcastProgressUpdate(statusDTO);

			JSONObject coverImageJsonObject = draftProjectJsonObject.optJSONObject("covers");
			if (coverImageJsonObject != null) {
                String smallCoverImageUrl = coverImageJsonObject.optString("202");
                String bigCoverImageUrl = coverImageJsonObject.optString("404");
                publishedProjectDTO.setProjectImageUrl(smallCoverImageUrl);
                if(bigCoverImageUrl != null && bigCoverImageUrl.length() > 0) {
                    statusDTO.setProjectCoverImageUrl(bigCoverImageUrl);
                } else {
                    statusDTO.setProjectCoverImageUrl(smallCoverImageUrl);
                }
			}

			String finalAddModuleUrl = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.ADD_PROJECT_MODULE_URL, urlParams);
			finalAddModuleUrl = finalAddModuleUrl.replace(PROJECTID_KEY, projectId);
			if (userAccessToken != null) {
				finalAddModuleUrl = BehanceSDKUrlUtil.appendQueryStringParam(finalAddModuleUrl, BehanceSDKUrlUtil.PARAM_KEY_ACCESS_TOKEN,
						userAccessToken);
			}

			// upload Project modules
            int currentUploadingModuleCount = 0;
			for (ProjectModule projectModule : projectModules) {
                if(isActiveUploadCancelled()) {

                    handleCancelAfterSuccessfulUpload(publishedProjectDTO, serviceParams);
                    result.setStatus(PublishStatus.CANCELLED);
                    return result;
                }
                currentUploadingModuleCount++;
				ProjectModuleTypes moduleType = projectModule.getType();

				BehanceMultipartEntity uploadModuleEntity = new BehanceMultipartEntity();
				uploadModuleEntity.setChunked(true);
				File imageFile = null;
				if (moduleType == ProjectModuleTypes.IMAGE || moduleType == ProjectModuleTypes.CREATIVECLOUD_ASSET) {
					uploadModuleEntity.addTextPart(MODULE_TYPE_KEY, BehanceMultipartEntity.CONTENT_TYPE_TEXT_UTF8,
							BehanceMultipartEntity.CONTENT_TRANSFER_8BIT, new String(MODULE_TYPE_IMAGE_VALUE).getBytes(BehanceMultipartEntity.CHARSET_ISO_8859));
					imageFile = ((ImageModule) projectModule).getFileForUpload();

					uploadModuleEntity.addImagePart(MODULE_IMAGE_KEY, imageFile.getName(), BehanceMultipartEntity.CONTENT_TYPE_APPLICATION_OCTETSTREAM,
							BehanceMultipartEntity.CONTENT_TRANSFER_BINARY, imageFile.getAbsolutePath());

				} else if (moduleType == ProjectModuleTypes.EMBED) {
					uploadModuleEntity.addTextPart(MODULE_TYPE_KEY, BehanceMultipartEntity.CONTENT_TYPE_TEXT_ISO_8859, BehanceMultipartEntity.CONTENT_TRANSFER_8BIT,
							new String(MODULE_TYPE_EMBED_VALUE).getBytes(BehanceMultipartEntity.CHARSET_ISO_8859));

					String embedContent = ((EmbedModule) projectModule).getEmbedContent();
					uploadModuleEntity.addTextPart(MODULE_EMBED_KEY, BehanceMultipartEntity.CONTENT_TYPE_APPLICATION_XHTML,
							BehanceMultipartEntity.CONTENT_TRANSFER_8BIT, embedContent.getBytes(BehanceMultipartEntity.CHARSET_ISO_8859));
				}

				BehanceRequestCancellationWrapper cancellationWrapper = new BehanceRequestCancellationWrapper();
				ProjectModuleUploadEntityWrapperCallback wrapper = new ProjectModuleUploadEntityWrapperCallback(
						serviceParams, statusDTO, cancellationWrapper, currentUploadingModuleCount);
				uploadModuleEntity.setBehanceHttpEntityWrapperCallback(wrapper);

				BehanceConnectionResponse<String> uploadModuleHttpResponse = BehanceHttpsConnection.getInstance().invokeHttpPostRequest(finalAddModuleUrl, uploadModuleEntity, cancellationWrapper);


				if (moduleType == ProjectModuleTypes.IMAGE && imageFile.equals(((ImageModule) projectModule).getFile()) == false) {
					imageFile.delete();
				} else if(moduleType == ProjectModuleTypes.CREATIVECLOUD_ASSET){
					projectModule.deleteFromFileSystem();
				}
                if(isActiveUploadCancelled()) {
                    handleCancelAfterSuccessfulUpload(publishedProjectDTO, serviceParams);
                    result.setStatus(PublishStatus.CANCELLED);
                    return result;
                }
				if (uploadModuleHttpResponse.getResponseCode() != BehanceHttpsConnection.SC_CREATED) {
					deleteProject(projectId, serviceParams, urlParams);
					cleanupCreativeCloudResources(projectModules);
					//Send Status MODULE_ADD_FAILED
					statusDTO.setProgressState(BehanceSDKPublishProjectProgressState.PUBLISH_FAILED);
					statusDTO.setReasonPhrase(uploadModuleHttpResponse.getResponseObject());
					broadcastProgressUpdate(statusDTO);

					result.setStatus(PublishStatus.MODULE_ADD_FAILED);
					result.setFailedMoudle(projectModule);
					result.setReasonPhrase(uploadModuleHttpResponse.getResponseObject());
					return result;
				}
			}

			// publish project details
			String finalPublishUrl = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.PUBLISH_PROJECT_URL, urlParams);
			finalPublishUrl = finalPublishUrl.replace(PROJECTID_KEY, projectId);
			if (userAccessToken != null) {
				finalPublishUrl = BehanceSDKUrlUtil
						.appendQueryStringParam(finalPublishUrl, BehanceSDKUrlUtil.PARAM_KEY_ACCESS_TOKEN, userAccessToken);
			}

			BehanceMultipartEntity publishProjectEntity = new BehanceMultipartEntity();
			publishProjectEntity.setBrowserCompatible(true);
			publishProjectEntity.addTextPart(TITLE_KEY, null, null, projectTitle.getBytes());

			String projectFields = serviceParams.getProjectCreativeFields();
			if (false == TextUtils.isEmpty(projectFields)) {
				publishProjectEntity.addTextPart(FIELDS_KEY, null, null, projectFields.getBytes());
			}
			String projectTags = serviceParams.getProjectTags();
			if (false == TextUtils.isEmpty(projectTags)) {
				publishProjectEntity.addTextPart(TAGS_KEY, null, null, projectTags.getBytes());
			}

			publishProjectEntity.addTextPart(DESCRIPTION, null, null, serviceParams.getProjectDescription().getBytes());
			publishProjectEntity.addTextPart(LICENSE_KEY, null, null, serviceParams.getProjectCopyright().getBytes());


			String containsAdultContentStr = "";
			if (serviceParams.isProjectContainsAdultContent()) {
				containsAdultContentStr = "1";
			} else {
				containsAdultContentStr = "0";
			}
			publishProjectEntity.addTextPart(MATURE_CONTENT_KEY, null, null, containsAdultContentStr.getBytes());
			publishProjectEntity.addTextPart(PUBLISHED_KEY, null, null, PUBLISHED.getBytes());

			BehanceConnectionResponse<String> publishProjectDetailsHttpResponse = BehanceHttpsConnection.getInstance().invokeHttpPutRequest(finalPublishUrl, publishProjectEntity, null);


			if (publishProjectDetailsHttpResponse.getResponseCode() == BehanceHttpsConnection.SC_OK) {
				result.setStatus(PublishStatus.SUCCESS);

				statusDTO.setProgressState(BehanceSDKPublishProjectProgressState.MODULES_UPLOAD_SUCCESSFUL);
                statusDTO.setProgressPercentage(
                        getProgressPercentage(BehanceSDKPublishProjectProgressState.MODULES_UPLOAD_SUCCESSFUL, serviceParams));
				broadcastProgressUpdate(statusDTO);
			} else {
				deleteProject(projectId, serviceParams, urlParams);
				//Send Status PUBLISH_FAILED
				statusDTO.setProgressState(BehanceSDKPublishProjectProgressState.PUBLISH_FAILED);
				statusDTO.setReasonPhrase(publishProjectDetailsHttpResponse.getResponseObject());
				broadcastProgressUpdate(statusDTO);

				result.setStatus(PublishStatus.PUBLISH_FAILED);
				result.setReasonPhrase(publishProjectDetailsHttpResponse.getResponseObject());
			}
		} catch (Throwable e) {
            logger.error(e, "Problem publishing project");
			cleanupCreativeCloudResources(projectModules);
            if(isActiveUploadCancelled()) {
                handleCancelAfterSuccessfulUpload(publishedProjectDTO, serviceParams);
                result.setStatus(PublishStatus.CANCELLED);
                return result;
            } else {
                if (TextUtils.isEmpty(projectId) == false)
                    deleteProject(projectId, serviceParams, urlParams);
                //Send Status EXCEPTION
                statusDTO.setProgressState(BehanceSDKPublishProjectProgressState.PUBLISH_FAILED);
                statusDTO.setException(e);
                broadcastProgressUpdate(statusDTO);

                result.setStatus(PublishStatus.EXCEPTION);
                result.setException(e);
            }
		}
		return result;
	}

	private void cleanupCreativeCloudResources(List<ProjectModule> projectModules){
		for(ProjectModule projectModule : projectModules){
			if(projectModule.getType() == ProjectModuleTypes.CREATIVECLOUD_ASSET){
				projectModule.deleteFromFileSystem();
			}
		}
	}

	private void publishOnFacebook(BehanceSDKPublishedProjectDTO publishedProject, BehanceSDKPublishProjectServiceParamsDTO serviceParamsDTO,
                                   BehanceSDKPublishProjectStatusDTO statusDTO) {
		try {
			AccessToken session = AccessToken.getCurrentAccessToken();
			if (session != null) {
				Bundle postParams = new Bundle();
				postParams.putString("name", publishedProject.getProjectTitle());
				postParams.putString("link", publishedProject.getProjectUrl());
				String projectImageUrl = publishedProject.getProjectImageUrl();
				// We will add the image only we have a valid link
				if (projectImageUrl != null && projectImageUrl.length() > 0) {
					postParams.putString("picture", projectImageUrl);
				}
				GraphRequest request = new GraphRequest(session, "me/feed", postParams, HttpMethod.POST, null);
				GraphResponse response = request.executeAndWait();
				FacebookRequestError error = response.getError();
				if (error != null) {
					// toString() display app client id along with other info,
					// we don't want to log this in release
					logger.debug("Problem sharing Project on Facebook [Message - %s]", error.toString());
					String errorUserMessage = error.getErrorUserMessage();
					logger.error("Problem sharing Project on Facebook [Message - %s]", errorUserMessage);
					statusDTO.setProgressState(BehanceSDKPublishProjectProgressState.SHARE_ON_FACEBOOK_FAILED);
					statusDTO.setReasonPhrase(errorUserMessage);
				} else {
					logger.debug("Successfully shared Project on Facebook");
					statusDTO.setProgressState(BehanceSDKPublishProjectProgressState.SHARE_ON_FACEBOOK_SUCCESSFUL);
				}
			}
		} catch (Throwable e) {
			logger.error(e, "Problem sharing Project on Facebook");
			statusDTO.setProgressState(BehanceSDKPublishProjectProgressState.SHARE_ON_FACEBOOK_FAILED);
			statusDTO.setException(e);
		}
        statusDTO.setProgressPercentage(getProgressPercentage(statusDTO.getProgressState(), serviceParamsDTO));
		broadcastProgressUpdate(statusDTO);
	}

	private void publishOnTwitter(BehanceSDKPublishProjectServiceParamsDTO serviceParamsDTO, BehanceSDKPublishProjectStatusDTO statusDTO ){
		
		try {
			// Create a new consumer using the commons implementation
			OAuthConsumer consumer = new DefaultOAuthConsumer(serviceParamsDTO.getTwitterConsumerKey(), serviceParamsDTO.getTwitterConsumerSecret());
			consumer.setTokenWithSecret(serviceParamsDTO.getTwitterUserAccessToken(), serviceParamsDTO.getTwitterUserAccessTokenSecret());
			String queryString = getString(R.string.bsdk_publish_project_service_publish_on_twitter_msg, statusDTO.getProjectUrl());
			String encodedQueryString = URLEncoder.encode(queryString, "UTF-8");
			String finalURL = TWITTER_UPDATE_API_URL + encodedQueryString;

			BehanceConnectionResponse<String> response = BehanceHttpsConnection.getInstance().invokeHttpPostRequest(finalURL,null,null,consumer);
			logger.debug("Share on Twitter URL is [URL - %s]", finalURL);

			int statusCode = response.getResponseCode();
			if (statusCode == BehanceHttpsConnection.SC_OK) {
				logger.debug("Successfully shared Project on Twitter");
				statusDTO.setProgressState(BehanceSDKPublishProjectProgressState.SHARE_ON_TWITTER_SUCCESSFUL);
			} else if (statusCode == BehanceHttpsConnection.SC_UNAUTHORIZED) {
				String reasonPhrase = response.getResponseObject();
				logger.error("Twitter User token has been revoked. [Error code - %s - message - %s]", statusCode, reasonPhrase);
				unlinkTwitterAccount();
				statusDTO.setProgressState(BehanceSDKPublishProjectProgressState.SHARE_ON_TWITTER_FAILED);
				statusDTO.setReasonPhrase(reasonPhrase);
			} else {
				String reasonPhrase = response.getResponseObject();
				logger.error("Error sharing Project on Twitter. [Error code - %s - message - %s]", statusCode, reasonPhrase);
				statusDTO.setProgressState(BehanceSDKPublishProjectProgressState.SHARE_ON_TWITTER_FAILED);
				statusDTO.setReasonPhrase(reasonPhrase);
			}

		} catch (Throwable e) {
			logger.error(e, "Error sharing Project on Twitter");
			statusDTO.setProgressState(BehanceSDKPublishProjectProgressState.SHARE_ON_TWITTER_FAILED);
			statusDTO.setException(e);
		}
        statusDTO.setProgressPercentage(getProgressPercentage(statusDTO.getProgressState(), serviceParamsDTO));
		broadcastProgressUpdate(statusDTO);
	}

	private void unlinkTwitterAccount() {
		BehanceSDKSocialAccountManager socialAccountManager = BehanceSDKSocialAccountManager.getInstance(this);
		BehanceSDKSocialAccountDTO twitterAccount = socialAccountManager.getAccount(BehanceSDKSocialAccountType.TWITTER);
		if (twitterAccount != null) {
			socialAccountManager.removeAccount(BehanceSDKSocialAccountType.TWITTER);
		}
	}

    private void broadcastUploadCancelledStatusUpdate() {
        logger.debug("Publish Project cancelled. Broadcasting message");
        BehanceSDKPublishProjectStatusDTO statusDto = new BehanceSDKPublishProjectStatusDTO();
        statusDto.setPublishRequestId(activePublishRequestId);
        statusDto.setProgressState(BehanceSDKPublishProjectProgressState.CANCEL_PUBLISH_SUCCESSFUL);
        statusDto.setProgressPercentage(getProgressPercentage(BehanceSDKPublishProjectProgressState.CANCEL_PUBLISH_SUCCESSFUL, null));
        Intent broadcastIntent = new Intent(BehanceSDKPublishProjectStatusDTO.INTENT_ACTION_PROJECT_PUBLISH_STATUS_BROADCAST);
        broadcastIntent.putExtra(BehanceSDKPublishProjectStatusDTO.INTENT_EXTRA_OBJ_KEY_PROJECT_PUBLISH_STATUS_BROADCAST_DATA, statusDto);
        LocalBroadcastManager.getInstance(this).sendBroadcast(broadcastIntent);
    }

    private void broadcastProgressUpdate(BehanceSDKPublishProjectStatusDTO statusDto) {
        if (!isActiveUploadCancelled()) {
            // analytics tracking
            if(statusDto.getProgressState() == BehanceSDKPublishProjectProgressState.PUBLISH_SUCCESSFUL) {
                AdobeAnalyticsSDKReporter.trackSharingAction(AdobeAnalyticsSDKReporter.AdobeAnalyticsShareTypePublishSuccess,
                        AdobeAnalyticsSDKReporter.AdobeAnalyticsShareTargetBehanceProject);
            } else if (statusDto.getProgressState() == BehanceSDKPublishProjectProgressState.PUBLISH_FAILED) {
                AdobeAnalyticsSDKReporter.trackSharingAction(AdobeAnalyticsSDKReporter.AdobeAnalyticsShareTypePublishFailure,
                        AdobeAnalyticsSDKReporter.AdobeAnalyticsShareTargetBehanceProject);
            }
            Intent localIntent = new Intent(BehanceSDKPublishProjectStatusDTO.INTENT_ACTION_PROJECT_PUBLISH_STATUS_BROADCAST);
            localIntent.putExtra(BehanceSDKPublishProjectStatusDTO.INTENT_EXTRA_OBJ_KEY_PROJECT_PUBLISH_STATUS_BROADCAST_DATA, statusDto);
            statusDto.setPublishRequestId(activePublishRequestId);

            // Broadcasts the Intent to receivers in this app.
            LocalBroadcastManager.getInstance(this).sendBroadcast(localIntent);
        }
    }

    private void deleteProject(String projectId, BehanceSDKPublishProjectServiceParamsDTO projectPublishAsyncTaskParams, Map<String, String> urlParams) {
        try {
            String url = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.PROJECT_DELETE_URL, urlParams);
            url = url.replace(PROJECTID_KEY, projectId);
            String userAccessToken = sdkUserManager.checkExpiryAndGetAccessToken();
            if (userAccessToken != null) {
                url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_ACCESS_TOKEN,
                        userAccessToken);
            }
			BehanceHttpsConnection.getInstance().invokeHttpDeleteRequest(url);
        } catch (Throwable e) {
            logger.error(e, "Problem deleting project");
        }
    }

    private void handleCancelAfterSuccessfulUpload(BehanceSDKPublishedProjectDTO publishedProject, BehanceSDKPublishProjectServiceParamsDTO serviceParams) {
        BehanceSDKPublishProjectStatusDTO statusDto = new BehanceSDKPublishProjectStatusDTO();
        statusDto.setPublishRequestId(activePublishRequestId);
        statusDto.setProgressState(BehanceSDKPublishProjectProgressState.CANCEL_PUBLISH_SUCCESSFUL);
        statusDto.setProgressPercentage(getProgressPercentage(BehanceSDKPublishProjectProgressState.CANCEL_PUBLISH_SUCCESSFUL, serviceParams));
        try {
			//cleanup resources downloaded from creative cloud
			List<ProjectModule> projectModules = serviceParams.getProjectModules();
			cleanupCreativeCloudResources(projectModules);

            Map<String, String> urlParams = new HashMap<String, String>();
            urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, serviceParams.getAppClientId());
            String url = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.PROJECT_DELETE_URL, urlParams);
            url = url.replace(PROJECTID_KEY, publishedProject.getProjectId());

            String userAccessToken = sdkUserManager.checkExpiryAndGetAccessToken();
            if (userAccessToken != null) {
                url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_ACCESS_TOKEN, userAccessToken);
            }
			BehanceHttpsConnection.getInstance().invokeHttpDeleteRequest(url);
        } catch (Throwable e) {
            logger.error(e, "Problem deleting project after cancel request");
            statusDto.setProgressState(BehanceSDKPublishProjectProgressState.CANCEL_PUBLISH_FAILED);
            statusDto.setException(e);
        }
        Intent broadcastIntent = new Intent(BehanceSDKPublishProjectStatusDTO.INTENT_ACTION_PROJECT_PUBLISH_STATUS_BROADCAST);
        broadcastIntent.putExtra(BehanceSDKPublishProjectStatusDTO.INTENT_EXTRA_OBJ_KEY_PROJECT_PUBLISH_STATUS_BROADCAST_DATA, statusDto);
        LocalBroadcastManager.getInstance(this).sendBroadcast(broadcastIntent);
    }

    private int convertModuleUploadProgressPercentage(BehanceSDKPublishProjectServiceParamsDTO paramsDTO,
                                                      int uploadedModuleCount, int moduleProgressPercentage) {
        int startPercentage = getProgressPercentage(BehanceSDKPublishProjectProgressState.PROJECT_DRAFT_CREATION_SUCCESSFUL, paramsDTO);
        int endPercentage = getProgressPercentage(BehanceSDKPublishProjectProgressState.MODULES_UPLOAD_SUCCESSFUL, paramsDTO);
        int availablePercentage = endPercentage - startPercentage;
        int availablePercentPerModule = availablePercentage / paramsDTO.getProjectModules().size();
        int response = startPercentage + ((uploadedModuleCount - 1) * availablePercentPerModule) +
                ((moduleProgressPercentage * availablePercentPerModule) / 100);
        return response;
    }

    private int getProgressPercentage(BehanceSDKPublishProjectProgressState progressState,
                                      BehanceSDKPublishProjectServiceParamsDTO paramsDTO) {
        int response = 0;
        switch (progressState) {
            case PUBLISH_STARTED: {
                response = 10;
                break;
            }
            case PROJECT_DRAFT_CREATION_SUCCESSFUL: {
                response = 20;
                break;
            }
            case MODULES_UPLOAD_SUCCESSFUL: {
                if (paramsDTO.isShareOnFacebook() && paramsDTO.isShareOnTwitter()) {
                    response = 70;
                } else if (paramsDTO.isShareOnFacebook() || paramsDTO.isShareOnTwitter()) {
                    response = 80;
                } else {
                    response = 90;
                }
                break;
            }
            case SHARE_ON_FACEBOOK_SUCCESSFUL:
            case SHARE_ON_FACEBOOK_FAILED: {
                if (paramsDTO.isShareOnTwitter()) {
                    response = 80;
                } else {
                    response = 90;
                }
                break;
            }
            case SHARE_ON_TWITTER_SUCCESSFUL:
            case SHARE_ON_TWITTER_FAILED: {
                response = 90;
                break;
            }
            case PUBLISH_FAILED:
            case CANCEL_PUBLISH_SUCCESSFUL:
            case CANCEL_PUBLISH_FAILED:
            case PUBLISH_SUCCESSFUL: {
                response = 100;
                break;
            }
        }
        return response;
    }

    private class ProjectModuleUploadEntityWrapperCallback implements BehanceMultipartEntityWrapper {
        private final BehanceSDKPublishProjectServiceParamsDTO serviceParams;
        private final int uploadedModulesCount;
		private final BehanceRequestCancellationWrapper cancellationWrapper;
        private final BehanceSDKPublishProjectStatusDTO statusDTO;
        private int lastBroadcastedProgressPercentage = 0;

        private ProjectModuleUploadEntityWrapperCallback(BehanceSDKPublishProjectServiceParamsDTO serviceParams,
                                                         BehanceSDKPublishProjectStatusDTO statusDTO,
														 BehanceRequestCancellationWrapper cancellationWrapper, int uploadedModulesCount) {
            this.serviceParams = serviceParams;
            this.uploadedModulesCount = uploadedModulesCount;
            this.cancellationWrapper = cancellationWrapper;
            this.statusDTO = statusDTO;
        }

        @Override
        public void onUploadProgressUpdate(float progressPercentage) {
            int progressPercentInt = (int) progressPercentage;
            if ((progressPercentInt % 10) == 0) {
                if (lastBroadcastedProgressPercentage != progressPercentInt) {
                    lastBroadcastedProgressPercentage = progressPercentInt;
                    // we will notify 100 percent when parsing is done
                    if (progressPercentInt < 100) {
                        statusDTO.setProgressState(BehanceSDKPublishProjectProgressState.MODULES_UPLOAD_IN_PROGRESS);
                        statusDTO.setProgressPercentage(convertModuleUploadProgressPercentage(serviceParams, uploadedModulesCount, lastBroadcastedProgressPercentage));
                        broadcastProgressUpdate(statusDTO);

                        if (isActiveUploadCancelled()) {
							cancellationWrapper.setCancelled();
                        }
                    }
                }
            }
        }
    }
}