/**
 * **********************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 * <p/>
 * Copyright 2014 Adobe Systems Incorporated
 * All Rights Reserved.
 * <p/>
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 * ************************************************************************
 */

package com.behance.sdk.services;

import android.app.*;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Bundle;
import android.os.IBinder;
import android.support.v4.app.NotificationCompat;
import android.support.v4.content.LocalBroadcastManager;
import com.adobe.creativesdk.foundation.internal.analytics.AdobeAnalyticsSDKReporter;
import com.behance.sdk.BehanceSDK;
import com.behance.sdk.BehanceSDKSocialAccountManager;
import com.behance.sdk.IBehanceSDKPublishWIPResultIntentExtras;
import com.behance.sdk.R;
import com.behance.sdk.asynctasks.BehanceSDKPublishWIPResult;
import com.behance.sdk.asynctasks.BehanceSDKPublishWIPResult.PublishStatus;
import com.behance.sdk.dto.BehanceSDKPublishWIPServiceParamsDTO;
import com.behance.sdk.dto.BehanceSDKPublishWIPStatusDTO;
import com.behance.sdk.dto.BehanceSDKPublishedWIPRevisionDTO;
import com.behance.sdk.dto.BehanceSDKSocialAccountDTO;
import com.behance.sdk.enums.BehanceSDKPublishWIPProgressState;
import com.behance.sdk.enums.BehanceSDKSocialAccountType;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.managers.BehanceSDKUserManager;
import com.behance.sdk.network.BehanceConnectionResponse;
import com.behance.sdk.network.BehanceHttpsConnection;
import com.behance.sdk.network.BehanceMultipartEntity;
import com.behance.sdk.network.BehanceMultipartEntityWrapper;
import com.behance.sdk.network.BehanceRequestCancellationWrapper;
import com.behance.sdk.project.modules.ProjectModule;
import com.behance.sdk.project.modules.ProjectModuleTypes;
import com.behance.sdk.services.binders.BehanceSDKPublishWIPBinder;
import com.behance.sdk.util.BehanceSDKConstants;
import com.behance.sdk.util.BehanceSDKUrlUtil;
import com.facebook.*;
import oauth.signpost.OAuthConsumer;
import oauth.signpost.basic.DefaultOAuthConsumer;

import org.json.JSONArray;
import org.json.JSONObject;

import java.io.Serializable;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Sujit Reddy G
 *
 */

public class BehanceSDKPublishWIPService extends IntentService {

    private static final String TWITTER_UPDATE_API_URL = "https://api.twitter.com/1.1/statuses/update.json?status=";

    private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKPublishWIPService.class);

    public static final String INTENT_EXTRA_PARAMS = "INTENT_EXTRA_PARAMS";

    private BehanceSDKPublishWIPBinder wipBinder;
    private String activePublishRequestId;
    private BehanceSDKUserManager sdkUserManager;
    private BehanceSDK behanceSDK;


    public BehanceSDKPublishWIPService() {
        super("Behance Publish Work in Progress Service");
    }

    @Override
    public IBinder onBind(Intent intent) {
        if (wipBinder == null) {
            wipBinder = new BehanceSDKPublishWIPBinder();
        }
        return wipBinder;
    }

    @Override
    public boolean onUnbind(Intent intent) {
        return super.onUnbind(intent);
    }

    private boolean isActiveUploadCancelled() {
        if (wipBinder != null) {
            return wipBinder.isUploadCancelled(activePublishRequestId);
        }
        return false;
    }

    @Override
    protected void onHandleIntent(Intent intent) {

        //generate id for this request
        activePublishRequestId = String.valueOf(System.currentTimeMillis());
        behanceSDK = BehanceSDK.getInstance();

        sdkUserManager = BehanceSDKUserManager.getInstance();

        NotificationCompat.Builder mBuilder = new NotificationCompat.Builder(this);
        mBuilder.setSmallIcon(R.drawable.bsdk_anim_list_publish_in_progress_indicator);


        //Allowing other apps to override Behance Title in notification.
        if(behanceSDK.useBehanceTheme())
            mBuilder.setContentTitle(getString(R.string.bsdk_publish_wip_service_in_progress_notification_title));
        else {
            mBuilder.setContentTitle(getString(behanceSDK.getBehanceSDKCustomResourcesOptions().getAppTitle()));
            int color = behanceSDK.getBehanceSDKCustomResourcesOptions().getAppColor();
            if(color != 0)
                mBuilder.setColor(getResources().getColor(color));
        }

        mBuilder.setContentText(getString(R.string.bsdk_publish_wip_service_in_progress_notification_text));
        mBuilder.setStyle(new NotificationCompat.BigTextStyle().bigText(getString(R.string.bsdk_publish_wip_service_in_progress_notification_text)));
        mBuilder.setProgress(0, 0, true);
        Notification notification = mBuilder.build();
        startForeground(BehanceSDKConstants.NOTIFICATION_ID_PUBLISH_WIP_IN_PROGRESS, notification);

        Serializable params = intent.getSerializableExtra(INTENT_EXTRA_PARAMS);
        if (params instanceof BehanceSDKPublishWIPServiceParamsDTO) {
            BehanceSDKPublishWIPServiceParamsDTO serviceParamsDTO = (BehanceSDKPublishWIPServiceParamsDTO) params;

            if (isActiveUploadCancelled()) {
                cleanupCreativeCloudAsset(serviceParamsDTO.getWipImageFile());
                broadcastUploadCancelledStatusUpdate();
                return;
            }
            // publish on server
            BehanceSDKPublishWIPStatusDTO publishStartBroadcastDTO = new BehanceSDKPublishWIPStatusDTO();
            publishStartBroadcastDTO.setProgressState(BehanceSDKPublishWIPProgressState.PUBLISH_STARTED);
            publishStartBroadcastDTO.setProgressPercentage(getProgressPercentage(BehanceSDKPublishWIPProgressState.PUBLISH_STARTED, serviceParamsDTO));
            broadcastStatusUpdate(publishStartBroadcastDTO);
            BehanceSDKPublishWIPResult publishWIPResult = publishWIPOnServer(serviceParamsDTO);

            // check result and display Notification
            BehanceSDKPublishedWIPRevisionDTO publishedWIP = publishWIPResult.getPublishedWIP();
            Class<? extends Activity> notificationHandlerActivity = serviceParamsDTO.getNotificationHandlerActivity();

            BehanceSDKPublishWIPStatusDTO wipPublishBroadcastDTO = new BehanceSDKPublishWIPStatusDTO();

            if (publishedWIP != null && publishWIPResult.getStatus() == BehanceSDKPublishWIPResult.PublishStatus.SUCCESS) {

                BehanceSDKPublishWIPStatusDTO imageUploadBroadcastDTO = new BehanceSDKPublishWIPStatusDTO();
                imageUploadBroadcastDTO.setProgressState(BehanceSDKPublishWIPProgressState.IMAGE_UPLOAD_SUCCESSFUL);
                imageUploadBroadcastDTO.setProgressPercentage(getProgressPercentage(BehanceSDKPublishWIPProgressState.IMAGE_UPLOAD_SUCCESSFUL, serviceParamsDTO));
                broadcastStatusUpdate(imageUploadBroadcastDTO);

                if (!isActiveUploadCancelled() && serviceParamsDTO.isShareOnFacebook()) {
                    BehanceSDKPublishWIPStatusDTO facebookBroadcastDTO = publishOnFacebook(publishedWIP.getWipTitle(), serviceParamsDTO.getWipDescription(), publishedWIP.getRevisionURL(),
                            publishedWIP.getWipImageURL());
                    facebookBroadcastDTO.setWipRevisionUrl(publishedWIP.getRevisionURL());
                    facebookBroadcastDTO.setWipId(publishedWIP.getWipId());
                    facebookBroadcastDTO.setWipRevisionDescription(serviceParamsDTO.getWipDescription());
                    facebookBroadcastDTO.setWipRevisionId(publishedWIP.getRevisionId());
                    facebookBroadcastDTO.setWipTitle(publishedWIP.getWipTitle());
                    facebookBroadcastDTO.setProgressPercentage(getProgressPercentage(BehanceSDKPublishWIPProgressState.SHARE_ON_FACEBOOK_SUCCESSFUL, serviceParamsDTO));
                    broadcastStatusUpdate(facebookBroadcastDTO);
                }

                if (!isActiveUploadCancelled() && serviceParamsDTO.isShareOnTwitter()) {
                    BehanceSDKPublishWIPStatusDTO twitterBroadcastDTO = publishOnTwitter(serviceParamsDTO.getTwitterUserAccessToken(), serviceParamsDTO.getTwitterUserAccessTokenSecret(),
                            serviceParamsDTO.getTwitterConsumerKey(), serviceParamsDTO.getTwitterConsumerSecret(), publishedWIP.getRevisionURL());
                    twitterBroadcastDTO.setWipRevisionUrl(publishedWIP.getRevisionURL());
                    twitterBroadcastDTO.setWipId(publishedWIP.getWipId());
                    twitterBroadcastDTO.setWipRevisionDescription(serviceParamsDTO.getWipDescription());
                    twitterBroadcastDTO.setWipRevisionId(publishedWIP.getRevisionId());
                    twitterBroadcastDTO.setWipTitle(publishedWIP.getWipTitle());
                    twitterBroadcastDTO.setProgressPercentage(getProgressPercentage(BehanceSDKPublishWIPProgressState.SHARE_ON_TWITTER_SUCCESSFUL, serviceParamsDTO));
                    broadcastStatusUpdate(twitterBroadcastDTO);
                }

                if (isActiveUploadCancelled()) {
                    handleCancelAfterSuccessfulUpload(publishedWIP, serviceParamsDTO);
                    return;
                }

                //Allowing other apps to override Behance title in notification.
                if(behanceSDK.useBehanceTheme()) {
                    mBuilder.setContentTitle(getString(R.string.bsdk_publish_wip_service_success_notification_title));
                    mBuilder.setContentText(getString(R.string.bsdk_publish_wip_service_success_notification_text));

                }
                else {
                    mBuilder.setContentTitle(getString(behanceSDK.getBehanceSDKCustomResourcesOptions().getAppTitle()));
                    mBuilder.setContentText(getString(R.string.bsdk_publish_wip_service_success_notification_text_behance));
                }
                mBuilder.setTicker(getString(R.string.bsdk_publish_wip_service_success_notification_ticker));
                mBuilder.setStyle(new NotificationCompat.BigTextStyle().bigText(getString(R.string.bsdk_publish_wip_service_success_notification_text)));

                if (notificationHandlerActivity != null) {
                    Intent notifyIntent = createNotifyIntent(notificationHandlerActivity);
                    notifyIntent.putExtra(IBehanceSDKPublishWIPResultIntentExtras.INTENT_BOOL_EXTRA_PUBLISHED_WIP_SUCCESSFULLY, true);
                    notifyIntent.putExtra(IBehanceSDKPublishWIPResultIntentExtras.INTENT_INT_EXTRA_PUBLISHED_WIP_ID, publishedWIP.getWipId());
                    notifyIntent.putExtra(IBehanceSDKPublishWIPResultIntentExtras.INTENT_STR_EXTRA_PUBLISHED_WIP_TITLE, publishedWIP.getWipTitle());
                    notifyIntent.putExtra(IBehanceSDKPublishWIPResultIntentExtras.INTENT_INT_EXTRA_PUBLISHED_WIP_REVISION_ID,
                            publishedWIP.getRevisionId());
                    notifyIntent.putExtra(IBehanceSDKPublishWIPResultIntentExtras.INTENT_STR_EXTRA_PUBLISHED_WIP_REVISION_DESC,
                            publishedWIP.getRevisionDescription());
                    notifyIntent.putExtra(IBehanceSDKPublishWIPResultIntentExtras.INTENT_STR_EXTRA_PUBLISHED_WIP_REVISION_URL,
                            publishedWIP.getRevisionURL());
                    PendingIntent resultPendingIntent = createPendingIntent(notifyIntent);

                    mBuilder.setContentIntent(resultPendingIntent);
                }

                wipPublishBroadcastDTO.setProgressState(BehanceSDKPublishWIPProgressState.PUBLISH_SUCCESSFUL);
                wipPublishBroadcastDTO.setProgressPercentage(getProgressPercentage(BehanceSDKPublishWIPProgressState.PUBLISH_SUCCESSFUL, serviceParamsDTO));
                wipPublishBroadcastDTO.setWipRevisionUrl(publishedWIP.getRevisionURL());
                wipPublishBroadcastDTO.setWipId(publishedWIP.getWipId());
                wipPublishBroadcastDTO.setWipRevisionDescription(serviceParamsDTO.getWipDescription());
                wipPublishBroadcastDTO.setWipRevisionId(publishedWIP.getRevisionId());
                wipPublishBroadcastDTO.setWipTitle(publishedWIP.getWipTitle());
                wipPublishBroadcastDTO.setWipRevisionImageUrl(publishedWIP.getWipImageURL());
            } else {
                if (isActiveUploadCancelled()) {
                    broadcastUploadCancelledStatusUpdate();
                    return;
                }
                
                //Allowing other apps to override Behance Title in notification.
                if(behanceSDK.useBehanceTheme())
                    mBuilder.setContentTitle(getString(R.string.bsdk_publish_wip_service_failure_notification_title));
                else
                    mBuilder.setContentTitle(getString(behanceSDK.getBehanceSDKCustomResourcesOptions().getAppTitle()));


                mBuilder.setTicker(getString(R.string.bsdk_publish_wip_service_failure_notification_ticker));
                String msgToDisplay = "";
                String reasonPhrase = publishWIPResult.getReasonPhrase();
                if (reasonPhrase != null) {
                    msgToDisplay = getString(R.string.bsdk_publish_wip_service_failure_notification_msg, reasonPhrase);
                } else {
                    msgToDisplay = getString(R.string.bsdk_publish_wip_service_unknown_failure_notification_msg);
                }
                mBuilder.setContentText(msgToDisplay);
                mBuilder.setStyle(new NotificationCompat.BigTextStyle().bigText(msgToDisplay));

                if (notificationHandlerActivity != null) {
                    Intent notifyIntent = createNotifyIntent(notificationHandlerActivity);
                    notifyIntent.putExtra(IBehanceSDKPublishWIPResultIntentExtras.INTENT_BOOL_EXTRA_PUBLISHED_WIP_SUCCESSFULLY, false);
                    notifyIntent.putExtra(IBehanceSDKPublishWIPResultIntentExtras.INTENT_STR_EXTRA_PUBLISHED_WIP_TITLE,
                            serviceParamsDTO.getWipTitle());
                    notifyIntent.putExtra(IBehanceSDKPublishWIPResultIntentExtras.INTENT_STR_EXTRA_PUBLISHED_WIP_REVISION_DESC,
                            serviceParamsDTO.getWipDescription());
                    notifyIntent.putExtra(IBehanceSDKPublishWIPResultIntentExtras.INTENT_STR_EXTRA_PUBLISH_WIP_FAILURE_MSG, msgToDisplay);
                    PendingIntent resultPendingIntent = createPendingIntent(notifyIntent);

                    mBuilder.setContentIntent(resultPendingIntent);
                }

                wipPublishBroadcastDTO.setProgressState(BehanceSDKPublishWIPProgressState.PUBLISH_FAILED);
                wipPublishBroadcastDTO.setProgressPercentage(getProgressPercentage(BehanceSDKPublishWIPProgressState.PUBLISH_FAILED, serviceParamsDTO));
                wipPublishBroadcastDTO.setWipRevisionDescription(serviceParamsDTO.getWipDescription());
                wipPublishBroadcastDTO.setWipTitle(serviceParamsDTO.getWipTitle());
                wipPublishBroadcastDTO.setErrorMessage(reasonPhrase);
                wipPublishBroadcastDTO.setException(publishWIPResult.getException());
            }

            if (!isActiveUploadCancelled()) {
                //Allowing other apps to override Behance Icon in notification.
                if(behanceSDK.useBehanceTheme()){
                    mBuilder.setSmallIcon(R.drawable.bsdk_icon_notification_publish_progress);
                } else{
                    mBuilder.setSmallIcon(behanceSDK.getBehanceSDKCustomResourcesOptions().getAppSmallIcon());
                }
                mBuilder.setProgress(0, 0, false);
                mBuilder.setAutoCancel(true);
                notification = mBuilder.build();
                NotificationManager mNotificationManager = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);
                mNotificationManager.notify(BehanceSDKConstants.NOTIFICATION_ID_PUBLISH_WIP_COMPLETE, notification);
                broadcastStatusUpdate(wipPublishBroadcastDTO);
            } else {
                if (publishedWIP != null && publishWIPResult.getStatus() == BehanceSDKPublishWIPResult.PublishStatus.SUCCESS) {
                    handleCancelAfterSuccessfulUpload(publishedWIP, serviceParamsDTO);
                } else {
                    broadcastUploadCancelledStatusUpdate();
                }
            }
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        wipBinder = null;
    }

    private PendingIntent createPendingIntent(Intent notifyIntent) {
        return PendingIntent.getActivity(this, 0, notifyIntent, PendingIntent.FLAG_UPDATE_CURRENT);
    }

    private Intent createNotifyIntent(Class<? extends Activity> notificationHandlerActivity) {
        Intent notifyIntent = new Intent(this, notificationHandlerActivity);
        notifyIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        return notifyIntent;
    }

    private BehanceSDKPublishWIPResult publishWIPOnServer(final BehanceSDKPublishWIPServiceParamsDTO serviceParamsDTO) {
        BehanceSDKPublishWIPResult result = new BehanceSDKPublishWIPResult();
        ProjectModule wipImageModule = serviceParamsDTO.getWipImageFile();
        try {
            Map<String, String> urlParams = new HashMap<String, String>();
            urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, serviceParamsDTO.getAppClientId());

            String url;
            if (serviceParamsDTO.getWipId() > 0) {
                urlParams.put(BehanceSDKUrlUtil.KEY_WIP_ID, String.valueOf(serviceParamsDTO.getWipId()));
                url = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.PUBLISH_IMAGE_TO_EXISTING_WIP_API_URL, urlParams);
            } else {
                url = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.PUBLISH_NEW_WIP_API_URL, urlParams);
            }

            String userAccessToken = sdkUserManager.checkExpiryAndGetAccessToken();
            if (userAccessToken != null) {
                url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_ACCESS_TOKEN, userAccessToken);
            }

            BehanceMultipartEntity multipartEntity = new BehanceMultipartEntity();
            multipartEntity.setChunked(true);
            multipartEntity.setBrowserCompatible(true);

            multipartEntity.addImagePart("image", wipImageModule.getFile().getName(), BehanceMultipartEntity.CONTENT_TYPE_APPLICATION_OCTETSTREAM,
                    BehanceMultipartEntity.CONTENT_TRANSFER_BINARY, wipImageModule.getFile().getAbsolutePath());
            if (serviceParamsDTO.getWipTitle() != null) {
                multipartEntity.addTextPart("title", BehanceMultipartEntity.CONTENT_TYPE_TEXT_UTF8, BehanceMultipartEntity.CONTENT_TRANSFER_8BIT, serviceParamsDTO.getWipTitle().trim().getBytes(BehanceMultipartEntity.CHARSET_UTF8));
            }
            multipartEntity.addTextPart("description", BehanceMultipartEntity.CONTENT_TYPE_TEXT_UTF8, BehanceMultipartEntity.CONTENT_TRANSFER_8BIT, serviceParamsDTO.getWipDescription().getBytes(BehanceMultipartEntity.CHARSET_UTF8));
            multipartEntity.addTextPart("tags", BehanceMultipartEntity.CONTENT_TYPE_TEXT_UTF8, BehanceMultipartEntity.CONTENT_TRANSFER_8BIT,serviceParamsDTO.getWipTags().getBytes(BehanceMultipartEntity.CHARSET_UTF8));
            multipartEntity.addTextPart("mature", BehanceMultipartEntity.CONTENT_TYPE_TEXT_UTF8, BehanceMultipartEntity.CONTENT_TRANSFER_8BIT,
                    serviceParamsDTO.isContainsAdultContent() ? "1".getBytes(BehanceMultipartEntity.CHARSET_UTF8) : "0".getBytes(BehanceMultipartEntity.CHARSET_UTF8));

            BehanceRequestCancellationWrapper cancellationWrapper = new BehanceRequestCancellationWrapper();
            WIPProjectModuleUploadEntityWrapperCallback customWrapperCallback = new WIPProjectModuleUploadEntityWrapperCallback(serviceParamsDTO,cancellationWrapper);
            multipartEntity.setBehanceHttpEntityWrapperCallback(customWrapperCallback);

            if (!isActiveUploadCancelled()) {
                BehanceConnectionResponse<String> httpResponse = BehanceHttpsConnection.getInstance().invokeHttpPostRequest(url, multipartEntity, cancellationWrapper);

                String jsonResponseString = httpResponse.getResponseObject();
                logger.debug("Publish WIP json response [%s]", jsonResponseString);

                if (httpResponse.getResponseCode() == BehanceHttpsConnection.SC_CREATED) {
                    result.setStatus(PublishStatus.SUCCESS);

                    // parse WIP from server
                    BehanceSDKPublishedWIPRevisionDTO publishedWIPDTO = new BehanceSDKPublishedWIPRevisionDTO();

                    JSONObject rootJsonObject = new JSONObject(jsonResponseString);

                    JSONObject wipJSONObject = rootJsonObject.optJSONObject("wip");
                    if (wipJSONObject != null) {
                        // published to new WIP
                        publishedWIPDTO.setWipTitle(wipJSONObject.optString("title"));

                        // get the published revision details
                        JSONObject revisionsJsonObject = wipJSONObject.optJSONObject("revisions");
                        if (revisionsJsonObject != null) {
                            JSONArray revisionIDsJsonArray = revisionsJsonObject.names();
                            if (revisionIDsJsonArray != null && revisionIDsJsonArray.length() > 0) {
                                for (int counter2 = 0; counter2 < revisionIDsJsonArray.length(); counter2++) {
                                    JSONObject revisionJsonObject = revisionsJsonObject.optJSONObject(revisionIDsJsonArray.getString(counter2));
                                    if (revisionJsonObject != null) {
                                        publishedWIPDTO.setRevisionId(revisionJsonObject.optInt("id"));
                                        publishedWIPDTO.setRevisionURL(revisionJsonObject.optString("url"));
                                        publishedWIPDTO.setRevisionDescription(revisionJsonObject.optString("description"));
                                        // we are getting wip id from here, because
                                        // this is from server
                                        publishedWIPDTO.setWipId(revisionJsonObject.optInt("wip_id"));
                                        JSONObject imagesJsonObject = revisionJsonObject.optJSONObject("images");
                                        if (imagesJsonObject != null) {
                                            JSONObject resolutionJsonObject = imagesJsonObject.optJSONObject("normal_resolution");
                                            if (resolutionJsonObject != null) {
                                                publishedWIPDTO.setWipImageURL(resolutionJsonObject.optString("url"));
                                            }
                                        }

                                        // we need only the first non null one
                                        break;
                                    }
                                }
                            }
                        }

                        result.setPublishedWIP(publishedWIPDTO);
                    } else {
                        // published to existing WIP. We will not get title from
                        // server
                        publishedWIPDTO.setWipTitle(serviceParamsDTO.getWipTitle());
                        JSONObject revisionJsonObject = rootJsonObject.optJSONObject("revision");
                        if (revisionJsonObject != null) {
                            publishedWIPDTO.setRevisionId(revisionJsonObject.optInt("id"));
                            publishedWIPDTO.setRevisionURL(revisionJsonObject.optString("url"));
                            publishedWIPDTO.setRevisionDescription(revisionJsonObject.optString("description"));
                            // we are getting wip id from here, because this is from
                            // server
                            publishedWIPDTO.setWipId(revisionJsonObject.optInt("wip_id"));
                            JSONObject imagesJsonObject = revisionJsonObject.optJSONObject("images");
                            if (imagesJsonObject != null) {
                                JSONObject resolutionJsonObject = imagesJsonObject.optJSONObject("normal_resolution");
                                if (resolutionJsonObject != null) {
                                    publishedWIPDTO.setWipImageURL(resolutionJsonObject.optString("url"));
                                }
                            }

                            result.setPublishedWIP(publishedWIPDTO);
                        }
                    }
                } else {
                    result.setStatus(PublishStatus.PUBLISH_FAILED);
                    JSONObject rootJsonObject = new JSONObject(jsonResponseString);
                    JSONArray messagesJSONArray = rootJsonObject.optJSONArray("messages");
                    if (messagesJSONArray != null) {
                        for (int counter = 0; counter < messagesJSONArray.length(); counter++) {
                            JSONObject messageJsonObject = messagesJSONArray.getJSONObject(counter);
                            String messageType = messageJsonObject.optString("type");
                            if (messageType != null && messageType.equalsIgnoreCase("error")) {
                                result.setReasonPhrase(messageJsonObject.optString("message"));
                                logger.debug("Problem publishing WIP. Response from server [%s]", result.getReasonPhrase());
                                break;
                            }
                        }
                    }
                }
            }
            cleanupCreativeCloudAsset(wipImageModule);

        } catch (Exception e) {
            cleanupCreativeCloudAsset(wipImageModule);
            result.setStatus(PublishStatus.PUBLISH_FAILED);
            result.setException(e);
            logger.error(e, "Problem publishing WIP");
        } catch (Error e) {
            cleanupCreativeCloudAsset(wipImageModule);
            result.setStatus(PublishStatus.PUBLISH_FAILED);
            result.setException(new Exception(e));
            logger.error(e, "Problem publishing WIP");
        }
        return result;
    }

    private void cleanupCreativeCloudAsset(ProjectModule wipImageModule) {
        if(wipImageModule.getType() == ProjectModuleTypes.CREATIVECLOUD_ASSET){
            wipImageModule.deleteFromFileSystem();;
        }
    }

    private BehanceSDKPublishWIPStatusDTO publishOnFacebook(String wipTitle, String wipRevisionDescription, String wipRevisionUrl, String wipImageURL) {
        BehanceSDKPublishWIPStatusDTO statusDTO = new BehanceSDKPublishWIPStatusDTO();
        try {
            AccessToken session = AccessToken.getCurrentAccessToken();
            if (session != null) {
                Bundle postParams = new Bundle();
                postParams.putString("name", wipTitle);
                postParams.putString("link", wipRevisionUrl);
                if (wipImageURL != null && wipImageURL.length() > 0) {
                    postParams.putString("picture", wipImageURL);
                }
                if (!isActiveUploadCancelled()) {
                    GraphRequest request = new GraphRequest(session, "me/feed", postParams, HttpMethod.POST, null);
                    GraphResponse response = request.executeAndWait();
                    FacebookRequestError error = response.getError();
                    if (error != null) {
                        // toString() display app client id along with other info,
                        // we don't want to log this in release
                        logger.debug("Problem sharing WIP on Facebook [Message - %s]", error.toString());
                        logger.error("Problem sharing WIP on Facebook [Message - %s]", error.getErrorUserMessage());
                        statusDTO.setProgressState(BehanceSDKPublishWIPProgressState.SHARE_ON_FACEBOOK_FAILED);
                        statusDTO.setErrorMessage(error.getErrorUserMessage());
                    } else {
                        logger.debug("Successfully shared WIP on Facebook");
                        statusDTO.setProgressState(BehanceSDKPublishWIPProgressState.SHARE_ON_FACEBOOK_SUCCESSFUL);
                    }
                }
            }
        } catch (Throwable e) {
            logger.error(e, "Problem sharing WIP on Facebook");
            statusDTO.setProgressState(BehanceSDKPublishWIPProgressState.SHARE_ON_FACEBOOK_FAILED);
            statusDTO.setErrorMessage(e.getMessage());
            statusDTO.setException(e);
        }
        return statusDTO;
    }

    private BehanceSDKPublishWIPStatusDTO publishOnTwitter(String accessToken, String accessTokenSecret, String consumerKey, String consumerSecret, String WIPRevisionURL) {
        BehanceSDKPublishWIPStatusDTO statusDTO = new BehanceSDKPublishWIPStatusDTO();
        try {
            // Create a new consumer using the commons implementation
            OAuthConsumer consumer = new DefaultOAuthConsumer(consumerKey, consumerSecret);
            consumer.setTokenWithSecret(accessToken, accessTokenSecret);
            String queryString = getString(R.string.bsdk_publish_wip_service_publish_on_twitter_msg, WIPRevisionURL);
            String encodedQueryString = URLEncoder.encode(queryString, "UTF-8");
            String finalURL = TWITTER_UPDATE_API_URL + encodedQueryString;

            if (!isActiveUploadCancelled()) {
                BehanceConnectionResponse<String> response = BehanceHttpsConnection.getInstance().invokeHttpPostRequest(finalURL,null,null,consumer);

                int statusCode = response.getResponseCode();
                if (statusCode == BehanceHttpsConnection.SC_OK) {
                    logger.debug("Successfully shared WIP on Twitter");
                    statusDTO.setProgressState(BehanceSDKPublishWIPProgressState.SHARE_ON_TWITTER_SUCCESSFUL);
                } else if (statusCode == BehanceHttpsConnection.SC_UNAUTHORIZED) {
                    String reasonPhrase = response.getResponseObject();
                    logger.error("Twitter User token has been revoked. [Error code - %s - message - %s]", statusCode, reasonPhrase);
                    unlinkTwitterAccount();
                    statusDTO.setProgressState(BehanceSDKPublishWIPProgressState.SHARE_ON_TWITTER_FAILED);
                    statusDTO.setErrorMessage(reasonPhrase);
                } else {
                    String reasonPhrase = response.getResponseObject();
                    logger.error("Error sharing WIP on Twitter. [Error code - %s - message - %s]", statusCode, reasonPhrase);
                    statusDTO.setProgressState(BehanceSDKPublishWIPProgressState.SHARE_ON_TWITTER_FAILED);
                    statusDTO.setErrorMessage(reasonPhrase);
                }
            }
        } catch (Throwable e) {
            logger.error(e, "Error sharing WIP on Twitter");
            statusDTO.setProgressState(BehanceSDKPublishWIPProgressState.SHARE_ON_TWITTER_FAILED);
            statusDTO.setErrorMessage(e.getMessage());
            statusDTO.setException(e);
        }
        return statusDTO;
    }

    private void unlinkTwitterAccount() {
        BehanceSDKSocialAccountManager socialAccountManager = BehanceSDKSocialAccountManager.getInstance(this);
        BehanceSDKSocialAccountDTO twitterAccount = socialAccountManager.getAccount(BehanceSDKSocialAccountType.TWITTER);
        if (twitterAccount != null) {
            socialAccountManager.removeAccount(BehanceSDKSocialAccountType.TWITTER);
        }
    }

    private void handleCancelAfterSuccessfulUpload(BehanceSDKPublishedWIPRevisionDTO publishedWIP, BehanceSDKPublishWIPServiceParamsDTO serviceParamsDTO) {
        BehanceSDKPublishWIPStatusDTO statusDto = new BehanceSDKPublishWIPStatusDTO();
        statusDto.setPublishRequestId(activePublishRequestId);
        statusDto.setProgressPercentage(getProgressPercentage(BehanceSDKPublishWIPProgressState.CANCEL_PUBLISH_SUCCESSFUL, null));
        try {
            Map<String, String> urlParams = new HashMap<String, String>();
            urlParams.put(BehanceSDKUrlUtil.KEY_CLIENT_ID, serviceParamsDTO.getAppClientId());
            urlParams.put(BehanceSDKUrlUtil.KEY_WIP_ID, String.valueOf(publishedWIP.getWipId()));
            urlParams.put(BehanceSDKUrlUtil.KEY_WIP_REVISION_ID, String.valueOf(publishedWIP.getRevisionId()));

            String url = BehanceSDKUrlUtil.getUrlFromTemplate(BehanceSDKUrlUtil.DELETE_WIP_REVISION_API_URL, urlParams);

            String userAccessToken = sdkUserManager.checkExpiryAndGetAccessToken();
            if (userAccessToken != null) {
                url = BehanceSDKUrlUtil.appendQueryStringParam(url, BehanceSDKUrlUtil.PARAM_KEY_ACCESS_TOKEN, userAccessToken);
            }

            BehanceConnectionResponse<String> httpResponse = BehanceHttpsConnection.getInstance().invokeHttpDeleteRequest(url);

            logger.debug("Publish WIP cancelled after successful upload. Deleting WIP [URL - %s]", url);


            logger.debug("Delete WIP Revision HTTP response code - %s", httpResponse.getResponseCode());
            String jsonResponseString = httpResponse.getResponseObject();
            logger.debug("Delete WIP Revision server response - %s", jsonResponseString);
            if (httpResponse.getResponseCode() == BehanceHttpsConnection.SC_OK) {
                statusDto.setProgressState(BehanceSDKPublishWIPProgressState.CANCEL_PUBLISH_SUCCESSFUL);
            } else {
                statusDto.setProgressState(BehanceSDKPublishWIPProgressState.CANCEL_PUBLISH_FAILED);
            }
        } catch (Throwable e) {
            statusDto.setProgressState(BehanceSDKPublishWIPProgressState.CANCEL_PUBLISH_FAILED);
            statusDto.setException(e);
            logger.error(e, "Problem deleting WIP Revision");
        }
        Intent broadcastIntent = new Intent(BehanceSDKPublishWIPStatusDTO.INTENT_ACTION_WIP_PUBLISH_STATUS_BROADCAST);
        broadcastIntent.putExtra(BehanceSDKPublishWIPStatusDTO.INTENT_EXTRA_OBJ_KEY_WIP_PUBLISH_STATUS_BROADCAST_DATA, statusDto);
        LocalBroadcastManager.getInstance(this).sendBroadcast(broadcastIntent);
    }

    private void broadcastUploadCancelledStatusUpdate() {
        logger.debug("Publish WIP cancelled. Broadcasting message");
        BehanceSDKPublishWIPStatusDTO statusDto = new BehanceSDKPublishWIPStatusDTO();
        statusDto.setPublishRequestId(activePublishRequestId);
        statusDto.setProgressState(BehanceSDKPublishWIPProgressState.CANCEL_PUBLISH_SUCCESSFUL);
        statusDto.setProgressPercentage(getProgressPercentage(BehanceSDKPublishWIPProgressState.CANCEL_PUBLISH_SUCCESSFUL, null));
        Intent broadcastIntent = new Intent(BehanceSDKPublishWIPStatusDTO.INTENT_ACTION_WIP_PUBLISH_STATUS_BROADCAST);
        broadcastIntent.putExtra(BehanceSDKPublishWIPStatusDTO.INTENT_EXTRA_OBJ_KEY_WIP_PUBLISH_STATUS_BROADCAST_DATA, statusDto);
        LocalBroadcastManager.getInstance(this).sendBroadcast(broadcastIntent);
    }

    private void broadcastStatusUpdate(BehanceSDKPublishWIPStatusDTO statusDto) {
        if (!isActiveUploadCancelled()) {
            if(statusDto.getProgressState() == BehanceSDKPublishWIPProgressState.PUBLISH_SUCCESSFUL) {
                AdobeAnalyticsSDKReporter.trackSharingAction(AdobeAnalyticsSDKReporter.AdobeAnalyticsShareTypePublishSuccess,
                        AdobeAnalyticsSDKReporter.AdobeAnalyticsShareTargetBehanceWIP);
            } else if (statusDto.getProgressState() == BehanceSDKPublishWIPProgressState.PUBLISH_FAILED) {
                AdobeAnalyticsSDKReporter.trackSharingAction(AdobeAnalyticsSDKReporter.AdobeAnalyticsShareTypePublishFailure,
                        AdobeAnalyticsSDKReporter.AdobeAnalyticsShareTargetBehanceWIP);
            }
            statusDto.setPublishRequestId(activePublishRequestId);
            Intent broadcastIntent = new Intent(BehanceSDKPublishWIPStatusDTO.INTENT_ACTION_WIP_PUBLISH_STATUS_BROADCAST);
            broadcastIntent.putExtra(BehanceSDKPublishWIPStatusDTO.INTENT_EXTRA_OBJ_KEY_WIP_PUBLISH_STATUS_BROADCAST_DATA, statusDto);
            LocalBroadcastManager.getInstance(this).sendBroadcast(broadcastIntent);
        }
    }

    private int convertImageUploadProgressPercentage(BehanceSDKPublishWIPServiceParamsDTO paramsDTO, int progressPercentage) {
        int startPercentage = getProgressPercentage(BehanceSDKPublishWIPProgressState.PUBLISH_STARTED, paramsDTO);
        int endPercentage = getProgressPercentage(BehanceSDKPublishWIPProgressState.IMAGE_UPLOAD_SUCCESSFUL, paramsDTO);
        int availablePercentage = endPercentage - startPercentage;
        int response = startPercentage + ((progressPercentage * availablePercentage) / 100);
        return response;
    }

    private int getProgressPercentage(BehanceSDKPublishWIPProgressState progressState, BehanceSDKPublishWIPServiceParamsDTO paramsDTO) {
        int response = 0;
        switch (progressState) {
            case PUBLISH_STARTED: {
                response = 10;
                break;
            }
            case IMAGE_UPLOAD_SUCCESSFUL: {
                if (paramsDTO.isShareOnFacebook() && paramsDTO.isShareOnTwitter()) {
                    response = 50;
                } else if (paramsDTO.isShareOnFacebook() || paramsDTO.isShareOnTwitter()) {
                    response = 70;
                } else {
                    response = 90;
                }
                break;
            }
            case SHARE_ON_FACEBOOK_SUCCESSFUL:
            case SHARE_ON_FACEBOOK_FAILED: {
                if (paramsDTO.isShareOnTwitter()) {
                    response = 70;
                } else {
                    response = 90;
                }
                break;
            }
            case SHARE_ON_TWITTER_SUCCESSFUL:
            case SHARE_ON_TWITTER_FAILED: {
                response = 90;
                break;
            }
            case PUBLISH_FAILED:
            case CANCEL_PUBLISH_SUCCESSFUL:
            case CANCEL_PUBLISH_FAILED:
            case PUBLISH_SUCCESSFUL: {
                response = 100;
                break;
            }
        }
        return response;
    }

    private class WIPProjectModuleUploadEntityWrapperCallback implements BehanceMultipartEntityWrapper{
        private int lastBroadcastedProgressPercentage = 0;
        private final BehanceRequestCancellationWrapper cancellationWrapper;
        private final BehanceSDKPublishWIPServiceParamsDTO serviceParamsDTO;

        private WIPProjectModuleUploadEntityWrapperCallback(BehanceSDKPublishWIPServiceParamsDTO serviceParamsDTO,
                                                            BehanceRequestCancellationWrapper cancellationWrapper) {
            this.cancellationWrapper = cancellationWrapper;
            this.serviceParamsDTO = serviceParamsDTO;
        }

        @Override
        public void onUploadProgressUpdate(float progressPercentage) {
            int progressPercentInt = (int) progressPercentage;
            if ((progressPercentInt % 10) == 0) {
                if (lastBroadcastedProgressPercentage != progressPercentInt) {
                    lastBroadcastedProgressPercentage = progressPercentInt;
                    // we will notify 100 percent when parsing is done
                    if (progressPercentInt < 100) {
                        BehanceSDKPublishWIPStatusDTO imageUploadBroadcastDTO = new BehanceSDKPublishWIPStatusDTO();
                        imageUploadBroadcastDTO.setProgressState(BehanceSDKPublishWIPProgressState.IMAGE_UPLOAD_IN_PROGRESS);
                        imageUploadBroadcastDTO.setProgressPercentage(
                                convertImageUploadProgressPercentage(serviceParamsDTO, progressPercentInt));
                        broadcastStatusUpdate(imageUploadBroadcastDTO);
                        if (isActiveUploadCancelled()) {
                            cancellationWrapper.setCancelled();
                        }
                    }
                }
            }
        }
    };
}
