package com.behance.sdk.ui.dialogs;

import android.content.Context;
import android.os.Bundle;
import android.support.v4.app.DialogFragment;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.Animation;
import android.view.inputmethod.EditorInfo;
import android.view.inputmethod.InputMethodManager;
import android.widget.AdapterView;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.ViewFlipper;

import com.behance.sdk.R;
import com.behance.sdk.asynctask.params.BehanceSDKGetCitiesAsyncTaskParams;
import com.behance.sdk.datamanager.BehanceSDKProfileLocationsDataManager;
import com.behance.sdk.dto.location.BehanceSDKCityDTO;
import com.behance.sdk.dto.location.BehanceSDKCountryDTO;
import com.behance.sdk.dto.location.BehanceSDKStateDTO;
import com.behance.sdk.listeners.IBehanceSDKLocationFiltersDataManagerListener;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.ui.adapters.BehanceSDKLocationFilterListItemArrayAdapter;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

public class BehanceSDKLocationSelectionDialog extends DialogFragment implements View.OnClickListener, IBehanceSDKLocationFiltersDataManagerListener {

    private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKLocationSelectionDialog.class);

    private static final String INSTANCE_STATE_KEY_LAST_DISPLAYED_CHILD_INDEX = "INSTANCE_STATE_KEY_LAST_DISPLAYED_CHILD_INDEX";
    public static final String INSTANCE_STATE_KEY_SELECTED_COUNTRY = "INSTANCE_STATE_KEY_SELECTED_COUNTRY";
    public static final String INSTANCE_STATE_KEY_SELECTED_STATE = "INSTANCE_STATE_KEY_SELECTED_STATE";
    public static final String INSTANCE_STATE_KEY_SELECTED_CITY = "INSTANCE_STATE_KEY_SELECTED_CITY";

    private static final int MAIN_VIEW_DISPLAY_INDEX = 0;
    private static final int COUNTRIES_VIEW_DISPLAY_INDEX = 1;
    private static final int STATES_VIEW_DISPLAY_INDEX = 2;
    private static final int CITIES_VIEW_DISPLAY_INDEX = 3;

    private ViewFlipper viewFlipper;
    private View rootView;
    private View progressSpinner;
    private ListView countriesListView;
    private TextView selectedCountryTxtView;
    private TextView selectedStateTxtView;
    private ListView statesListView;
    private View selectedStateContainer;
    private View selectedCityContainer;
    private TextView selectedCityTxtView;
    private ListView citiesListView;
    private EditText citySearchStrEditText;

    private BehanceSDKProfileLocationsDataManager locationFiltersDataManager;
    private BehanceSDKCountryDTO selectedCountryDTO;
    private BehanceSDKStateDTO selectedStateDTO;
    private BehanceSDKCityDTO selectedCityDTO;

    private Callbacks callbacks;
    private boolean loadingInProgress;

    public static BehanceSDKLocationSelectionDialog getInstance() {
        BehanceSDKLocationSelectionDialog dialogInstance = new BehanceSDKLocationSelectionDialog();
        return dialogInstance;
    }

    public interface Callbacks {
        public void onLocationFilterDone(BehanceSDKCountryDTO selectedCountry, BehanceSDKStateDTO selectedState, BehanceSDKCityDTO selectedCity);
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setStyle(DialogFragment.STYLE_NO_FRAME, R.style.BsdkFilterDialogTheme);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        rootView = inflater.inflate(R.layout.bsdk_dialog_fragment_location_filter, container, false);
        View backButton = rootView.findViewById(R.id.bsdkLocationFilterDialogBackBtn);
        backButton.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {
                dismiss();

            }
        });
        viewFlipper = (ViewFlipper) rootView.findViewById(R.id.bsdkLocationFilterDialogViewFlipper);
        Bundle arguments = getBundleArguments(savedInstanceState);
        int indexOfViewToDisplay = MAIN_VIEW_DISPLAY_INDEX;

        if (arguments != null) {
            indexOfViewToDisplay = arguments.getInt(INSTANCE_STATE_KEY_LAST_DISPLAYED_CHILD_INDEX, MAIN_VIEW_DISPLAY_INDEX);

            if (selectedCountryDTO == null) {
                Serializable serializedCountry = arguments.getSerializable(INSTANCE_STATE_KEY_SELECTED_COUNTRY);
                if (serializedCountry instanceof BehanceSDKCountryDTO) {
                    selectedCountryDTO = (BehanceSDKCountryDTO) serializedCountry;
                }
            }

            if (selectedStateDTO == null) {
                Serializable serializedState = arguments.getSerializable(INSTANCE_STATE_KEY_SELECTED_STATE);
                if (serializedState instanceof BehanceSDKStateDTO) {
                    selectedStateDTO = (BehanceSDKStateDTO) serializedState;
                }
            }

            if (selectedCityDTO == null) {
                Serializable serializedCity = arguments.getSerializable(INSTANCE_STATE_KEY_SELECTED_CITY);
                if (serializedCity instanceof BehanceSDKCityDTO) {
                    selectedCityDTO = (BehanceSDKCityDTO) serializedCity;
                }
            }
        }

        // if the selected country is worldwide, show the countries screen
        if (selectedCountryDTO == null || selectedCountryDTO.getId().equals(BehanceSDKCountryDTO.WORLD_WIDE_COUNTRY_ID)) {
            indexOfViewToDisplay = COUNTRIES_VIEW_DISPLAY_INDEX;
        }

        // by default the first view will displayed, hence we check for > 0 and
        // not >= 0
        if (indexOfViewToDisplay > 0) {
            viewFlipper.setDisplayedChild(indexOfViewToDisplay);
        }

        progressSpinner = rootView.findViewById(R.id.bsdkLocationFilterDialogProgressSpinner);

        // State related UI elements
        selectedStateContainer = rootView.findViewById(R.id.bsdkLocationFilterDialogSelectedStateContainer);
        selectedStateContainer.setTag(ClickTagData.SHOW_STATES_VIEW);
        selectedStateContainer.setOnClickListener(this);

        statesListView = (ListView) rootView.findViewById(R.id.bsdkLocationFilterDialogStatesListView);
        statesListView.setOnItemClickListener(new AdapterView.OnItemClickListener() {

            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                handleStateListItemClick(parent, position);
            }
        });

        // City related UI elements
        selectedCityContainer = rootView.findViewById(R.id.bsdkLocationFilterDialogSelectedCityContainer);
        selectedCityContainer.setTag(ClickTagData.SHOW_CITIES_VIEW);
        selectedCityContainer.setOnClickListener(this);
        selectedCityTxtView = (TextView) rootView.findViewById(R.id.bsdkLocationFilterDialogSelectedCityTxtView);

        citiesListView = (ListView) rootView.findViewById(R.id.bsdkLocationFilterDialogCitiesListView);
        citiesListView.setOnItemClickListener(new AdapterView.OnItemClickListener() {

            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                handleCityListItemClick(parent, position);
            }
        });

        citiesListView.setOnTouchListener(new View.OnTouchListener() {

            @Override
            public boolean onTouch(View v, MotionEvent event) {
                InputMethodManager imm = (InputMethodManager) getActivity().getSystemService(Context.INPUT_METHOD_SERVICE);
                imm.hideSoftInputFromWindow(rootView.getWindowToken(), 0);
                return false;
            }
        });

        citySearchStrEditText = (EditText) rootView.findViewById(R.id.bsdkLocationFilterDialogCitySearchStrEditText);
        citySearchStrEditText.addTextChangedListener(new TextWatcher() {

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
            }

            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }

            @Override
            public void afterTextChanged(Editable s) {
                if (s.length() >= 3) {
                    searchForCities(s.toString());
                }
            }
        });

        citySearchStrEditText.setOnEditorActionListener(new TextView.OnEditorActionListener() {

            @Override
            public boolean onEditorAction(TextView v, int actionId, KeyEvent event) {
                if (actionId == EditorInfo.IME_ACTION_GO) {
                    searchForCities(v.getText().toString());
                    return true;
                }
                return false;
            }
        });

        View closeDialogBtn = rootView.findViewById(R.id.bsdkLocationFilterDialogCloseDialogBtn);
        closeDialogBtn.setOnClickListener(this);

        View citiesSectionBackBtn = rootView.findViewById(R.id.bsdkLocationFilterDialogCitiesSectionBackBtn);
        citiesSectionBackBtn.setOnClickListener(this);

        View countriesSectionBackBtn = rootView.findViewById(R.id.bsdkLocationFilterDialogCountriesSectionBackBtn);
        countriesSectionBackBtn.setOnClickListener(this);

        View statesSectionBackBtn = rootView.findViewById(R.id.bsdkLocationFilterDialogStatesSectionBackBtn);
        statesSectionBackBtn.setOnClickListener(this);

        // create Country list view
        createCountriesListView();

        if (selectedCountryDTO != null) {
            String id = selectedCountryDTO.getId();
            if (BehanceSDKCountryDTO.COUNTRY_CODES_FOR_LOAD_STATES.contains(id)) {
                loadStates(id);
                displaySelectedStateItemInMainView();
            } else if (!id.equals(BehanceSDKCountryDTO.WORLD_WIDE_COUNTRY_ID)) {
                displaySelectedCityItemInMainView();
            }
        }

        if (selectedCityDTO != null) {
            displaySelectedCityItemInMainView();
        }

        return rootView;
    }

    private Bundle getBundleArguments(Bundle savedInstanceState) {
        Bundle arguments = savedInstanceState;
        if (arguments == null) {
            arguments = getArguments();
        }
        return arguments;
    }

    private void createCountriesListView() {
        View selectedCountryContainer = rootView.findViewById(R.id.bsdkLocationFilterDialogSelectedCountryContainer);
        selectedCountryTxtView = (TextView) rootView.findViewById(R.id.bsdkLocationFilterDialogSelectedCountryTxtView);
        selectedCountryTxtView.setText(getSelectedCountry().getDisplayName());

        selectedCountryContainer.setTag(ClickTagData.SHOW_COUNTRIES_VIEW);
        selectedCountryContainer.setOnClickListener(this);

        countriesListView = (ListView) rootView.findViewById(R.id.bsdkLocationFilterDialogCountriesListView);
        countriesListView.setOnItemClickListener(new AdapterView.OnItemClickListener() {

            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                handleCountryListItemClick(parent, position);
            }
        });

        locationFiltersDataManager = BehanceSDKProfileLocationsDataManager.getInstance();
        locationFiltersDataManager.addListener(this);
        showProgressSpinner();
        locationFiltersDataManager.loadCountries();
    }

    private void updateSelectedCountryInCountriesListView(BehanceSDKCountryDTO countryDTO) {
        @SuppressWarnings("unchecked")
        BehanceSDKLocationFilterListItemArrayAdapter<BehanceSDKCountryDTO> arrayAdapter = (BehanceSDKLocationFilterListItemArrayAdapter<BehanceSDKCountryDTO>) countriesListView.getAdapter();
        if (arrayAdapter != null) {
            BehanceSDKCountryDTO currentSelectedItem = (BehanceSDKCountryDTO) arrayAdapter.getSelectedLocationDTO();
            if (!currentSelectedItem.equals(countryDTO)) {
                arrayAdapter.setSelectedLocationDTO(countryDTO);
                arrayAdapter.notifyDataSetChanged();
            }
        }
    }

    private void displaySelectedStateItemInMainView() {
        selectedStateContainer.setVisibility(View.VISIBLE);
        selectedStateTxtView = (TextView) rootView.findViewById(R.id.bsdkLocationFilterDialogSelectedStateTxtView);
        selectedStateTxtView.setText(getSelectedState().getDisplayName());
    }

    private void hideSelectedStateItemInMainView() {
        selectedStateDTO = null;
        selectedStateContainer.setVisibility(View.GONE);
    }

    private void updateSelectedStateInStatesListView(BehanceSDKStateDTO stateDTO) {
        @SuppressWarnings("unchecked")
        BehanceSDKLocationFilterListItemArrayAdapter<BehanceSDKStateDTO> arrayAdapter = (BehanceSDKLocationFilterListItemArrayAdapter<BehanceSDKStateDTO>) statesListView.getAdapter();
        if (arrayAdapter != null) {
            BehanceSDKStateDTO currentSelectedItem = (BehanceSDKStateDTO) arrayAdapter.getSelectedLocationDTO();
            if (!currentSelectedItem.equals(stateDTO)) {
                arrayAdapter.setSelectedLocationDTO(stateDTO);
                arrayAdapter.notifyDataSetChanged();
            }
        }
    }

    private void displaySelectedCityItemInMainView() {
        selectedCityContainer.setVisibility(View.VISIBLE);
        selectedCityTxtView.setText(getSelectedCity().getDisplayName());
    }

    private void hideSelectedCityItemInMainView() {
        selectedCityDTO = null;
        selectedCityContainer.setVisibility(View.GONE);
    }

    private void setCitiesListAdapter(List<BehanceSDKCityDTO> result) {
        List<BehanceSDKCityDTO> citiesList;
        if (result != null && !result.isEmpty()) {
            citiesList = new ArrayList<BehanceSDKCityDTO>(result);
        } else {
            citiesList = new ArrayList<BehanceSDKCityDTO>();
        }
        BehanceSDKLocationFilterListItemArrayAdapter<BehanceSDKCityDTO> arrayAdapter = new BehanceSDKLocationFilterListItemArrayAdapter<BehanceSDKCityDTO>(getActivity(), selectedCityDTO,
                citiesList);
        citiesListView.setAdapter(arrayAdapter);
    }

    @Override
    public void onSaveInstanceState(Bundle arg0) {
        super.onSaveInstanceState(arg0);

        int displayedChild = viewFlipper.getDisplayedChild();
        arg0.putInt(INSTANCE_STATE_KEY_LAST_DISPLAYED_CHILD_INDEX, displayedChild);
        arg0.putSerializable(INSTANCE_STATE_KEY_SELECTED_COUNTRY, selectedCountryDTO);
        arg0.putSerializable(INSTANCE_STATE_KEY_SELECTED_STATE, selectedStateDTO);
        arg0.putSerializable(INSTANCE_STATE_KEY_SELECTED_CITY, selectedCityDTO);

        locationFiltersDataManager.removeListener(this);
    }

    @Override
    public void onDetach() {
        super.onDetach();
        locationFiltersDataManager.removeListener(this);
    }

    @Override
    public void onClick(View clickedView) {
        Object clickedItemTag = clickedView.getTag();

        if (clickedView.getId() == R.id.bsdkLocationFilterDialogCitiesSectionBackBtn
                || clickedView.getId() == R.id.bsdkLocationFilterDialogStatesSectionBackBtn
                || clickedView.getId() == R.id.bsdkLocationFilterDialogCountriesSectionBackBtn) {
            displayMainView();
            if (clickedView.getId() == R.id.bsdkLocationFilterDialogCitiesSectionBackBtn) {
                clearCitySearchResultsInDisplay();
            }
        } else if (clickedView.getId() == R.id.bsdkLocationFilterDialogCloseDialogBtn) {
            handleDoneClick();
        } else if (clickedItemTag instanceof ClickTagData) {
            if (clickedItemTag == ClickTagData.SHOW_COUNTRIES_VIEW) {
                displayCountriesView();
            } else if (clickedItemTag == ClickTagData.SHOW_STATES_VIEW) {
                displayStatesView();
            } else if (clickedItemTag == ClickTagData.SHOW_CITIES_VIEW) {
                displayCitiesView();
            }
        }
    }

    private void handleDoneClick() {
        if (selectedCityDTO == null || BehanceSDKCityDTO.ALL_CITIES_ID.equals(selectedCityDTO.getId()) || selectedCityDTO.getDisplayName() == null || selectedCityDTO.getDisplayName().length() <= 0)
            Toast.makeText(getActivity(), getResources().getString(R.string.bsdk_location_filter_dialog_city_empty_error), Toast.LENGTH_SHORT).show();
        else {
            notifyListenersAndCloseDialog();
        }
    }

    private void notifyListenersAndCloseDialog() {
        if (getCallbacks() != null) {
            getCallbacks().onLocationFilterDone(selectedCountryDTO, selectedStateDTO, selectedCityDTO);
        }
        getDialog().dismiss();
    }

    private void handleCountryListItemClick(AdapterView<?> parent, int position) {
        Object selectedObject = parent.getItemAtPosition(position);
        if (selectedObject instanceof BehanceSDKCountryDTO) {
            // hide selected state
            selectedStateContainer.setVisibility(View.GONE);
            selectedStateDTO = null;

            selectedCountryDTO = (BehanceSDKCountryDTO) selectedObject;
            selectedCountryTxtView.setText(selectedCountryDTO.getDisplayName());
            displayMainView();
            hideSelectedStateItemInMainView();
            hideSelectedCityItemInMainView();
            // if user selected Worldwide as option, don't load states
            String id = selectedCountryDTO.getId();
            if (BehanceSDKCountryDTO.COUNTRY_CODES_FOR_LOAD_STATES.contains(id)) {
                loadStates(id);
            } else if (!id.equals(BehanceSDKCountryDTO.WORLD_WIDE_COUNTRY_ID)) {
                displaySelectedCityItemInMainView();
            }
        }
    }

    private void handleStateListItemClick(AdapterView<?> parent, int position) {
        Object selectedObject = parent.getItemAtPosition(position);
        if (selectedObject instanceof BehanceSDKStateDTO) {
            selectedStateDTO = (BehanceSDKStateDTO) selectedObject;
            selectedStateTxtView.setText(selectedStateDTO.getDisplayName());
            displayMainView();
            selectedCityDTO = null;
            // if it's all states, don't display the City option
            if (!selectedStateDTO.getId().equals(BehanceSDKStateDTO.ALL_STATES_ID)) {
                displaySelectedCityItemInMainView();
            } else {
                hideSelectedCityItemInMainView();
            }
        }
    }

    private void handleCityListItemClick(AdapterView<?> parent, int position) {
        Object selectedObject = parent.getItemAtPosition(position);
        if (selectedObject instanceof BehanceSDKCityDTO) {
            selectedCityDTO = (BehanceSDKCityDTO) selectedObject;
            selectedCityTxtView.setText(selectedCityDTO.getDisplayName());
            displayMainView();
            clearCitySearchResultsInDisplay();
        }
    }

    private void displayMainView() {
        viewFlipper.setInAnimation(getActivity(), R.anim.bsdk_dialog_enter);
        viewFlipper.setOutAnimation(getActivity(), R.anim.bsdk_dialog_exit);
        viewFlipper.setDisplayedChild(MAIN_VIEW_DISPLAY_INDEX);
    }

    private void displayCountriesView() {
        if (!loadingInProgress) {
            viewFlipper.setInAnimation(getActivity(), R.anim.bsdk_dialog_enter);
            viewFlipper.setOutAnimation(getActivity(), R.anim.bsdk_dialog_exit);
            viewFlipper.setDisplayedChild(COUNTRIES_VIEW_DISPLAY_INDEX);
            if (locationFiltersDataManager.isGetCountriesTaskInProgress()) {
                showProgressSpinner();
            } else {
                updateSelectedCountryInCountriesListView(selectedCountryDTO);
                hideProgressSpinner();
            }
        }
    }

    private void displayStatesView() {
        if (!loadingInProgress) {
            updateSelectedStateInStatesListView(selectedStateDTO);
            viewFlipper.setInAnimation(getActivity(), R.anim.bsdk_dialog_enter);
            viewFlipper.setOutAnimation(getActivity(), R.anim.bsdk_dialog_exit);
            viewFlipper.setDisplayedChild(STATES_VIEW_DISPLAY_INDEX);
        }
    }

    private void displayCitiesView() {
        if (!loadingInProgress) {
            setCitiesListAdapter(null);
            viewFlipper.setInAnimation(getActivity(),  R.anim.bsdk_dialog_enter);
            viewFlipper.setOutAnimation(getActivity(),  R.anim.bsdk_dialog_exit);
            Animation inAnimation = viewFlipper.getInAnimation();
            // show keyboard after animation completes, else the animation will
            // not be smooth
            inAnimation.setAnimationListener(new Animation.AnimationListener() {

                @Override
                public void onAnimationStart(Animation animation) {
                }

                @Override
                public void onAnimationRepeat(Animation animation) {
                }

                @Override
                public void onAnimationEnd(Animation animation) {
                    citySearchStrEditText.requestFocus();
                    InputMethodManager imm = (InputMethodManager) getActivity().getSystemService(Context.INPUT_METHOD_SERVICE);
                    imm.showSoftInput(citySearchStrEditText, 0);
                }
            });
            viewFlipper.setDisplayedChild(CITIES_VIEW_DISPLAY_INDEX);
        }
    }

    private BehanceSDKCountryDTO getSelectedCountry() {
        if (selectedCountryDTO == null) {
            BehanceSDKCountryDTO country = getWorldWideCountryDTO();
            selectedCountryDTO = country;
        }
        return selectedCountryDTO;
    }

    private BehanceSDKCountryDTO getWorldWideCountryDTO() {
        BehanceSDKCountryDTO country = new BehanceSDKCountryDTO();
        country.setDisplayName(getResources().getString(R.string.bsdk_location_filter_dialog_country_label));
        country.setId(BehanceSDKCountryDTO.WORLD_WIDE_COUNTRY_ID);
        return country;
    }

    private BehanceSDKStateDTO getSelectedState() {
        if (selectedStateDTO == null) {
            BehanceSDKStateDTO stateDTO = getAllStatesDTO();
            selectedStateDTO = stateDTO;
        }
        return selectedStateDTO;
    }

    private BehanceSDKStateDTO getAllStatesDTO() {
        BehanceSDKStateDTO stateDTO = new BehanceSDKStateDTO();
        stateDTO.setDisplayName(getResources().getString(R.string.bsdk_location_filter_dialog_state_label));
        stateDTO.setId(BehanceSDKStateDTO.ALL_STATES_ID);
        return stateDTO;
    }

    private BehanceSDKCityDTO getSelectedCity() {
        if (selectedCityDTO == null) {
            BehanceSDKCityDTO cityDTO = getAllCitiesDTO();
            selectedCityDTO = cityDTO;
        }
        return selectedCityDTO;
    }

    private BehanceSDKCityDTO getAllCitiesDTO() {
        BehanceSDKCityDTO cityDTO = new BehanceSDKCityDTO();
        cityDTO.setDisplayName(getResources().getString(R.string.bsdk_location_filter_dialog_city_label));
        cityDTO.setId(BehanceSDKCityDTO.ALL_CITIES_ID);
        return cityDTO;
    }

    private void showProgressSpinner() {
        loadingInProgress = true;
        progressSpinner.setVisibility(View.VISIBLE);
    }

    private void hideProgressSpinner() {
        loadingInProgress = false;
        progressSpinner.setVisibility(View.INVISIBLE);
    }

    private void loadStates(String countryId) {
        showProgressSpinner();
        locationFiltersDataManager.loadStates(countryId);
    }

    @Override
    public void onCountriesLoadingSuccess(List<BehanceSDKCountryDTO> countries) {
        // we are creating a local copy, to add the top element without
        ArrayList<BehanceSDKCountryDTO> countriesList;
        if (countries != null && !countries.isEmpty()) {
            countriesList = new ArrayList<BehanceSDKCountryDTO>(countries);
        } else {
            countriesList = new ArrayList<BehanceSDKCountryDTO>();
        }

        BehanceSDKLocationFilterListItemArrayAdapter<BehanceSDKCountryDTO> arrayAdapter = new BehanceSDKLocationFilterListItemArrayAdapter<BehanceSDKCountryDTO>(getActivity(),
                selectedCountryDTO, countriesList);
        countriesListView.setAdapter(arrayAdapter);
        hideProgressSpinner();
    }

    @Override
    public void onCountriesLoadingFailure(Exception e) {
        hideProgressSpinner();
        logger.error(e, "Problem loading Countries list");
        Toast.makeText(getActivity(), getResources().getString(R.string.bsdk_location_filter_dialog_failed_to_load_error), Toast.LENGTH_SHORT).show();
    }

    @Override
    public void onStatesLoadingSuccess(String countryId, List<BehanceSDKStateDTO> result) {
        ArrayList<BehanceSDKStateDTO> statesList;
        if (result != null && !result.isEmpty()) {
            statesList = new ArrayList<BehanceSDKStateDTO>(result);
        } else {
            statesList = new ArrayList<BehanceSDKStateDTO>();
        }

        displaySelectedStateItemInMainView();
        BehanceSDKLocationFilterListItemArrayAdapter<BehanceSDKStateDTO> arrayAdapter = new BehanceSDKLocationFilterListItemArrayAdapter<BehanceSDKStateDTO>(getActivity(), selectedStateDTO,
                statesList);
        statesListView.setAdapter(arrayAdapter);
        hideProgressSpinner();
    }

    @Override
    public void onStatesLoadingFailure(String countryId, Exception e) {
        logger.error(e, "Problem loading States for [Country id - %s]", countryId);
        Toast.makeText(getActivity(), getResources().getString(R.string.bsdk_location_filter_dialog_failed_to_load_error), Toast.LENGTH_SHORT).show();
        hideProgressSpinner();
        hideSelectedStateItemInMainView();
        displaySelectedCityItemInMainView();
    }

    private void searchForCities(String searchString) {
        showProgressSpinner();
        String selectedCountryId = null;
        if (selectedCountryDTO != null) {
            selectedCountryId = selectedCountryDTO.getId();
        }

        String selectedStateId = null;
        if (selectedStateDTO != null) {
            selectedStateId = selectedStateDTO.getId();
        }
        locationFiltersDataManager.searchForCities(selectedCountryId, selectedStateId, searchString);
    }

    private void clearCitySearchResultsInDisplay() {
        // if the loading was in progress, when user clicked
        hideProgressSpinner();

        // hide the keyboard
        InputMethodManager imm = (InputMethodManager) getActivity().getSystemService(Context.INPUT_METHOD_SERVICE);
        imm.hideSoftInputFromWindow(rootView.getWindowToken(), 0);

        // clear the search term and clear list
        citySearchStrEditText.setText("");
        citiesListView.setAdapter(null);
    }

    @Override
    public void onCitiesLoadingSuccess(BehanceSDKGetCitiesAsyncTaskParams params, List<BehanceSDKCityDTO> result) {
        if (viewFlipper.getDisplayedChild() == CITIES_VIEW_DISPLAY_INDEX) {
            setCitiesListAdapter(result);
            hideProgressSpinner();
        }
    }

    @Override
    public void onCitiesLoadingFailure(BehanceSDKGetCitiesAsyncTaskParams params, Exception e) {
        logger.error(e, "Problem loading Cities for [Country id - %s] [State id - %s] [Search Str - %s]", params.getCountryId(), params.getStateId(),
                params.getCitySearchStr());
        Toast.makeText(getActivity(), getResources().getString(R.string.bsdk_location_filter_dialog_failed_to_load_error), Toast.LENGTH_SHORT).show();
        if (viewFlipper.getDisplayedChild() == CITIES_VIEW_DISPLAY_INDEX) {
            hideProgressSpinner();
        }
    }

    public Callbacks getCallbacks() {
        return callbacks;
    }

    public void setCallbacks(Callbacks callbacks) {
        this.callbacks = callbacks;
    }

    private enum ClickTagData {
        SHOW_COUNTRIES_VIEW, SHOW_STATES_VIEW, SHOW_CITIES_VIEW;
    }

    @Override
    public void onStart() {
        super.onStart();
        if (getDialog() != null && getActivity() != null) {
            if (getActivity().getResources().getBoolean(R.bool.bsdk_big_screen)) {
                int windowWidth = getActivity().getResources().getDimensionPixelSize(R.dimen.global_filter_dialog_width);
                int windowHeight = getActivity().getResources().getDimensionPixelSize(R.dimen.global_filter_dialog_height);
                getDialog().getWindow().setLayout(windowWidth, windowHeight);
            }
        }
    }
}
