/*************************************************************************
 * ADOBE CONFIDENTIAL
 * ___________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by all applicable intellectual property
 * laws, including trade secret and copyright laws.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.behance.sdk.ui.fragments;

import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.EnumSet;
import java.util.List;

import android.app.Activity;
import android.content.DialogInterface;
import android.content.DialogInterface.OnKeyListener;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.provider.MediaStore;
import android.support.v4.app.DialogFragment;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.GridView;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.ViewFlipper;

import com.adobe.creativesdk.foundation.storage.AdobeAssetMimeTypes;
import com.adobe.creativesdk.foundation.storage.AdobeUXAssetBrowserOption;
import com.behance.sdk.BehanceSDKImageDimensions;
import com.behance.sdk.BehanceSDKImageSelectorOptions;
import com.behance.sdk.R;
import com.behance.sdk.dto.BehanceSDKAlbumDTO;
import com.behance.sdk.enums.BehanceSDKAlbumType;
import com.behance.sdk.files.LocalImageLoader;
import com.behance.sdk.fragments.headless.BehanceSDKImageSelectorHeadlessFragment;
import com.behance.sdk.logger.ILogger;
import com.behance.sdk.logger.LoggerFactory;
import com.behance.sdk.project.modules.CreativeCloudAssetModule;
import com.behance.sdk.project.modules.ImageModule;
import com.behance.sdk.ui.activities.BehanceSDKCCLauncherActivity;
import com.behance.sdk.ui.activities.BehanceSDKCreateWIPWorkflowActivity;
import com.behance.sdk.ui.adapters.BehanceSDKAlbumGridArrayAdapter;
import com.behance.sdk.ui.adapters.BehanceSDKAlbumImageGridArrayAdapter;
import com.behance.sdk.util.BehanceSDKPermissionHelper;
import com.behance.sdk.util.BehanceSDKUtils;

/**
 * @author Sujit Reddy G
 *
 */
public class BehanceSDKImageSelectorDialogFragment extends DialogFragment implements OnClickListener, OnKeyListener,
		BehanceSDKImageSelectorHeadlessFragment.Callbacks {

	private static final ILogger logger = LoggerFactory.getLogger(BehanceSDKImageSelectorDialogFragment.class);

	private static final String HEADLESS_FRAGMENT_TAG = "IMAGE_SELECTOR_FRAGMENT_HEADLESS_FRAGMENT_TAG_SELECT_IMAGES";

	private static final String BUNDLE_KEY_LAST_DISPLAYED_ALBUM_INDEX = "BUNDLE_KEY_LAST_DISPLAYED_ALBUM_INDEX";
	private static final String BUNDLE_KEY_LAST_DISPLAYED_CHILD_INDEX = "BUNDLE_KEY_LAST_DISPLAYED_CHILD_INDEX";
	private static final String BUNDLE_KEY_SELECTED_IMAGE_MODULES_LIST = "BUNDLE_KEY_SELECTED_IMAGE_MODULES_LIST";
	private static final String BUNDLE_KEY_CAMERA_IMAGE_URI = "BUNDLE_KEY_CAMERA_IMAGE_URI";

	private static final int VIEW_FLIPPER_ALBUMS_VIEW_INDEX = 0;
	private static final int VIEW_FLIPPER_ALBUM_IMAGES_VIEW_INDEX = 1;

	public static final String ARGS_MAX_NUMBER_OF_IMAGES = "ARGS_MAX_NUMBER_OF_IMAGES";
	public static final String ARGS_MIN_IMAGE_WIDTH = "ARGS_MIN_IMAGE_WIDTH";
	public static final String ARGS_MIN_IMAGE_HEIGHT = "ARGS_MIN_IMAGE_HEIGHT";
	public static final String ARGS_MAX_IMAGE_WIDTH = "ARGS_MAX_IMAGE_WIDTH";
	public static final String ARGS_MAX_IMAGE_HEIGHT = "ARGS_MAX_IMAGE_HEIGHT";
	public static final String ARGS_MAX_IMAGE_SIZE_BYTES = "ARGS_MAX_IMAGE_SIZE_BYTES";
	public static final String ARGS_MIN_IMAGE_SIZE_BYTES = "ARGS_MIN_IMAGE_SIZE_BYTES";
	public static final String ARGS_ALLOWED_FILE_EXTENSIONS_LIST = "ARGS_ALLOWED_FILE_EXTENSIONS_LIST";
	public static final String ARGS_PROHIBITED_FILE_EXTENSIONS_LIST = "ARGS_PROHIBITED_FILE_EXTENSIONS_LIST";
	public static final String ARGS_ALBUMS_WITH_IMAGE_SELECTION_SOURCES = "ARGS_ALBUMS_WITH_IMAGE_SELECTION_SOURCES";
	public static final String ARGS_IMAGE_VALITATOR_TYPE = "ARGS_IMAGE_VALITATOR_TYPE";
	public static final String ARGS_HIDE_CC_ASSET_BROWSER = "ARGS_HIDE_CC_ASSET_BROWSER";

	private static final int ACTIVITY_RESULT_REQUEST_CODE_PICK_PICTURE_FROM_CAMERA = 1;
	private static final int ACTIVITY_RESULT_REQUEST_CODE_CC_ASSET_BROWSER = 6789;

	private ViewFlipper viewFlipper;
	private boolean ccAssetBrowserDisabled;
	private GridView albumsGridView;
	private GridView albumImagesGridView;
	private TextView titleBarTitleTxtView;
	private TextView titleBarActionBtnTxtView;
	private View progressSpinner;

	private int lastDisplayedAlbumIndex;
	private Uri selectedCameraImageUri;
	private ArrayList<ImageModule> selectedImageModulesList;
	private int maxNumberOfImages;
	private int minImageWidth;
	private int minImageHeight;
	private int maxImageWidth;
	private int maxImageHeight;
	private long minImageSizeBytes;
	private long maxImageSizeBytes;
	private boolean showImageSelectionSources;
	private ArrayList<String> allowedFileExtensionsList;
	private ArrayList<String> prohibitedFileExtensionsList;

	private BehanceSDKImageSelectorHeadlessFragment headlessFragment;
	private Callbacks callbacks;
	private String imageValidatorType;

	public interface Callbacks {
		public void onImageSelectorSelectionFinished(List<ImageModule> selectedImageModulesList);

		public void onImageSelectorSelectionCanceled();

		public void onImageSelectorSelectionBackPressed();

		public void onImageSelectorOnError(Exception e);
	}

	public static BehanceSDKImageSelectorDialogFragment getInstance(Callbacks callbacks, BehanceSDKImageSelectorOptions options) {
		BehanceSDKImageSelectorDialogFragment fragmentInstance = new BehanceSDKImageSelectorDialogFragment();
		fragmentInstance.setCallbacks(callbacks);
		Bundle args = new Bundle();
		if (options != null) {
			args.putInt(ARGS_MAX_NUMBER_OF_IMAGES, options.getMaxNumberOfImages());
			BehanceSDKImageDimensions minImageDimensions = options.getMinImageDimensions();
			if (minImageDimensions != null) {
				args.putInt(ARGS_MIN_IMAGE_WIDTH, minImageDimensions.getWidth());
				args.putInt(ARGS_MIN_IMAGE_HEIGHT, minImageDimensions.getHeight());
			}

			BehanceSDKImageDimensions maxImageDimensions = options.getMaxImageDimensions();
			if (maxImageDimensions != null) {
				args.putInt(ARGS_MAX_IMAGE_WIDTH, maxImageDimensions.getWidth());
				args.putInt(ARGS_MAX_IMAGE_HEIGHT, maxImageDimensions.getHeight());
			}
			args.putBoolean(ARGS_ALBUMS_WITH_IMAGE_SELECTION_SOURCES, options.isShowImageImageSelectionSources());
			args.putBoolean(ARGS_HIDE_CC_ASSET_BROWSER,options.isCCAssetBrowserDisabled());


			args.putLong(ARGS_MIN_IMAGE_SIZE_BYTES, options.getMinImageSizeInBytes());
			args.putLong(ARGS_MAX_IMAGE_SIZE_BYTES, options.getMaxImageSizeInBytes());

			args.putStringArrayList(ARGS_ALLOWED_FILE_EXTENSIONS_LIST, options.getAllowedFileExtensions());
			args.putStringArrayList(ARGS_PROHIBITED_FILE_EXTENSIONS_LIST, options.getProhibitedFileExtensions());
			args.putString(ARGS_IMAGE_VALITATOR_TYPE, options.getImageValidatorType());
		} else {
			args.putInt(ARGS_MAX_NUMBER_OF_IMAGES, BehanceSDKImageSelectorOptions.UNLIMITED_NUM_OF_IMAGES);
		}
		fragmentInstance.setArguments(args);
		return fragmentInstance;
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setStyle(DialogFragment.STYLE_NO_FRAME, R.style.BsdkImageSelectorViewTheme);
	}

	@SuppressWarnings("unchecked")
	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		super.onCreateView(inflater, container, savedInstanceState);
		View rootView = inflater.inflate(R.layout.bsdk_dialog_fragment_image_selector, container, false);

		lastDisplayedAlbumIndex = 0;
		int childIndexToDisplay = 0;
		if (savedInstanceState != null) {
			childIndexToDisplay = savedInstanceState.getInt(BUNDLE_KEY_LAST_DISPLAYED_CHILD_INDEX, 0);
			lastDisplayedAlbumIndex = savedInstanceState.getInt(BUNDLE_KEY_LAST_DISPLAYED_ALBUM_INDEX, 0);
			Serializable serializedImageModulesList = savedInstanceState.getSerializable(BUNDLE_KEY_SELECTED_IMAGE_MODULES_LIST);
			if (serializedImageModulesList instanceof ArrayList<?>) {
				selectedImageModulesList = (ArrayList<ImageModule>) serializedImageModulesList;
			}
			selectedCameraImageUri = savedInstanceState.getParcelable(BUNDLE_KEY_CAMERA_IMAGE_URI);
		}

		if (selectedImageModulesList == null) {
			selectedImageModulesList = new ArrayList<ImageModule>();
		}

		Bundle arguments = getArguments();
		maxNumberOfImages = BehanceSDKImageSelectorOptions.UNLIMITED_NUM_OF_IMAGES;
		minImageHeight = -1;
		minImageWidth = -1;
		maxImageHeight = -1;
		maxImageWidth = -1;
		minImageSizeBytes = -1;
		maxImageSizeBytes = -1;
		imageValidatorType =null;
		if (arguments != null) {
			maxNumberOfImages = arguments.getInt(ARGS_MAX_NUMBER_OF_IMAGES, BehanceSDKImageSelectorOptions.UNLIMITED_NUM_OF_IMAGES);
			minImageWidth = arguments.getInt(ARGS_MIN_IMAGE_WIDTH, -1);
			minImageHeight = arguments.getInt(ARGS_MIN_IMAGE_HEIGHT, -1);
			maxImageWidth = arguments.getInt(ARGS_MAX_IMAGE_WIDTH, -1);
			maxImageHeight = arguments.getInt(ARGS_MAX_IMAGE_HEIGHT, -1);
			showImageSelectionSources = arguments.getBoolean(ARGS_ALBUMS_WITH_IMAGE_SELECTION_SOURCES, true);
			minImageSizeBytes = arguments.getLong(ARGS_MIN_IMAGE_SIZE_BYTES, -1);
			maxImageSizeBytes = arguments.getLong(ARGS_MAX_IMAGE_SIZE_BYTES, -1);
			allowedFileExtensionsList = arguments.getStringArrayList(ARGS_ALLOWED_FILE_EXTENSIONS_LIST);
			prohibitedFileExtensionsList = arguments.getStringArrayList(ARGS_PROHIBITED_FILE_EXTENSIONS_LIST);
			imageValidatorType = arguments.getString(ARGS_IMAGE_VALITATOR_TYPE);
			ccAssetBrowserDisabled = arguments.getBoolean(ARGS_HIDE_CC_ASSET_BROWSER,false);

		}

		headlessFragment = (BehanceSDKImageSelectorHeadlessFragment) getActivity().getSupportFragmentManager().findFragmentByTag(
				HEADLESS_FRAGMENT_TAG);
		if (headlessFragment == null) {
			headlessFragment = new BehanceSDKImageSelectorHeadlessFragment();
			getActivity().getSupportFragmentManager().beginTransaction().add(headlessFragment, HEADLESS_FRAGMENT_TAG).commit();
		}
		headlessFragment.setCallbacks(this);

		viewFlipper = (ViewFlipper) rootView.findViewById(R.id.bsdkImageSelectorViewFlipper);
		progressSpinner = rootView.findViewById(R.id.bsdkImageSelectorViewProgressSpinner);

		View backBtn = rootView.findViewById(R.id.bsdkImageSelectorViewBackBtnImageView);
		backBtn.setOnClickListener(this);

		titleBarActionBtnTxtView = (TextView) rootView.findViewById(R.id.bsdkImageSelectorViewTitlebarActionBtnTxtView);
		titleBarActionBtnTxtView.setOnClickListener(this);

		titleBarTitleTxtView = (TextView) rootView.findViewById(R.id.bsdkImageSelectorViewTitlebarTitleTxtView);

		albumsGridView = (GridView) rootView.findViewById(R.id.bsdkImageSelectorViewAlbumsGridView);
		albumsGridView.setOnItemClickListener(new AdapterView.OnItemClickListener() {

			@Override
			public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
				onAlbumGridItemClick(parent, view, position, id);
			}
		});

		albumImagesGridView = (GridView) rootView.findViewById(R.id.bsdkImageSelectorViewAlbumImagesGridView);
		albumImagesGridView.setOnItemClickListener(new AdapterView.OnItemClickListener() {

			@Override
			public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
				onAlbumImageGridItemClick(parent, view, position, id);
			}
		});

		if (maxNumberOfImages == BehanceSDKImageSelectorOptions.UNLIMITED_NUM_OF_IMAGES) {
			titleBarActionBtnTxtView.setText(R.string.bsdk_image_selector_view_titlebar_action_btn_multiple_photos_cancel_label);
		} else {
			titleBarActionBtnTxtView.setText(R.string.bsdk_image_selector_view_titlebar_action_btn_cancel_label);
		}

		if (lastDisplayedAlbumIndex >= 0 && childIndexToDisplay == VIEW_FLIPPER_ALBUM_IMAGES_VIEW_INDEX) {
			BehanceSDKAlbumDTO albumDTO = LocalImageLoader.getFromAlbumWithImageSelectionSources(getActivity(), lastDisplayedAlbumIndex);
			if (albumDTO != null && albumDTO.getAlbumType() == BehanceSDKAlbumType.DEVICE_ALBUM) {
				displaySelectedAlbumImagesView(albumDTO, false);
			} else {
				viewFlipper.setDisplayedChild(VIEW_FLIPPER_ALBUMS_VIEW_INDEX);
			}
		} else {
			viewFlipper.setDisplayedChild(childIndexToDisplay);
		}

		getDialog().setOnKeyListener(this);

		if (headlessFragment.isAlbumsLoadingInProgress()) {
			showProgressSpinner();
		} else {
			loadAlbums();
		}
		return rootView;
	}

	private void loadAlbums() {
		if (headlessFragment.getAlbumsList() != null && !headlessFragment.getAlbumsList().isEmpty()) {
			setAlbumsGridAdapter();
			hideProgressSpinner();
		} else {
			loadAlbumsFromDevice();
		}
	}

	private void loadAlbumsFromDevice() {
		showProgressSpinner();
		headlessFragment.loadAlbums(getActivity().getApplicationContext(), showImageSelectionSources);
	}

	@Override
	public void onDetach() {
		super.onDetach();
		if (headlessFragment != null) {
			headlessFragment.setCallbacks(null);
		}
	}

	@Override
	public void onSaveInstanceState(Bundle arg0) {
		super.onSaveInstanceState(arg0);
		arg0.putInt(BUNDLE_KEY_LAST_DISPLAYED_CHILD_INDEX, viewFlipper.getDisplayedChild());
		arg0.putInt(BUNDLE_KEY_LAST_DISPLAYED_ALBUM_INDEX, lastDisplayedAlbumIndex);
		arg0.putSerializable(BUNDLE_KEY_SELECTED_IMAGE_MODULES_LIST, selectedImageModulesList);
		if (null != selectedCameraImageUri)
			arg0.putParcelable(BUNDLE_KEY_CAMERA_IMAGE_URI, selectedCameraImageUri);
	}

	private void setAlbumsGridAdapter() {
		List<BehanceSDKAlbumDTO> albumsList = headlessFragment.getAlbumsList();
		if (albumsList != null) {
			if (ccAssetBrowserDisabled) {
				for (BehanceSDKAlbumDTO dto : albumsList) {
					if ((dto.getAlbumType()).equals(BehanceSDKAlbumType.CREATIVE_CLOUD)) {
						albumsList.remove(dto);
						break;
					}
				}
			}
			BehanceSDKAlbumGridArrayAdapter arrayAdapter = new BehanceSDKAlbumGridArrayAdapter(getActivity(), albumsList);
			albumsGridView.setAdapter(arrayAdapter);
		}
	}


	@Override
	public void onClick(View v) {
		if (v.getId() == R.id.bsdkImageSelectorViewTitlebarActionBtnTxtView) {
			handleActionBtnClick();
		} else if (v.getId() == R.id.bsdkImageSelectorViewBackBtnImageView) {
			handleBackBtnClick();
		}
	}

	private void handleBackBtnClick() {
		int displayedChildIndex = viewFlipper.getDisplayedChild();
		if (displayedChildIndex == VIEW_FLIPPER_ALBUM_IMAGES_VIEW_INDEX) {
			displayAlbumsView();
		} else {
			notifyCallbacksOnBackPressed();
			closeThisDialog();
		}
	}

	private void handleActionBtnClick() {
		int displayedChildIndex = viewFlipper.getDisplayedChild();
		if (displayedChildIndex == VIEW_FLIPPER_ALBUM_IMAGES_VIEW_INDEX) {
			if (maxNumberOfImages > 1 || maxNumberOfImages == BehanceSDKImageSelectorOptions.UNLIMITED_NUM_OF_IMAGES) {
				if (selectedImageModulesList.isEmpty()) {
					notifyCallbacksOnCancel();
					closeThisDialog();
				} else {
					notifyCallbacksOnImageSelection(selectedImageModulesList);
					closeThisDialog();
				}
			} else {
				notifyCallbacksOnCancel();
				closeThisDialog();
			}
		} else {
			notifyCallbacksOnCancel();
			closeThisDialog();
		}
	}

	private void onAlbumImageGridItemClick(AdapterView<?> parent, View view, int position, long id) {
		Object selectedItemObject = parent.getItemAtPosition(position);
		if (selectedItemObject instanceof ImageModule) {
			ImageModule imageModule = (ImageModule) selectedItemObject;
			if (imageMatchesRequirements(imageModule)) {
				File imageFile = imageModule.getFile();
				if (imageFile != null) {
					if (selectedImageModulesList.contains(imageModule)) {
						selectedImageModulesList.remove(imageModule);
					} else {
						selectedImageModulesList.add(imageModule);
					}

					Object adapterObject = parent.getAdapter();
					if (adapterObject instanceof BehanceSDKAlbumImageGridArrayAdapter) {
						BehanceSDKAlbumImageGridArrayAdapter albumImageAdapter = (BehanceSDKAlbumImageGridArrayAdapter) adapterObject;
						albumImageAdapter.setSelectedImageModulesList(selectedImageModulesList);
						albumImageAdapter.notifyDataSetChanged();
					}

					if (maxNumberOfImages != BehanceSDKImageSelectorOptions.UNLIMITED_NUM_OF_IMAGES
							&& selectedImageModulesList.size() >= maxNumberOfImages) {
						notifyCallbacksOnImageSelection(selectedImageModulesList);
						closeThisDialog();
					}

					if (!selectedImageModulesList.isEmpty()) {
						titleBarActionBtnTxtView.setText(R.string.bsdk_image_selector_view_titlebar_action_btn_multiple_photos_done_label);
					} else {
						titleBarActionBtnTxtView.setText(R.string.bsdk_image_selector_view_titlebar_action_btn_multiple_photos_cancel_label);
					}
				}
			}
		}
	}

	private boolean imageMatchesRequirements(ImageModule imageModule) {

		String fileName = imageModule.getName();
		String fileExtension = BehanceSDKUtils.getFileExtension(fileName);
		if (fileExtension != null && fileExtension.length() > 0) {
			if (prohibitedFileExtensionsList != null && !prohibitedFileExtensionsList.isEmpty()) {
				if (prohibitedFileExtensionsList.contains(fileExtension)) {
					String msgToDisplay = getString(R.string.bsdk_image_selector_view_image_file_type_not_allowed_msg, fileName,
							prohibitedFileExtensionsList.toString());
					displayToastMessage(msgToDisplay);
					return false;
				}
			}

			if (allowedFileExtensionsList != null && !allowedFileExtensionsList.isEmpty()) {
				if (!allowedFileExtensionsList.contains(fileExtension)) {
					String msgToDisplay = getString(R.string.bsdk_image_selector_view_image_file_type_allowed_msg, fileName,
							allowedFileExtensionsList.toString());
					displayToastMessage(msgToDisplay);
					return false;
				}
			}
		} else {
			String msgToDisplay = getString(R.string.bsdk_image_selector_view_image_file_type_invalid_msg, fileName);
			displayToastMessage(msgToDisplay);
			return false;
		}

		int imageWidth = imageModule.getWidth();
		int imageHeight = imageModule.getHeight();
		long imageSizeInBytes = imageModule.getSizeInBytes();

		if (imageSizeInBytes > 0) {
			if (minImageSizeBytes > 0 && imageSizeInBytes < minImageSizeBytes) {
				String sizeInMBStr = (minImageSizeBytes / (1024 * 1024)) + " MB";
				String msgToDisplay = getString(R.string.bsdk_image_selector_view_image_size_less_than_min_required_msg, sizeInMBStr);
				displayToastMessage(msgToDisplay);
				return false;
			}

			if (maxImageSizeBytes > 0 && imageSizeInBytes > maxImageSizeBytes) {
				String sizeInMBStr = (maxImageSizeBytes / (1024 * 1024)) + " MB";
				String msgToDisplay = getString(R.string.bsdk_image_selector_view_image_size_more_than_max_allowed_msg, sizeInMBStr);
				displayToastMessage(msgToDisplay);
				return false;
			}
		} else {
			String msgToDisplay = getString(R.string.bsdk_image_selector_view_image_size_invalid_msg);
			displayToastMessage(msgToDisplay);
			return false;
		}

		if (imageWidth > 0 && imageHeight > 0) {
			if (minImageWidth > 0 && minImageHeight > 0) {
				if (imageWidth < minImageWidth || imageHeight < minImageHeight) {
					String msgToDisplay = getString(R.string.bsdk_image_selector_view_image_dimensions_less_than_min_required_msg, minImageWidth,
							minImageHeight, imageWidth, imageHeight);
					displayToastMessage(msgToDisplay);
					return false;
				}
			}

			if (maxImageWidth > 0 && maxImageHeight > 0) {
				if (imageWidth > maxImageWidth || imageHeight > maxImageHeight) {
					String msgToDisplay = getString(R.string.bsdk_image_selector_view_image_dimensions_more_than_max_allowed_msg, maxImageWidth,
							maxImageHeight, imageWidth, imageHeight);
					displayToastMessage(msgToDisplay);
					return false;
				}
			}
			return true;
		} else {
			String msgToDisplay = getString(R.string.bsdk_image_selector_view_image_dimensions_invalid_msg);
			displayToastMessage(msgToDisplay);
			return false;
		}
	}

	private void displayToastMessage(String msgToDisplay) {
		if (msgToDisplay != null) {
			Toast.makeText(getActivity(), msgToDisplay, Toast.LENGTH_LONG).show();
		}
	}

	private void notifyCallbacksOnCameraImageCaptureSuccess(ImageModule imageModule) {
		List<ImageModule> modulesList = new ArrayList<ImageModule>();
		modulesList.add(imageModule);
		notifyCallbacksOnImageSelection(modulesList);
	}

	private void notifyCallbacksOnImageSelection(final List<ImageModule> imageModulesList) {
		clearAlbumsCache();
		if (callbacks != null) {
			viewFlipper.post(new Runnable() {

				@Override
				public void run() {
					callbacks.onImageSelectorSelectionFinished(imageModulesList);
				}
			});
		}
	}

	private void clearAlbumsCache() {
		LocalImageLoader.clearCachedAlbumAndImagesData();
		if (headlessFragment != null) {
			headlessFragment.clearAlbumsCache();
		}
	}

	private void notifyCallbacksOnCancel() {
		clearAlbumsCache();
		if (callbacks != null) {
			viewFlipper.post(new Runnable() {

				@Override
				public void run() {
					callbacks.onImageSelectorSelectionCanceled();
				}
			});
		}
	}

	private void notifyCallbacksOnBackPressed() {
		clearAlbumsCache();
		if (callbacks != null) {
			viewFlipper.post(new Runnable() {

				@Override
				public void run() {
					callbacks.onImageSelectorSelectionBackPressed();
				}
			});
		}
	}

	private void notifyCallbacksOnError(final Exception e) {
		clearAlbumsCache();
		if (callbacks != null) {
			viewFlipper.post(new Runnable() {

				@Override
				public void run() {
					callbacks.onImageSelectorOnError(e);
				}
			});
		}
	}

	private void launchCamera(){
		if(BehanceSDKPermissionHelper.checkPermission(getActivity(), BehanceSDKPermissionHelper.REQUEST_CODE_LAUNCH_CAMERA)) {
			launchCameraToCapturePicture();
		}
		else{
			if (Build.VERSION.SDK_INT >= 23) {
				// Marshmallow+
				requestPermissions(BehanceSDKPermissionHelper.getPermissionFromRequestCode(BehanceSDKPermissionHelper.REQUEST_CODE_LAUNCH_CAMERA,getActivity()),BehanceSDKPermissionHelper.REQUEST_CODE_LAUNCH_CAMERA);
			}
		}

	}

	private void onAlbumGridItemClick(AdapterView<?> parent, View view, int position, long id) {
		Object selectedItemObject = parent.getItemAtPosition(position);
		if (selectedItemObject instanceof BehanceSDKAlbumDTO) {
			lastDisplayedAlbumIndex = position;
			BehanceSDKAlbumDTO selectedImageAlbum = (BehanceSDKAlbumDTO) parent.getItemAtPosition(position);
			if (selectedImageAlbum.getAlbumType() == BehanceSDKAlbumType.DEVICE_ALBUM) {
				selectedImageModulesList.clear();
				displaySelectedAlbumImagesView(selectedImageAlbum, true);
			} else if (selectedImageAlbum.getAlbumType() == BehanceSDKAlbumType.CAMERA) {
				launchCamera();
			} else if (selectedImageAlbum.getAlbumType() == BehanceSDKAlbumType.CREATIVE_CLOUD) {
				launchCreativeCloudAssetBrowser();
			}
		}
	}

	private void displayAlbumsView() {
		titleBarTitleTxtView.setText(R.string.bsdk_image_selector_view_titlebar_title_albums_txt);
		// when user navigates back to albums view, images selected is considered invalid
		// so, we change the action button label to Cancel or Skip
		if (maxNumberOfImages == BehanceSDKImageSelectorOptions.UNLIMITED_NUM_OF_IMAGES) {
			titleBarActionBtnTxtView.setText(R.string.bsdk_image_selector_view_titlebar_action_btn_multiple_photos_cancel_label);
		} else {
			titleBarActionBtnTxtView.setText(R.string.bsdk_image_selector_view_titlebar_action_btn_cancel_label);
		}

		viewFlipper.setInAnimation(null);
		viewFlipper.setOutAnimation(getActivity(), R.anim.bsdk_dialog_exit);
		viewFlipper.setDisplayedChild(VIEW_FLIPPER_ALBUMS_VIEW_INDEX);
	}

	private void displaySelectedAlbumImagesView(BehanceSDKAlbumDTO albumDTO, boolean animateViewDisplay) {
		List<ImageModule> selectedAlbumImages = albumDTO.getImages();
		if (selectedAlbumImages != null && !selectedAlbumImages.isEmpty()) {

			BehanceSDKAlbumImageGridArrayAdapter albumImagesGridAdapter = new BehanceSDKAlbumImageGridArrayAdapter(getActivity(), selectedAlbumImages);
			albumImagesGridAdapter.setSelectedImageModulesList(selectedImageModulesList);
			albumImagesGridView.setAdapter(albumImagesGridAdapter);

			if (maxNumberOfImages == BehanceSDKImageSelectorOptions.UNLIMITED_NUM_OF_IMAGES) {
				titleBarTitleTxtView.setText(R.string.bsdk_image_selector_view_titlebar_title_choose_photos_txt);
				if (!selectedImageModulesList.isEmpty()) {
					titleBarActionBtnTxtView.setText(R.string.bsdk_image_selector_view_titlebar_action_btn_multiple_photos_done_label);
				} else {
					titleBarActionBtnTxtView.setText(R.string.bsdk_image_selector_view_titlebar_action_btn_multiple_photos_cancel_label);
				}
			} else {
				// if we get a requirement where more then one image selection
				// is supported, we need a different string
				titleBarTitleTxtView.setText(R.string.bsdk_image_selector_view_titlebar_title_choose_single_photo_txt);
				titleBarActionBtnTxtView.setText(R.string.bsdk_image_selector_view_titlebar_action_btn_cancel_label);
			}

			if (animateViewDisplay) {
				viewFlipper.setInAnimation(getActivity(), R.anim.bsdk_dialog_enter);
				viewFlipper.setOutAnimation(null);
			} else {
				viewFlipper.setInAnimation(null);
				viewFlipper.setOutAnimation(null);
			}
			viewFlipper.setDisplayedChild(VIEW_FLIPPER_ALBUM_IMAGES_VIEW_INDEX);
		} else {
			String messageStr = getResources().getString(R.string.bsdk_image_selector_view_empty_album_msg, albumDTO.getName());
			Toast.makeText(getActivity(), messageStr, Toast.LENGTH_SHORT).show();
			displayAlbumsView();
		}
	}

	private void launchCreativeCloudAssetBrowser() {
		Activity context = getActivity();
		Intent detailsIntent = new Intent(context, BehanceSDKCCLauncherActivity.class);
		EnumSet<AdobeAssetMimeTypes> mimeTypes = BehanceSDKUtils.getAdobeAssetMimeTypes(allowedFileExtensionsList);
		if(null != mimeTypes)
			detailsIntent.putExtra(BehanceSDKCCLauncherActivity.ARGS_ALLOWED_MIME_TYPES, mimeTypes);
		EnumSet<AdobeUXAssetBrowserOption> browserOptions;
		if(maxNumberOfImages != 1){
			browserOptions = EnumSet.of(AdobeUXAssetBrowserOption.ENABLE_MULTI_SELECT,AdobeUXAssetBrowserOption.SHOW_MULTI_SELECT_ON_POPUP);
			detailsIntent.putExtra(BehanceSDKCCLauncherActivity.ARGS_ASSET_BROWSER_OPTIONS, browserOptions);
		}
		if(null != imageValidatorType)
			detailsIntent.putExtra(BehanceSDKCCLauncherActivity.ARGS_IMAGE_VALIDATOR_TYPE, imageValidatorType);
		startActivityForResult(detailsIntent, ACTIVITY_RESULT_REQUEST_CODE_CC_ASSET_BROWSER);
	}

	private void launchCameraToCapturePicture() {
		try {
			File photoFile = BehanceSDKUtils.createTempImageFile();
			Intent takePictureIntent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
			// Ensure that there's a camera activity to handle the intent
			if (takePictureIntent.resolveActivity(getActivity().getPackageManager()) != null) {
				// Continue only if the File was successfully created
				if (photoFile != null) {

					selectedCameraImageUri = Uri.fromFile(photoFile);
					takePictureIntent.putExtra(MediaStore.EXTRA_OUTPUT, selectedCameraImageUri);
					startActivityForResult(takePictureIntent, ACTIVITY_RESULT_REQUEST_CODE_PICK_PICTURE_FROM_CAMERA);
				}
			} else {
				logger.error("Camera intent not found");
				Toast.makeText(getActivity(), R.string.bsdk_add_wip_pick_source_view_camera_launch_error, Toast.LENGTH_LONG).show();
			}
		} catch (IOException ex) {
			logger.error(ex, "Problem creating temp file to capture image");
			Toast.makeText(getActivity(), R.string.bsdk_add_wip_pick_source_view_camera_launch_error, Toast.LENGTH_LONG).show();
		}
	}

	@Override
	public void onActivityResult(int requestCode, int resultCode, Intent data) {

		switch (requestCode){
			case ACTIVITY_RESULT_REQUEST_CODE_PICK_PICTURE_FROM_CAMERA:
				if (resultCode == Activity.RESULT_OK) {
					logger.debug("Camera Activity returned with ok result. [Photo Uri - %s]", selectedCameraImageUri);
					if (selectedCameraImageUri != null) {
						File cameraImageFile = new File(selectedCameraImageUri.getPath());
						ImageModule selectedCameraImageModule = new ImageModule(cameraImageFile);
						notifyCallbacksOnCameraImageCaptureSuccess(selectedCameraImageModule);
						// add the new image to the gallery and reload the images.
						BehanceSDKUtils.addImageToGallery(getActivity(), cameraImageFile.getAbsolutePath());
						clearAlbumsCache();
						loadAlbums();
						closeThisDialog();
					}
				} else {
					logger.debug("Camera Activity returned with not ok result. [Result - %s]", resultCode);
				}
				break;
			case ACTIVITY_RESULT_REQUEST_CODE_CC_ASSET_BROWSER:
				if (resultCode == Activity.RESULT_OK) {
					Bundle bundle = data.getExtras();
					List<File> downloadedFiles = (List<File>) bundle.getSerializable(BehanceSDKCCLauncherActivity.ACTIVITY_RESULT_CC_SUCCESSFULLY_DOWNLOADED_FILES);
					if (null != downloadedFiles && false == downloadedFiles.isEmpty()) {
						List<ImageModule> modules = new ArrayList<ImageModule>(downloadedFiles.size());
						for (File file : downloadedFiles) {
							ImageModule selectedImageModule = new CreativeCloudAssetModule(file);
							if(imageMatchesRequirements(selectedImageModule))
								modules.add(selectedImageModule);
						}
						notifyCallbacksOnImageSelection(modules);
					}
					closeThisDialog();
				}
				else if(resultCode == Activity.RESULT_CANCELED){
					if(getActivity() != null && getActivity() instanceof BehanceSDKCreateWIPWorkflowActivity)
					{
						notifyCallbacksOnBackPressed();
						closeThisDialog();
					}else{
						notifyCallbacksOnCancel();
						closeThisDialog();
					}
				}
				break;
			default:
				break;
		}

	}

	public Callbacks getCallbacks() {
		return callbacks;
	}

	public void setCallbacks(Callbacks callbacks) {
		this.callbacks = callbacks;
	}

	private void closeThisDialog() {
		dismissAllowingStateLoss();
	}

	@Override
	public boolean onKey(DialogInterface dialog, int keyCode, KeyEvent event) {
		if (keyCode == KeyEvent.KEYCODE_BACK && event.getAction() != KeyEvent.ACTION_DOWN) {
			handleBackBtnClick();
			return true;
		}
		return false;
	}

	private void showProgressSpinner() {
		if (progressSpinner != null) {
			progressSpinner.setVisibility(View.VISIBLE);
		}
	}

	private void hideProgressSpinner() {
		if (progressSpinner != null) {
			progressSpinner.setVisibility(View.GONE);
		}
	}

	@Override
	public void onGetAlbumsSuccess() {
		if (getActivity() != null) {
			setAlbumsGridAdapter();
			hideProgressSpinner();
		}
	}

	@Override
	public void onGetAlbumsFailure(Exception e) {
		if (getActivity() != null) {
			hideProgressSpinner();
			logger.error(e, "Problem loading albums");
			Toast.makeText(getActivity(), R.string.bsdk_image_selector_view_error_loading_albums_msg, Toast.LENGTH_LONG).show();
			notifyCallbacksOnError(e);
			closeThisDialog();
		}
	}

	@Override
	public void onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
		switch (requestCode) {
			case BehanceSDKPermissionHelper.REQUEST_CODE_LAUNCH_CAMERA : {
				if((grantResults.length == 2 && grantResults[0] == PackageManager.PERMISSION_GRANTED && grantResults[1] == PackageManager.PERMISSION_GRANTED) || (grantResults.length ==1 && grantResults[0] == PackageManager.PERMISSION_GRANTED)) {
					launchCameraToCapturePicture();
				} else {
					int messageId = R.string.behance_sdk_permissions_error_camera;
					//Permission denied! Show some dialog about how it will limits the functionality!
					if(grantResults.length ==2 ){
						if(grantResults[0] != PackageManager.PERMISSION_GRANTED && grantResults[1]!=PackageManager.PERMISSION_GRANTED){
							messageId = R.string.behance_sdk_permissions_error;
						}else if(grantResults[0]!= PackageManager.PERMISSION_GRANTED){
							messageId = R.string.behance_sdk_permissions_error_camera;
						}else{
							messageId = R.string.behance_sdk_permissions_error_generic;
						}
					}else if(grantResults.length ==1){
						messageId = R.string.behance_sdk_permissions_error_generic;
					}
					Toast.makeText(getActivity(), getActivity().getString(messageId), Toast.LENGTH_LONG).show();
				}
			}
			break;
		}
	}
}